@@ -6,10 +6,10 @@
    Copyright (c) 1993.  Don Maszle, Frederic Bois.  All rights reserved.
 
    -- Revisions -----
-     Logfile:  SCCS/s.modd.c
-    Revision:  1.4
-        Date:  23 Aug 1996
-     Modtime:  05:48:22
+     Logfile:  %F%
+    Revision:  %I%
+        Date:  %G%
+     Modtime:  %U%
       Author:  @a
    -- SCCS  ---------
 
@@ -40,20 +40,22 @@ PSTR vrgszMathFuncs[] = {
   "sinh", "sqrt", "tan",  "tanh",
   "CDFNormal", "erfc", "lnDFNormal", "lnGamma", "UniformRandom",
   "NormalRandom",
-  ""    /* End flag */
+  "" /* End flag */
 }; /* vrgszMathFuncs[] = */
 
 /* Global used by modd.c and modo.c as a flag */
-char vszHasInitializer[] = "0.0; /*-- Redefined later */";
+char vszHasInitializer[] = "0.0; /* Redefined later */";
       
 
-/* ----- Prototypes for this file  */
+/* Prototypes for this file */
 
 PSTR CopyString (PSTR);
 
-/*----- The functions ! */
 
+/* Functions */
 
+/* ----------------------------------------------------------------------------
+*/
 BOOL IsMathFunc (PSTR sz)
 {
   int i = 0;
@@ -63,63 +65,90 @@ BOOL IsMathFunc (PSTR sz)
     i++;
 
   return (*vrgszMathFuncs[i]);
+
 }  /* IsMathFunc */
 
 
+/* ----------------------------------------------------------------------------
+*/
 BOOL VerifyEqn (PINPUTBUF pibIn, PSTR szEqn)
 {
-  static char vszEqnPunct[] = "+-/*()><?:,";
   INPUTBUF ibDummy;
-  PSTRLEX szLex;
-  int iType, fContext;
-  BOOL bReturn = TRUE;
-  BOOL bOK = TRUE;
+  PSTRLEX  szLex;
+  int      iType, fContext;
+  BOOL     bReturn = TRUE;
+  BOOL     bOK = TRUE;
   PINPUTINFO pinfo;
   
   pinfo = (PINPUTINFO) pibIn->pInfo;
   
   MakeStringBuffer (pibIn, &ibDummy, szEqn);
   
-  while (!EOB(&ibDummy)) {              /* bOK not checked here... */
-    NextLex(&ibDummy, szLex, &iType);   /* ...all errors reported */
+  while (!EOB(&ibDummy)) { /* bOK not checked here... */
+
+    NextLex (&ibDummy, szLex, &iType); /* ...all errors reported */
+
     switch (iType) {
 
       case LX_IDENTIFIER:
         if ((iType = GetKeywordCode (szLex, &fContext))) {
           if (!(bOK = (iType == KM_DXDT && (fContext & pinfo->wContext))))
-            ReportError (pibIn, RE_BADCONTEXT, szLex, NULL);
-        }  /* GetKeywordCode */
-        else
+            ReportError (pibIn, RE_BADCONTEXT | RE_FATAL, szLex, NULL);
+        } /* if GetKeywordCode */
+        else {
+          /* an input function cannot be assigned to something else than an
+             input, so if szLex is recognized by GetFnType print an error
+             message */
+          if (GetFnType (szLex)) {
+            ReportError (pibIn, RE_BADCONTEXT | RE_FATAL, szLex, NULL); 
+          }
           /* Allowable identifiers are declared variables,
-             C functions, and the time variable (in Dynamics{} only )*/
+             C functions, and the time variable (in Dynamics{} only) */
           if (!(bOK = (GetVarType (pinfo->pvmGloVars, szLex)
-                       || IsMathFunc (szLex)
-                       || (pinfo->wContext == CN_DYNAMICS
-                           && !strcmp(szLex, VSZ_TIME)))))
-            ReportError (pibIn, RE_UNDEFINED, szLex, NULL);
+              || IsMathFunc (szLex)
+              || ((pinfo->wContext == CN_DYNAMICS || 
+                   pinfo->wContext == CN_CALCOUTPUTS)
+                  && !strcmp(szLex, VSZ_TIME)))))
+             ReportError (pibIn, RE_UNDEFINED | RE_FATAL, szLex, NULL);
+          }
+
         break;
-        
-        
-      case LX_PUNCT:
-        if (!(bOK = (int) strchr (vszEqnPunct, szLex[0])))
+ 
+      case LX_EQNPUNCT:
+        if ((szLex[0] == '!' || szLex[0] == '=') && strlen(szLex) == 1) {
           ReportError (pibIn, RE_UNEXPECTED, szLex, ".. in equation");
+          bOK = FALSE;
+        }
         break;
-        
+ 
+      case LX_PUNCT:
+        ReportError (pibIn, RE_UNEXPECTED, szLex, ".. in equation");
+        bOK = FALSE;
+        break;
+ 
       case LX_INTEGER:
       case LX_FLOAT:
-        break;  
+        break; 
+
+      default:
+        ReportError (pibIn, RE_UNEXPECTED, szLex, ".. in equation");
+        bOK = FALSE;
+        break;
         
-    }  /* switch */
+    } /* switch */
+    
     bReturn = (bReturn && bOK);
-  }  /* while */
+    
+  } /* while */
     
   return (bReturn);
 
-}  /* VerifyEqn */
+} /* VerifyEqn */
 
 
 
-/* AddEquation
+/* ----------------------------------------------------------------------------
+   AddEquation
    
    Adds an equation to the list given.  Note that this may be a
    variable table with initializer, or function defining equations.
@@ -132,38 +161,37 @@ BOOL VerifyEqn (PINPUTBUF pibIn, PSTR szEqn)
    items is not a big deal considering the overall time savings will
    be minimal.
 */
-
-void AddEquation (PVM *ppvm, PSTR szName, PSTR szEqn, HANDLE hType)
+void AddEquation (PVMMAPSTRCT *ppvm, PSTR szName, PSTR szEqn, HANDLE hType)
 {
-  PVM  pvmNew;
+  PVMMAPSTRCT  pvmNew;
   
   if (!ppvm || !szName)
     return;
   
-  if ((pvmNew = (PVM) malloc (sizeof(VM)))) {
+  if ((pvmNew = (PVMMAPSTRCT) malloc (sizeof(VMMAPSTRCT)))) {
     pvmNew->szName = CopyString (szName); 
     pvmNew->szEqn = CopyString (szEqn); 
     pvmNew->hType = hType;
     pvmNew->pvmNextVar = *ppvm;
     
     *ppvm = pvmNew; /* Redefine Head */
-  }  /* if */
+  } /* if */
   
   else
     ReportError (NULL, RE_OUTOFMEM | RE_FATAL, szName,
-        "* .. creating new equation in AddEquation");
+                 "* .. creating new equation in AddEquation");
   
-}  /* AddEquation */
+} /* AddEquation */
 
 
-/* CopyString
+/* ----------------------------------------------------------------------------
+   CopyString
    
    Creates a buffer large enough to hold the given equation, copies it
    into this buffer and returns a pointer to the buffer.
    
    Reports memory errors.
 */
-
 PSTR CopyString (PSTR szOrg)
 {
   PSTR szBuf;
@@ -174,35 +202,33 @@ PSTR CopyString (PSTR szOrg)
     else
       ReportError (NULL, RE_OUTOFMEM | RE_FATAL, szOrg,
                    "* .. defining equation in CopyString");
-  }  /* if */
+  } /* if */
 
-  return (NULL);    /* else */
-}  /* CopyString */
+  return (NULL); /* else */
+
+} /* CopyString */
 
 
-
-
-
-
-/* SetEquation
+/* ----------------------------------------------------------------------------
+   SetEquation
    
-   Sets the equation field of PVM given to the equation szEqn.
+   Sets the equation field of PVMMAPSTRCT given to the equation szEqn.
 */
-
-
-void SetEquation (PVM pvm, PSTR szEqn)
+void SetEquation (PVMMAPSTRCT pvm, PSTR szEqn)
 {
   if (!pvm || !szEqn)
     return;
 
   if (pvm->szEqn)
     free (pvm->szEqn);
+
   pvm->szEqn = CopyString (szEqn);
   
-}  /* SetEquation */
+} /* SetEquation */
 
 
-/* SetVarType
+/* ----------------------------------------------------------------------------
+   SetVarType
    
    Sets the iType field of the variable with szName.  If szName is
    not found, nothing is done without report of error.  This routine
@@ -210,8 +236,7 @@ void SetEquation (PVM pvm, PSTR szEqn)
    an initialization (to ID_PARM) before declaration of a model
    variable.
 */
-
-void  SetVarType (PVM pvm, PSTR szName, HANDLE hType)
+void SetVarType (PVMMAPSTRCT pvm, PSTR szName, HANDLE hType)
 {
   while (pvm && strcmp (szName, pvm->szName))
     pvm = pvm->pvmNextVar;
@@ -219,26 +244,27 @@ void  SetVarType (PVM pvm, PSTR szName, HANDLE hType)
   if (pvm)
     pvm->hType = hType;
 
-}  /* SetVarType */
+} /* SetVarType */
 
 
-/* GetVarPTR
+/* ----------------------------------------------------------------------------
+   GetVarPTR
    
-   Returns a pointer to the PVM structure of the variable specified
+   Returns a pointer to the PVMMAPSTRCT structure of the variable specified
    by szName, or NULL if it does not exist.
 */
-
-PVM GetVarPTR (PVM pvm, PSTR szName)
+PVMMAPSTRCT GetVarPTR (PVMMAPSTRCT pvm, PSTR szName)
 {
   while (pvm && strcmp (szName, pvm->szName))
     pvm = pvm->pvmNextVar;
   
   return (pvm);
 
-}  /* GetVarPTR */
+} /* GetVarPTR */
 
 
-/* GetVarType
+/* ----------------------------------------------------------------------------
+   GetVarType
    
    Returns the code of the szName given.  If the string does
    not reference a declared variable, returns ID_NULL.
@@ -246,17 +272,17 @@ PVM GetVarPTR (PVM pvm, PSTR szName)
    This function is also used as a test to prevent duplicate 
    declarations.
 */
-
-int GetVarType (PVM pvm, PSTR szName)
+int GetVarType (PVMMAPSTRCT pvm, PSTR szName)
 {
   pvm = GetVarPTR (pvm, szName);
   return (TYPE(pvm));
 
-}  /* GetVarType */
+} /* GetVarType */
 
 
 
-/* CalculateVarHandle
+/* ----------------------------------------------------------------------------
+   CalculateVarHandle
    
    Calculates what the handle created by the model generator will be.
    This is kind of a sloppy routine, but it is needed so that the
@@ -273,35 +299,34 @@ int GetVarType (PVM pvm, PSTR szName)
    the variables are known.
 
 */
-
-HANDLE CalculateVarHandle (PVM pvm, PSTR sz)
+HANDLE CalculateVarHandle (PVMMAPSTRCT pvm, PSTR sz)
 {
-  PVM pvmVar;
-  int cSameType = 0;    /* Count of same type of variable in map */
+  PVMMAPSTRCT pvmVar;
+  int cSameType = 0; /* Count of same type of variable in map */
 
-  pvm = pvmVar = GetVarPTR (pvm, sz);   /* Get PTR in map */
+  pvm = pvmVar = GetVarPTR (pvm, sz); /* Get PTR in map */
 
-  if (pvm)                              /* Don't count the variable */
+  if (pvm) /* Don't count the variable */
     pvm = pvm->pvmNextVar;
   
   while (pvm && pvm->hType == pvmVar->hType) {
-    cSameType++;                        /* Count vars define before it */
+    cSameType++; /* Count vars defined before it */
     pvm = pvm->pvmNextVar;
-  }  /* while */
+  } /* while */
   
   return ((HANDLE) (pvmVar ? pvmVar->hType | (HANDLE) cSameType : 0));
 
-}  /* CalculateVarHandle */
+} /* CalculateVarHandle */
 
 
 
-/* DefineGlobalVar
+/* ----------------------------------------------------------------------------
+   DefineGlobalVar
    
    Defines a global variable in the pvmGloVars list.
    Redefinitions are reported, and ignored.
 */
-
-void DefineGlobalVar (PINPUTBUF pibIn, PVM pvm,
+void DefineGlobalVar (PINPUTBUF pibIn, PVMMAPSTRCT pvm,
                       PSTR szName, PSTR szEqn, HANDLE hType)
 {
   PINPUTINFO pinfo = (PINPUTINFO) pibIn->pInfo;
@@ -315,41 +340,41 @@ void DefineGlobalVar (PINPUTBUF pibIn, PVM pvm,
   case ID_INPUT:
   case ID_OUTPUT:
   case ID_STATE:
-assert (pvm != NULL);
-    if (!pvm->szEqn) {          /*-- This is the first definition */
-
-      if (hType == ID_INPUT) {  /*-- Inputs use decl space for definition */
+    assert (pvm != NULL);
+    if (!pvm->szEqn) { /* This is the first definition */
+      if (hType == ID_INPUT) {  /* Inputs use decl space for definition */
         PIFN pifn = (PIFN) malloc (sizeof(IFN)); 
 
         if (GetInputFn (pibIn, szEqn, pifn))
           pvm->szEqn = (PSTR) pifn; /* THIS MAY CAUSE PROBS ON 68000 */
         else
-          pvm->szEqn = NULL;
-        
-      }  /* if */
+          pvm->szEqn = NULL;  
+      } /* if */
       else {
-        pvm->szEqn = vszHasInitializer; /*-- Flag this variable */
-                                /*-- Add a new entry at end of list so */
-                                /*   that dependencies are handled.    */
+        pvm->szEqn = vszHasInitializer; /* Flag this variable */
+        /* Add a new entry at end of list so
+           that dependencies are handled. */
         AddEquation (&pinfo->pvmGloVars, szName, szEqn, hType);
-      }  /* else */
+      } /* else */
       break;
-    }  /* if */
+    } /* if */
+
+    /* else  Redefinition -- Fall through ! */
 
-        /* else  Redefinition -- Fall through ! */
   case ID_PARM:
     ReportError (pibIn, RE_REDEF | RE_WARNING, szName, NULL);
-    break;      
+    break; 
 
   default:
     ReportError (pibIn, RE_BADCONTEXT, szName, NULL);
-  }  /* switch */
 
-}  /* DefineGlobalVar */
+  } /* switch */
 
-      
+} /* DefineGlobalVar */
 
-/* DefineDynamicsEqn
+
+/* ----------------------------------------------------------------------------
+   DefineDynamicsEqn
    
    Defines an equation in the pvmDynEqn list.  State variable
    assignments are meant as derivative assignments.
@@ -357,128 +382,123 @@ assert (pvm != NULL);
    Redefinitions are reported, and ignored.
 */
 
-void DefineDynamicsEqn(PINPUTBUF pibIn, PSTR szName, PSTR szEqn, HANDLE hType)
+void DefineDynamicsEqn (PINPUTBUF pibIn, PSTR szName, PSTR szEqn, HANDLE hType)
 {
   HANDLE hNewType = (hType ? hType : ID_LOCALDYN);
   PINPUTINFO pinfo = (PINPUTINFO) pibIn->pInfo;
-
-        /* If there was more than one line since last equation,
-           set ID_SPACEFLAG.  This is kind of a kludge so I can
-           use PVM struct. */
+ 
+  /* If there was more than one line since last equation, set ID_SPACEFLAG. 
+     This is kind of a kludge so I can use PVMMAPSTRCT struct. */
 
   if (pibIn->iLineNum - pibIn->iLNPrev - 1)
     hNewType |= ID_SPACEFLAG;
 
   switch (hType) {
       
-  case ID_NULL:
-    AddEquation (&pinfo->pvmGloVars, szName, NULL, hNewType);
+    case ID_NULL:
+      AddEquation (&pinfo->pvmGloVars, szName, NULL, hNewType);
+
       /* Fall through ! */
 
-  case ID_LOCALDYN:
-    AddEquation (&pinfo->pvmDynEqns, szName, szEqn, hNewType);
-    break;
-
-  case ID_DERIV:                        /*-- dt () assignment */
-  case ID_STATE:                        /*-- Non-standard direct state assgn */
-  case ID_OUTPUT:
-    AddEquation (&pinfo->pvmDynEqns, szName, szEqn, hNewType);
-    break;
-
-#ifdef ndef
-    if (!GetVarPTR (pinfo->pvmDynEqns, szName)) {
+    case ID_LOCALDYN:
       AddEquation (&pinfo->pvmDynEqns, szName, szEqn, hNewType);
       break;
-    }
-        /* else  Redefinition -- Fall through ! */
-#endif
-        
-  case ID_INPUT:
-  case ID_PARM:
-    ReportError (pibIn, RE_REDEF | RE_WARNING, szName, "  Inputs and parameters cannot be assigned in Dynamics\n");
-    break;      
-  }  /* switch */
+
+    case ID_DERIV: /* dt () assignment */
+    case ID_STATE: /* Non-standard direct state assgn */
+    case ID_OUTPUT:
+      AddEquation (&pinfo->pvmDynEqns, szName, szEqn, hNewType);
+      break;
+
+    case ID_INPUT:
+    case ID_PARM:
+      ReportError (pibIn, RE_REDEF | RE_WARNING, szName, 
+                   "  Inputs and parameters cannot be assigned in Dynamics\n");
+      break; 
+
+  } /* switch */
     
-  pibIn->iLNPrev = pibIn->iLineNum;     /* Update prev eqn line num */
-}  /* DefineDynamicsEqn */
+  pibIn->iLNPrev = pibIn->iLineNum; /* Update prev eqn line num */
+
+} /* DefineDynamicsEqn */
 
 
-/* DefineScaleEqn
+/* ----------------------------------------------------------------------------
+   DefineScaleEqn
    
    Defines an equation in the pvmScaleEqn list.  All model variables
    except inputs can be redefined here.
 */
-
 void DefineScaleEqn (PINPUTBUF pibIn, PSTR szName, PSTR szEqn, HANDLE hType)
 {
   HANDLE hNewType = (hType ? hType : ID_LOCALSCALE);
   PINPUTINFO pinfo = (PINPUTINFO) pibIn->pInfo;
 
-        /* If there was more than one line since last equation,
-           set ID_SPACEFLAG.  This is kind of a kludge so I can
-           use PVM struct. */
+  /* If there was more than one line since last equation, set ID_SPACEFLAG.  
+     This is kind of a kludge so I can use PVMMAPSTRCT struct. */
 
   if (pibIn->iLineNum - pibIn->iLNPrev - 1)
     hNewType |= ID_SPACEFLAG;
 
-  if (!hType)                           /* Original type is NULL */
+  if (!hType) /* Original type is NULL */
     AddEquation (&pinfo->pvmGloVars, szName, NULL, hNewType);
     
-  if ((hType & ID_LOCALSCALE)                           /* Many per local */
-      || !GetVarPTR (pinfo->pvmScaleEqns, szName))      /* 1 eqn per parm */
+  if ((hType & ID_LOCALSCALE) /* Many per local */
+      || !GetVarPTR (pinfo->pvmScaleEqns, szName)) /* 1 eqn per parm */
     AddEquation (&pinfo->pvmScaleEqns, szName, szEqn, hNewType);
     
   else
     ReportError (pibIn, RE_REDEF | RE_WARNING, szName, "* Ignoring");    
 
-  pibIn->iLNPrev = pibIn->iLineNum;     /* Update prev eqn line num */
+  pibIn->iLNPrev = pibIn->iLineNum; /* Update prev eqn line num */
 
-}  /* DefineScaleEqn */
+} /* DefineScaleEqn */
 
 
-/* DefineCalcOutEqn
+/* ----------------------------------------------------------------------------
+   DefineCalcOutEqn
    
    Defines an equation in the pvmCalcOutEqn list.  Only outputs
    can be redefined here.
 */
-
 void DefineCalcOutEqn (PINPUTBUF pibIn, PSTR szName, PSTR szEqn, HANDLE hType)
 {
   HANDLE hNewType = (hType ? hType : ID_LOCALCALCOUT);
   PINPUTINFO pinfo = (PINPUTINFO) pibIn->pInfo;
 
-        /* If there was more than one line since last equation,
-           set ID_SPACEFLAG.  This is kind of a kludge so I can
-           use PVM struct. */
+  /* If there was more than one line since last equation, set ID_SPACEFLAG.  
+     This is kind of a kludge so I can use PVMMAPSTRCT struct. */
 
   if (pibIn->iLineNum - pibIn->iLNPrev - 1)
     hNewType |= ID_SPACEFLAG;
 
-  if (!hType)                           /* Original type is NULL */
+  if (!hType) /* Original type is NULL */
     AddEquation (&pinfo->pvmGloVars, szName, NULL, hNewType);
     
-  if ((hType & ID_LOCALCALCOUT)                         /* Many per local */
-      || !GetVarPTR (pinfo->pvmCalcOutEqns, szName))    /* 1 eqn per parm */
+  if ((hType & ID_LOCALCALCOUT) /* Many per local */
+      || !GetVarPTR (pinfo->pvmCalcOutEqns, szName)) /* 1 eqn per parm */
     AddEquation (&pinfo->pvmCalcOutEqns, szName, szEqn, hNewType);
     
   else
     ReportError (pibIn, RE_REDEF | RE_WARNING, szName, "* Ignoring");    
 
-  pibIn->iLNPrev = pibIn->iLineNum;     /* Update prev eqn line num */
+  pibIn->iLNPrev = pibIn->iLineNum; /* Update prev eqn line num */
 
-}  /* DefineCalcOutEqn */
+} /* DefineCalcOutEqn */
 
 
+/* ----------------------------------------------------------------------------
+*/
 void DefineJacobEqn (PINPUTBUF pibIn, PSTR szName, PSTR szEqn, HANDLE hType)
 {
   ReportError (pibIn, RE_WARNING, NULL,
-      "Jacobian equation handler not implemented");
+               "Jacobian equation handler not implemented");
 
-}  /* DefineJacobEqn */
+} /* DefineJacobEqn */
 
 
-
-/* DefineVariable
+/* ----------------------------------------------------------------------------
+   DefineVariable
    
    Define the variable szLex according to szEqn if the variable
    type is valid for the assignment in the given fContext.
@@ -494,16 +514,15 @@ void DefineJacobEqn (PINPUTBUF pibIn, PSTR szName, PSTR szEqn, HANDLE hType)
    * In the global parameter declarations, a duplicate definition issues
      a warning and ignores the redefinitions
 */
-
 void DefineVariable (PINPUTBUF pibIn, PSTR szName, PSTR szEqn, int iKWCode)
 {
-  PVM pvm;
+  PVMMAPSTRCT pvm;
   HANDLE hGloVarType;
   PINPUTINFO pinfo;
   
   pinfo = (PINPUTINFO) pibIn->pInfo;
 
-assert (pinfo->wContext != CN_END);
+  assert (pinfo->wContext != CN_END);
 
   if (!szName || !szEqn)
     return;
@@ -513,7 +532,7 @@ assert (pinfo->wContext != CN_END);
 
   if ((hGloVarType != ID_INPUT || pinfo->wContext != CN_GLOBAL)
       && !VerifyEqn (pibIn, szEqn))
-    return;                     /*-- Errors reported in Verify eqn */
+    return; /* Errors reported in Verify eqn */
 
   switch (pinfo->wContext) {
     case CN_GLOBAL:
@@ -528,17 +547,10 @@ assert (pinfo->wContext != CN_END);
 
       if (hGloVarType == ID_STATE && iKWCode != KM_DXDT)
         ReportError (pibIn, RE_REDEF | RE_WARNING, szName,
-                     "Non-standard assignment in Dynamics section. Potential state discontinuity.\n");
+                     "Non-standard assignment in Dynamics section. "
+                     "Potential state discontinuity.\n");
       break;
 
-#ifdef ndef
-      if (hGloVarType != ID_STATE || iKWCode == KM_DXDT)
-        DefineDynamicsEqn (pibIn, szName, szEqn, hGloVarType);
-      else
-        ReportError (pibIn, RE_BADCONTEXT, szName, NULL);
-      break;
-#endif
-
     case CN_JACOB:
       DefineJacobEqn (pibIn, szName, szEqn, hGloVarType);
       break;
@@ -557,13 +569,14 @@ assert (pinfo->wContext != CN_END);
 
     default:
       break;
-  }  /* switch */
 
-}  /* DefineVariable */
+  } /* switch */
+
+} /* DefineVariable */
 
 
-
-/* DeclareModelVar
+/* ----------------------------------------------------------------------------
+   DeclareModelVar
    
    Declares szName to be a model variable of the type indicated
    by iKWCode.  Since the declaration commands States, Inputs, and
@@ -575,7 +588,6 @@ assert (pinfo->wContext != CN_END);
    The number of States, Outputs, or Inputs is incremented, and 
    nominal value and defining equations strings are initialized to NULL.
 */
-
 void DeclareModelVar (PINPUTBUF pibIn, PSTR szName, int iKWCode)
 {
   HANDLE hType;
@@ -583,25 +595,26 @@ void DeclareModelVar (PINPUTBUF pibIn, PSTR szName, int iKWCode)
   
   pinfo = (PINPUTINFO) pibIn->pInfo;
 
-assert(iKWCode == KM_STATES || iKWCode == KM_INPUTS || iKWCode == KM_OUTPUTS);
+  assert(iKWCode == KM_STATES || iKWCode == KM_INPUTS || iKWCode == KM_OUTPUTS);
 
-  iKWCode = ((iKWCode == KM_STATES    ? ID_STATE        /* Translate to ID_ */
+  iKWCode = ((iKWCode == KM_STATES    ? ID_STATE /* Translate to ID_ */
               : (iKWCode == KM_INPUTS ? ID_INPUT
                                       : ID_OUTPUT)));
 
-  if (!(hType = GetVarType (pinfo->pvmGloVars, szName)))    /* New id */
+  if (!(hType = GetVarType (pinfo->pvmGloVars, szName))) /* New id */
     AddEquation (&pinfo->pvmGloVars, szName, NULL, (HANDLE) iKWCode);
 
-  else if (hType == iKWCode)                            /* Same type redecl */
-     ReportError (pibIn, RE_DUPDECL | RE_WARNING, szName, NULL);
+  else if (hType == iKWCode) /* Same type redecl */
+    ReportError (pibIn, RE_DUPDECL | RE_WARNING, szName, NULL);
 
-  else if (hType == ID_PARM) {                          /* Already init'd */
-     ReportError (pibIn, RE_DUPDECL | RE_WARNING, szName,
-                  "* Model variable initialized before declaration");
-     SetVarType (pinfo->pvmGloVars, szName, iKWCode);
+  else 
+    if (hType == ID_PARM) { /* Already init'd */
+      ReportError (pibIn, RE_DUPDECL | RE_WARNING, szName,
+                   "Model variable initialized before declaration");
+      SetVarType (pinfo->pvmGloVars, szName, iKWCode);
      
-  }  /* if */
-  else                                          /* Unresolvable conflict */
-     ReportError (pibIn, RE_DUPDECL | RE_FATAL, szName, NULL);
+    } /* if */
+    else /* Unresolvable conflict */
+      ReportError (pibIn, RE_DUPDECL | RE_FATAL, szName, NULL);
   
-}  /* DeclareModelVar */
+} /* DeclareModelVar */
