@@ -6,10 +6,10 @@
    Copyright (c) 1993.  Don Maszle, Frederic Bois.  All rights reserved.
 
    -- Revisions -----
-     Logfile:  SCCS/s.modelu.h
-    Revision:  1.1
-        Date:  14 Jul 1993
-     Modtime:  19:11:17
+     Logfile:  %F%
+    Revision:  %I%
+        Date:  %G%
+     Modtime:  %U%
       Author:  @a
    -- SCCS  ---------
 
@@ -30,14 +30,14 @@
 
 /* ----- Constants  */
 
-#define ID_NULL         0x0000
-#define ID_STATE        0x1000
-#define ID_INPUT        0x2000
-#define ID_OUTPUT       0x3000
-#define ID_PARM         0x4000  /* Global parameter */
+#define ID_NULL		0x0000
+#define ID_STATE	0x1000
+#define ID_INPUT	0x2000
+#define ID_OUTPUT	0x3000
+#define ID_PARM		0x4000	/* Global parameter */
 
-#define HV_TYPE_MASK    0xF000        /* Handle to variable is a WORD */
-#define HV_INDEX_MASK   0x0FFF        /* == 0xtiii, type and index */
+#define HV_TYPE_MASK	0xF000	      /* Handle to variable is a WORD */
+#define HV_INDEX_MASK	0x0FFF	      /* == 0xtiii, type and index */
 
 
 /* ----- Typedefs  */
@@ -46,7 +46,7 @@
 
 typedef struct tagVM {
     PSTR szName;     /* Name of the variable */
-    PVOID pVar;      /* Ptr to C variable */
+    PVOID pVar;	     /* Ptr to C variable */
     HVAR hvar;       /* Handle to variable: ID_TYPE | index */
 
 } VM, *PVM; /* Variable Map element */
@@ -54,11 +54,11 @@ typedef struct tagVM {
 
 /* ----- Macros  */
 
-#define TYPE(pvm)       ((pvm) ? (pvm)->hvar & HV_TYPE_MASK : ID_NULL)
-#define INDEX(pvm)      ((pvm) ? (pvm)->hvar & HV_INDEX_MASK: ID_NULL)
+#define TYPE(pvm)	((pvm) ? (pvm)->hvar & HV_TYPE_MASK : ID_NULL)
+#define INDEX(pvm)	((pvm) ? (pvm)->hvar & HV_INDEX_MASK: ID_NULL)
 
-#define HTYPE(hvar)     ((hvar) & HV_TYPE_MASK)
-#define HINDEX(hvar)    ((int) ((hvar) & HV_INDEX_MASK))
+#define HTYPE(hvar)	((hvar) & HV_TYPE_MASK)
+#define HINDEX(hvar)	((int) ((hvar) & HV_INDEX_MASK))
 
 /* ----- Globals/Externals  */
 
