@@ -1,4 +1,5 @@
-/* modo.c
+/* ----------------------------------------------------------------------------
+   modo.c
 
    written by Don Maszle
    21 September 1991
@@ -6,10 +7,10 @@
    Copyright (c) 1993.  Don Maszle, Frederic Bois.  All rights reserved.
 
    -- Revisions -----
-     Logfile:  SCCS/s.modo.c
-    Revision:  1.10
-        Date:  23 Aug 1996
-     Modtime:  05:48:22
+     Logfile:  %F%
+    Revision:  %I%
+        Date:  %G%
+     Modtime:  %U%
       Author:  @a
    -- SCCS  ---------
 
@@ -23,7 +24,6 @@
 #include <assert.h>
 #include <time.h>
 
-
 #include "mod.h"
 #include "lexfn.h"
 #include "lexerr.h"
@@ -39,7 +39,7 @@
 
 #define WriteIndexName(pfile, pvm)  (fprintf ((pfile), "ID_%s", (pvm)->szName))
 
-typedef int (*PFI_CALLBACK) (PFILE, PVM, PVOID);
+typedef int (*PFI_CALLBACK) (PFILE, PVMMAPSTRCT, PVOID);
 
 static char *vszModelFilename = NULL;
 static char *vszModGenName = NULL;
@@ -48,101 +48,103 @@ static char vszModelArrayName[] = "vrgModelVars";
 static char vszInputArrayName[] = "vrgInputs";
 extern char vszHasInitializer[]; /* decl'd in modd.c */
 
-PVM vpvmGloVarList;
+PVMMAPSTRCT vpvmGloVarList;
 
-char *vszIFNTypes[] = {         /* Must match defines in lexfn.h */
+char *vszIFNTypes[] = { /* Must match defines in lexfn.h */
   "IFN_NULL /* ?? */",
   "IFN_CONSTANT",
   "IFN_PERDOSE",
   "IFN_PERRATE",
   "IFN_PEREXP",
   "IFN_NDOSES"
-};  /* vszIFNTypes[] = */
-
+}; /* vszIFNTypes[] = */
 
 
 /* Global Variables */
 
 int vnStates, vnOutputs, vnInputs, vnParms, vnModelVars;
 
-/* ForAllVar
+/* ----------------------------------------------------------------------------
+   ForAllVar
    
    Takes a pfile, a pvm list head and a callback function which is called
    for all variables in the list if hType == ALL_VARS or only for
    only those that match hType otherwise.
 */
-
-int ForAllVar (PFILE pfile, PVM pvm, PFI_CALLBACK pfiFunc,
-   HANDLE hType, PVOID pinfo)
+int ForAllVar (PFILE pfile, PVMMAPSTRCT pvm, PFI_CALLBACK pfiFunc,
+               HANDLE hType, PVOID pinfo)
 {
   int iTotal = 0;
   
   while (pvm) {
-    if (hType == ALL_VARS           /* Do for all ...*/
-        || TYPE(pvm) == hType)      /* ..or do only for vars of hType */
+    if (hType == ALL_VARS          /* Do for all ...*/
+        || TYPE(pvm) == hType)     /* ..or do only for vars of hType */
       if (pfiFunc)
         iTotal += (*pfiFunc) (pfile, pvm, pinfo);
       else
-        iTotal++;                   /* No func! just count */
+        iTotal++; /* No func! just count */
 
     pvm = pvm->pvmNextVar;
-  }  /* while */
+  } /* while */
 
   return (iTotal);
-}  /* ForAllVar */
+
+} /* ForAllVar */
 
 
-/* CountOneDecl
+/* ----------------------------------------------------------------------------
+   CountOneDecl
    
    Counts declarations for variables.  Note that each variable may only
    be declared and assigned *once*.  This is a problem and should be fixed.
 
    Callback for ForAllVar().
 */
-
-int CountOneDecl (PFILE pfile, PVM pvm, PVOID pInfo)
+int CountOneDecl (PFILE pfile, PVMMAPSTRCT pvm, PVOID pInfo)
 {
-  if (pvm->szEqn != vszHasInitializer) { /*-- These appear later */
+  if (pvm->szEqn != vszHasInitializer) { /* These appear later */
     return (1);
-  }  /* if */
+  }
   return (0);
-}  /* CountOneDecl */
+
+} /* CountOneDecl */
 
 
-/* WriteOneName
+/* ----------------------------------------------------------------------------
+   WriteOneName
    
    Writes a name and nominal value for the given variable of pvm.
 
    Callback for ForAllVar().
 */
-
-int WriteOneName (PFILE pfile, PVM pvm, PVOID pInfo)
+int WriteOneName (PFILE pfile, PVMMAPSTRCT pvm, PVOID pInfo)
 {
-  if (pvm->szEqn != vszHasInitializer) { /*-- These appear later */
+  if (pvm->szEqn != vszHasInitializer) { /* These appear later */
     fprintf (pfile, "      %s", pvm->szName);
     if (TYPE(pvm) != ID_INPUT)
-      fprintf (pfile, " \t%s %s\n",
-   (pvm->szEqn ? ":=" : "->"),
-   (pvm->szEqn ? pvm->szEqn : "0.0"));
+      fprintf (pfile, " \t%s %s\n", (pvm->szEqn ? ":=" : "->"),
+               (pvm->szEqn ? pvm->szEqn : "0.0"));
     else
       fprintf (pfile, " is an input function\n");
+
     return (1);
-  }  /* if */
+  } /* if */
+
   return (0);
+
 } /* WriteOneName */
 
-        
-/* WriteHeader
+ 
+/* ----------------------------------------------------------------------------
+   WriteHeader
    
    That it does do dashingly.  Includes a nice comment at the
    beginning (hopefully!) of the file replete with filename,
    timestamp, and a list of model variables. 
 */
-
-
-void WriteHeader (PFILE pfile, PSTR szName, PVM pvmGlo)
+void WriteHeader (PFILE pfile, PSTR szName, PVMMAPSTRCT pvmGlo)
 {
-  struct tm *ptm;       /* Local time struct */
+  struct tm *ptm; /* Local time struct */
   time_t ttTime;
   
   time (&ttTime);
@@ -152,7 +154,7 @@ void WriteHeader (PFILE pfile, PSTR szName, PVM pvmGlo)
   fprintf(pfile, "   ___________________________________________________\n\n");
   fprintf (pfile, "   Model File:  %s\n\n", vszModelFilename);
   fprintf (pfile, "   Date:  %s\n", ctime(&ttTime));
-  fprintf (pfile, "   Created by:  \"%s\" %s\n", vszModGenName, VSZ_VERSION);
+  fprintf (pfile, "   Created by:  \"%s-%s\"\n", vszModGenName, VSZ_VERSION);
   fprintf (pfile, "    -- a model preprocessor by Don Maszle\n");
   fprintf(pfile, "   ___________________________________________________\n\n");
 
@@ -174,15 +176,18 @@ void WriteHeader (PFILE pfile, PSTR szName, PVM pvmGlo)
   
   fprintf (pfile, "*/\n\n");
   
-}  /* WriteHeader */
+} /* WriteHeader */
 
 
+/* ----------------------------------------------------------------------------
+*/
 void WriteIncludes (PFILE pfile)
 {
   fprintf (pfile, "\n#include <stdlib.h>\n");
   fprintf (pfile, "#include <stdio.h>\n");
   fprintf (pfile, "#include <math.h>\n");
-  fprintf (pfile, "#include <string.h>\n\n");
+  fprintf (pfile, "#include <string.h>\n");
+  fprintf (pfile, "#include <float.h>\n");
   
   fprintf (pfile, "#include \"modelu.h\"\n");
   fprintf (pfile, "#include \"random.h\"\n");
@@ -190,12 +195,12 @@ void WriteIncludes (PFILE pfile)
 } /* WriteIncludes */
 
 
-/* WriteOneDecl
+/* ----------------------------------------------------------------------------
+   WriteOneDecl
   
    Write one global or local declaration.  Callback for ForAllVar().
 */
-
-int WriteOneDecl (PFILE pfile, PVM pvm, PVOID pInfo)
+int WriteOneDecl (PFILE pfile, PVMMAPSTRCT pvm, PVOID pInfo)
 {
   assert(TYPE(pvm) != ID_INPUT);
   assert(TYPE(pvm) != ID_OUTPUT);
@@ -206,25 +211,33 @@ int WriteOneDecl (PFILE pfile, PVM pvm, PVOID pInfo)
 
   fprintf (pfile, "double %s;\n", pvm->szName);
   return (1);
-}  /* WriteOneDecl */
+
+} /* WriteOneDecl */
 
 
-int WriteOneIndexDefine (PFILE pfile, PVM pvm, PVOID pInfo)
+/* ----------------------------------------------------------------------------
+*/
+int WriteOneIndexDefine (PFILE pfile, PVMMAPSTRCT pvm, PVOID pInfo)
 {
-  if (pvm->szEqn != vszHasInitializer) { /*-- These appear later */
+  if (pvm->szEqn != vszHasInitializer) { /* These appear later */
     fprintf (pfile, "#define ");
     WriteIndexName(pfile, pvm);
     if (INDEX(pvm))
       fprintf (pfile, "\t0x%04x\n", INDEX(pvm));
     else
       fprintf (pfile, "\t0x0000\n");
+
     return 1;
   }  /* if */
+
   return 0;
-}  /* WriteOneIndexDefine */
+
+} /* WriteOneIndexDefine */
 
 
-void WriteDecls (PFILE pfile, PVM pvmGlo)
+/* ----------------------------------------------------------------------------
+*/
+void WriteDecls (PFILE pfile, PVMMAPSTRCT pvmGlo)
 {
   fprintf (pfile, "\n/*----- Indices to Global Variables */\n");
 
@@ -243,7 +256,7 @@ void WriteDecls (PFILE pfile, PVM pvmGlo)
   fprintf (pfile, "char szModelDescFilename[] = \"%s\";\n", vszModelFilename);
   fprintf (pfile, "char szModelSourceFilename[] = __FILE__;\n");
   fprintf (pfile, "char szModelGenAndVersion[] = \"%s-%s\";\n",
-           vszModGenName, VSZ_VERSION);
+    vszModGenName, VSZ_VERSION);
 
   fprintf (pfile, "\n  /* Externs */\n");
   fprintf (pfile, "extern BOOL vbModelReinitd;\n");
@@ -259,15 +272,18 @@ void WriteDecls (PFILE pfile, PVM pvmGlo)
   fprintf (pfile, "double %s[%d];\n", vszModelArrayName, vnModelVars);
 
   fprintf (pfile, "\n  /* Inputs */\n");
-  fprintf (pfile, "IFN %s[%d];\n", vszInputArrayName, vnInputs);
+  /* if vnInputs is zero put a dummy 1 for array size */
+  fprintf (pfile, "IFN %s[%d];\n", vszInputArrayName, 
+           (vnInputs > 0 ? vnInputs : 1));
 
-  fprintf (pfile, "\n/* Parameters */\n\n");
+  fprintf (pfile, "\n/* Parameters */\n");
   ForAllVar (pfile, pvmGlo, &WriteOneDecl, ID_PARM, NULL);
 
-}  /* WriteDecls */
+} /* WriteDecls */
 
 
-/* GetName 
+/* ----------------------------------------------------------------------------
+   GetName 
    
    returns a string to the name of the pvm variable given.  The
    string is static and must be used immediately or copied.  It will
@@ -279,8 +295,7 @@ void WriteDecls (PFILE pfile, PVM pvmGlo)
    The name is determined by hType if hType is non-NULL.  If hType is
    NULL, then the type is taken to be the hType field of pvm.
 */
-   
-PSTR GetName (PVM pvm, PSTR szModelVarName, PSTR szDerivName, HANDLE hType)
+PSTR GetName (PVMMAPSTRCT pvm, PSTR szModelVarName, PSTR szDerivName, HANDLE hType)
 {
 static PSTRLEX vszVarName;
   HANDLE hTypeToUse = (hType ? hType : TYPE(pvm));
@@ -304,56 +319,60 @@ static PSTRLEX vszVarName;
     sprintf (vszVarName, "%s[ID_%s]", szDerivName, pvm->szName);
     break;
 
-  default:                              /* Parms and local variables */
+  default:    /* Parms and local variables */
     sprintf (vszVarName, "%s", pvm->szName);
     break;
   }  /* switch */
 
   return (vszVarName);
-}  /* GetName */
+
+} /* GetName */
 
 
-
-/* WriteOneVMEntry
+/* ----------------------------------------------------------------------------
+   WriteOneVMEntry
 
    prints a single entry for the variable map symbol table.
 
    Callback for ForAllList.
 */
-
-int WriteOneVMEntry (PFILE pfile, PVM pvm, PVOID pInfo)
+int WriteOneVMEntry (PFILE pfile, PVMMAPSTRCT pvm, PVOID pInfo)
 {
   int iType = TYPE(pvm);
 
   if (!pvm) {
-    fprintf (pfile, "    {\"\",\tNULL,\t0}      /* End flag */\n");
+    fprintf (pfile, "    {\"\",\tNULL,\t0} /* End flag */\n");
     return 0;
-  }  /* if */
+  }
     
   assert(iType && \
     iType != ID_LOCALDYN && iType != ID_LOCALSCALE && iType != ID_LOCALJACOB);
 
-  if (pvm->szEqn != vszHasInitializer) { /*-- These appear later */
+  if (pvm->szEqn != vszHasInitializer) { /* These appear later */
     fprintf (pfile, "    {\"%s\",\t(PVOID) &%s", pvm->szName,
-       GetName(pvm, vszModelArrayName, NULL, ID_NULL));
+             GetName(pvm, vszModelArrayName, NULL, ID_NULL));
     
     fprintf (pfile, ", \tID_%s | ID_%s},\n", 
-       (iType == ID_PARM ? "PARM"
-  : (iType == ID_INPUT ? "INPUT"
-     : (iType == ID_OUTPUT ? "OUTPUT"
-  : "STATE"))),
-       pvm->szName);
+             (iType == ID_PARM ? "PARM"
+              : (iType == ID_INPUT ? "INPUT"
+              : (iType == ID_OUTPUT ? "OUTPUT"
+              : "STATE"))), pvm->szName);
+
     return (1);
+
   }  /* if */
+
   return 0;
-}  /* WriteOneVMEntry */
+
+} /* WriteOneVMEntry */
 
 
-
-void WriteVarMap (PFILE pfile, PVM pvmGlo)
+/* ----------------------------------------------------------------------------
+*/
+void WriteVarMap (PFILE pfile, PVMMAPSTRCT pvmGlo)
 {
-  fprintf (pfile, "\n\n/*----- Global Variable Map */\n\n");
-  fprintf (pfile, "VM vrgvmGlo[] = {\n");
+  fprintf (pfile, "\n\n/*----- Global Variable Map */\n");
+  fprintf (pfile, "VMMAPSTRCT vrgvmGlo[] = {\n");
   ForAllVar (pfile, pvmGlo, &WriteOneVMEntry, ID_STATE, NULL);  
   fprintf (pfile, "\n");
 
@@ -364,28 +383,30 @@ void WriteVarMap (PFILE pfile, PVM pvmGlo)
   fprintf (pfile, "\n");
 
   ForAllVar (pfile, pvmGlo, &WriteOneVMEntry, ID_PARM, NULL);
-  WriteOneVMEntry (pfile, NULL, NULL);  /* Include end flag */
-  fprintf (pfile, "};  /* vrgpvmGlo[] */\n\n");
+  WriteOneVMEntry (pfile, NULL, NULL); /* Include end flag */
+  fprintf (pfile, "};  /* vrgpvmGlo[] */\n");
 
 }  /* WriteVarMap */
 
 
-int WriteOneInit (PFILE pfile, PVM pvm, PVOID pInfo)
+/* ----------------------------------------------------------------------------
+*/
+int WriteOneInit (PFILE pfile, PVMMAPSTRCT pvm, PVOID pInfo)
 {
-  IFN   ifnNull = {IFN_CONSTANT};       /* Init other fields to zero */
-  PSTR  szVarName = GetName (pvm, NULL, NULL, ID_NULL);
+  IFN ifnNull = {IFN_CONSTANT}; /* Init other fields to zero */
+  PSTR szVarName = GetName (pvm, NULL, NULL, ID_NULL);
 
-  if (pvm->szEqn != vszHasInitializer /*-- These appear later */
-      && TYPE(pvm) <= ID_PARM) {       /*-- No Locals!  */
+  if (pvm->szEqn != vszHasInitializer /* These appear later */
+      && TYPE(pvm) <= ID_PARM) { /* No Locals!  */
     
     if (TYPE(pvm) == ID_INPUT) {
       PIFN pifn = (PIFN) pvm->szEqn;
       
-      if (!pifn)                                /* No eqn, init to zero */
-  pifn = &ifnNull;
+      if (!pifn) /* No eqn, init to zero */
+        pifn = &ifnNull;
       
       fprintf (pfile, "\n  %s.iType = %s;\n",
-   szVarName, vszIFNTypes[pifn->iType]);
+               szVarName, vszIFNTypes[pifn->iType]);
       fprintf (pfile, "  %s.dTStartPeriod = 0.0;\n", szVarName);
       fprintf (pfile, "  %s.bOn  = FALSE;\n", szVarName);
       fprintf (pfile, "  %s.dMag = %f;\n", szVarName, pifn->dMag);
@@ -402,41 +423,69 @@ int WriteOneInit (PFILE pfile, PVM pvm, PVOID pInfo)
       
       fprintf (pfile, "  %s.dVal  = 0.0;\n", szVarName);
       fprintf (pfile, "  %s.nDoses = 0;\n", szVarName);
-    }  /* if */
+    } /* if */
     else 
-      fprintf (pfile, "  %s = %s;\n",
-   szVarName,
-   (pvm->szEqn ? pvm->szEqn : "0.0"));
+      fprintf (pfile, "  %s = %s;\n", szVarName,
+               (pvm->szEqn ? pvm->szEqn : "0.0"));
+
     return (1);
-  }  /* if */
+  } /* if */
+
   return (0);
-}  /* WriteOneInit */
+
+} /* WriteOneInit */
 
 
-
-/* WriteInitModel
+/* ----------------------------------------------------------------------------
+   WriteInitModel
    
    Writes the routine to initialize the model variables.
 */
-
-void WriteInitModel (PFILE pfile, PVM pvmGlo)
+void WriteInitModel (PFILE pfile, PVMMAPSTRCT pvmGlo)
 {
   fprintf (pfile, "\n/* InitModel\n\n");
   fprintf (pfile, "   Should be called to initialize model variables at\n");
   fprintf (pfile, "   the beginning of experiment before reading\n");
   fprintf (pfile, "   variants from the simulation spec file.\n*/\n\n");
-  fprintf (pfile, "void InitModel ()\n{\n");
+  fprintf (pfile, "void InitModel()\n{\n");
   fprintf (pfile, "/*-- Initialize things in the order that they appear in\n"
       "     model definition file so that dependencies are\n"
       "     handled correctly. */\n\n");
   ForAllVar (pfile, pvmGlo, &WriteOneInit, ALL_VARS, NULL);
   fprintf (pfile, "\n  vbModelReinitd = TRUE;  /* Flag is initialized */\n");
   fprintf (pfile, "} /* InitModel */\n\n");
+
 } /* WriteInitModel */
 
 
+#ifdef ndef
+/* ----------------------------------------------------------------------------
+   WriteInitExpt
+   
+   Writes the routine to initialize certain model variables.
+*/
 
-/* TranslateID
+void WriteInitExpt (PFILE pfile, PVMMAPSTRCT pvmGlo)
+{
+  fprintf (pfile, "\n/* InitExpt\n\n");
+  fprintf (pfile, "   Should be called to initialize model variables at\n");
+  fprintf (pfile, "   the beginning of experiment after reading\n");
+  fprintf (pfile, "   variants from the simulation spec file.\n*/\n\n");
+  fprintf (pfile, "void InitExpt()\n{\n");
+  fprintf (pfile, "/*-- Initialize things in the order that they appear in\n"
+      "     model definition file so that dependencies are\n"
+      "     handled correctly. */\n\n");
+  ForAllVar (pfile, pvmGlo, &WriteOneInit, ID_STATE, NULL);
+  ForAllVar (pfile, pvmGlo, &WriteOneInit, ID_INPUT, NULL);
+  ForAllVar (pfile, pvmGlo, &WriteOneInit, ID_OUTPUT, NULL);
+  fprintf (pfile, "\n  vbModelReinitd = TRUE;  /* Flag is initialized */\n");
+  fprintf (pfile, "} /* InitExpt */\n\n");
+} /* WriteInitExpt */
+#endif
+
+
+/* ----------------------------------------------------------------------------
+   TranslateID
    
    Writes an equation id if it declared and in a valid context.
    Errors are reported if necessary.
@@ -451,7 +500,7 @@ void TranslateID (PINPUTBUF pibDum, PFILE pfile, PSTR szLex, int iEqType)
 
   case KM_DXDT: {
     int iArg = LX_IDENTIFIER;
-    PVM pvm = NULL;
+    PVMMAPSTRCT pvm = NULL;
     
     if (GetFuncArgs (pibDum, 1, &iArg, szLex)
         && (pvm = GetVarPTR(vpvmGloVarList, szLex))
@@ -459,48 +508,49 @@ void TranslateID (PINPUTBUF pibDum, PFILE pfile, PSTR szLex, int iEqType)
       fprintf (pfile, "%s", GetName (pvm, NULL, "rgDerivs", ID_DERIV));
     else
       ReportError (pibDum, RE_BADSTATE | RE_FATAL, (pvm? szLex : NULL), NULL);
-  }  /* KM_DXDT: */
-  break;
+    } /* KM_DXDT: */
+    break;
 
   case KM_NULL: {
-    PVM pvm = GetVarPTR(vpvmGloVarList, szLex);
+    PVMMAPSTRCT pvm = GetVarPTR(vpvmGloVarList, szLex);
 
-    /*-- Handle undeclared ids */
+    /* Handle undeclared ids */
     if (!pvm) {
-      if (iEqType == KM_DYNAMICS && !strcmp (szLex, VSZ_TIME))
-        /*-- If this is the time variable, convert to the correct formal arg */
+      if ((iEqType == KM_DYNAMICS || iEqType == KM_CALCOUTPUTS) && 
+          !strcmp (szLex, VSZ_TIME))
+        /* If this is the time variable, convert to the correct formal arg */
         fprintf(pfile, "(*pdTime)");
       else
-        /*-- otherwise output id exactly as is */
+        /* otherwise output id exactly as is */
         fprintf(pfile, "%s", szLex);
-    }  /* if */
-        
+    } /* if */
+ 
     else {
       fprintf(pfile, "%s", GetName(pvm, "rgModelVars", NULL, ID_NULL));
       if (TYPE(pvm) == ID_INPUT) {
         if (iEqType == KM_SCALE)
-          fprintf(pfile, ".dMag");      /* Can only scale with magnitude */
+          fprintf(pfile, ".dMag"); /* Can only scale with magnitude */
         else
-          fprintf(pfile, ".dVal");      /* Otherwise use current value */
-      }  /* if */
-    }  /* else */
+          fprintf(pfile, ".dVal"); /* Otherwise use current value */
+      } /* if */
+    } /* else */
 
-  }  /* KM_NULL: */
-  break;
+    } /* KM_NULL: */
+    break;
 
-  default:              /* No other keywords allowed in equations */
+  default: /* No other keywords allowed in equations */
 
     /* Allow for C keywords here, including math library functions */
-
     ReportError (pibDum, RE_BADCONTEXT | RE_FATAL, szLex, NULL);
     break;
-  }  /* switch */
+
+  } /* switch */
   
-}  /* TranslateID */
+} /* TranslateID */
 
 
-
-/* TranslateEquation
+/* ----------------------------------------------------------------------------
+   TranslateEquation
    
    Writes an equation to the output file substituting model variable
    names, inputs names, derivative names, etc.
@@ -523,154 +573,179 @@ void TranslateEquation (PFILE pfile, PSTR szEqn, int iEqType)
   if (!iType) {
     fprintf (pfile, "0.0;  /* NULL EQN!?? */");
     return;
-  }  /* if */
+  } /* if */
 
   do {
-    if (iType == LX_IDENTIFIER)                 /* Process Identifier */
+    if (iType == LX_IDENTIFIER)   /* Process Identifier */
       TranslateID (pibDum, pfile, szLex, iEqType);
     
-    else if (iType == LX_PUNCT && IsComment(szLex)) {
+    else if ((iType == LX_EQNPUNCT || iType == LX_PUNCT) && IsComment(szLex)) {
       while (*pibDum->pbufCur && *pibDum->pbufCur != CH_EOLN)
         pibDum->pbufCur++;
-      fprintf (pfile, "\n");
-    }  /* if */
 
-    else                                        /* Spew everything else */
+      fprintf (pfile, "\n");
+    } /* if */
+
+    else /* Spew everything else */
       fprintf (pfile, "%s", szLex);
 
     fprintf (pfile, " ");
     NextLex (pibDum, szLex, &iType);
 
   } while (iType);
+
   fprintf (pfile, ";\n");
   
-}  /* TranslateEquation */
+} /* TranslateEquation */
 
 
-/* WriteOneEquation
+/* ----------------------------------------------------------------------------
+   WriteOneEquation
    
    Writes one equation of a function definition.  Uses the hType
    flag for spacing set by DefineEquation().
    
    Callback function for ForAllVar().
 */
-
-int WriteOneEquation (PFILE pfile, PVM pvm, PVOID pInfo)
+int WriteOneEquation (PFILE pfile, PVMMAPSTRCT pvm, PVOID pInfo)
 {
   int iType = (int) pInfo;
   
-  if (pvm->hType & ID_SPACEFLAG)        /* Flag for space between equations */
+  if (pvm->hType & ID_SPACEFLAG) /* Flag for space between equations */
     fprintf (pfile, "\n");
 
   switch (iType) {
     default:
-    case KM_SCALE:                      /* Scale Global Names */
+    case KM_SCALE: /* Scale Global Names */
+
+      /* Inputs not allowed anymore in Scale section - FB 7/12/96 */
+      if (TYPE(pvm) == ID_INPUT) {
+        printf ("Error: input '%s' used in Scale context.\n", 
+                pvm->szName);
+        exit(0);
+      }
+
       fprintf (pfile, "  %s%s = ", GetName (pvm, NULL, NULL, ID_NULL),
-               (TYPE(pvm) == ID_INPUT ? ".dMag" : "") );
+        (TYPE(pvm) == ID_INPUT ? ".dMag" : "") );
       break;
 
     case KM_CALCOUTPUTS:
     case KM_DYNAMICS:
     case KM_JACOB:
       fprintf (pfile, "  %s = ", GetName (pvm, "rgModelVars", "rgDerivs", 
-      ID_NULL));
+               ID_NULL));
       break;
-  }  /* switch */
+
+  } /* switch */
 
   TranslateEquation (pfile, pvm->szEqn, iType);
   return 1;
-}  /* WriteOneEquation */
+
+} /* WriteOneEquation */
 
 
-/* WriteCalcDeriv
+/* ----------------------------------------------------------------------------
+   WriteCalcDeriv
    
    Writes the CalcDeriv() function.  Writes dynamics equations in 
    the order they appeared in the model definition file.
 */
 
-void WriteCalcDeriv (PFILE pfile, PVM pvmGlo, PVM pvmDyn)
+void WriteCalcDeriv (PFILE pfile, PVMMAPSTRCT pvmGlo, PVMMAPSTRCT pvmDyn)
 {
-  fprintf (pfile, "/* Dynamics section */\n\n");
+  if (!pvmDyn)
+    printf ("No Dynamics{} equations.\n");
+
+  fprintf (pfile, "/* Dynamics section */\n");
   fprintf (pfile, "void CalcDeriv (double  rgModelVars[],\n");
   fprintf (pfile, "   double  rgDerivs[],\n");
   fprintf (pfile, "   PDOUBLE pdTime)\n");
   fprintf (pfile, "{\n");
-  fprintf (pfile, "  /* Local variables */\n\n");
+  fprintf (pfile, "  /* Local variables */\n");
+
   ForAllVar (pfile, pvmGlo, &WriteOneDecl, ID_LOCALDYN, NULL);
+
   fprintf (pfile, "\n");
-  fprintf (pfile, "  CalcInputs (pdTime);    /* Get new input vals */\n\n");
+  fprintf (pfile, "  CalcInputs (pdTime);    /* Get new input vals */\n");
 
-  ForAllVar (pfile, pvmDyn, &WriteOneEquation, ALL_VARS, (PVOID) KM_DYNAMICS);  
-  fprintf (pfile, "} /* CalcDeriv */\n\n");
+  ForAllVar (pfile, pvmDyn, &WriteOneEquation, ALL_VARS, (PVOID) KM_DYNAMICS); 
+ 
+  fprintf (pfile, "} /* CalcDeriv */\n");
 
-}  /* WriteCalcDeriv */
+} /* WriteCalcDeriv */
 
 
 
-/* WriteScale
-   
-   
+/* ----------------------------------------------------------------------------
+   WriteScale
 */
-
-void WriteScale (PFILE pfile, PVM pvmGlo, PVM pvmScale)
+void WriteScale (PFILE pfile, PVMMAPSTRCT pvmGlo, PVMMAPSTRCT pvmScale)
 {
   if (!pvmScale)
-    printf ("* No ScaleModel () equations. Null function defined.\n");
+    printf ("No Scale{} equations. Null function defined.\n");
     
-  fprintf (pfile, "/* Model scaling */\n\n");
+  fprintf (pfile, "/* Model scaling */\n");
   fprintf (pfile, "\nvoid ScaleModel (void)\n");
   fprintf (pfile, "{\n");
-  fprintf (pfile, "  /* Local variables */\n\n");
+  fprintf (pfile, "  /* Local variables */\n");
+
   ForAllVar (pfile, pvmGlo, &WriteOneDecl, ID_LOCALSCALE, NULL);
+
   fprintf (pfile, "\n");
+
   ForAllVar (pfile, pvmScale, &WriteOneEquation, ALL_VARS, (PVOID) KM_SCALE);  
-  fprintf (pfile, "} /* ScaleModel */\n\n");
-}  /* WriteScale */
+
+  fprintf (pfile, "} /* ScaleModel */\n");
+
+} /* WriteScale */
 
 
-/* WriteCalcJacob
-   
-   
+/* ----------------------------------------------------------------------------
+   WriteCalcJacob
 */
-
-void WriteCalcJacob (PFILE pfile, PVM pvmGlo, PVM pvmJacob)
+void WriteCalcJacob (PFILE pfile, PVMMAPSTRCT pvmGlo, PVMMAPSTRCT pvmJacob)
 {
   fprintf (pfile, "/* Jacobian calculation */\n\n");
   fprintf (pfile, "\nvoid CalcJacob (double rgModelVars[], double *rgdSave[]");
   fprintf (pfile, ", PDOUBLE pdTime)\n");
   fprintf (pfile, "{\n");
-  fprintf (pfile, "  /* Local variables */\n\n");
+  fprintf (pfile, "  /* Local variables */\n");
   ForAllVar (pfile, pvmGlo, &WriteOneDecl, ID_LOCALJACOB, NULL);
   fprintf (pfile, "\n");
   ForAllVar (pfile, pvmJacob, &WriteOneEquation, ALL_VARS, (PVOID) KM_JACOB);  
-  fprintf (pfile, "} /* CalcJacob */\n\n");
-}  /* WriteCalcJacob */
+  fprintf (pfile, "} /* CalcJacob */\n");
+} /* WriteCalcJacob */
 
 
-/* WriteCalcOutputs
-   
-   
+/* ----------------------------------------------------------------------------
+   WriteCalcOutputs
 */
-
-void WriteCalcOutputs (PFILE pfile, PVM pvmGlo, PVM pvmCalcOut)
+void WriteCalcOutputs (PFILE pfile, PVMMAPSTRCT pvmGlo, PVMMAPSTRCT pvmCalcOut)
 {
   if (!pvmCalcOut)
-    printf ("* No CalcOutput () equations. Null function defined.\n");
+    printf ("No CalcOutput{} equations. Null function defined.\n");
     
-  fprintf (pfile, "/* Outputs calculation */\n\n");
+  fprintf (pfile, "/* Outputs calculation */\n");
   fprintf (pfile, "\nvoid CalcOutputs (double  rgModelVars[],\n");
   fprintf (pfile, "      double  rgDerivs[],\n");
   fprintf (pfile, "      PDOUBLE pdTime)\n");
   fprintf (pfile, "{\n");
-  fprintf (pfile, "  /* Local variables */\n\n");
+  fprintf (pfile, "  /* Local variables */\n");
+
   ForAllVar (pfile, pvmGlo, &WriteOneDecl, ID_LOCALCALCOUT, NULL);
+
   fprintf (pfile, "\n");
-  ForAllVar (pfile, pvmCalcOut, &WriteOneEquation, ALL_VARS, (PVOID) KM_CALCOUTPUTS);  
-  fprintf (pfile, "} /* CalcOutputs */\n\n");
-}  /* WriteCalcOutputs */
+
+  ForAllVar (pfile, pvmCalcOut, &WriteOneEquation, ALL_VARS, 
+             (PVOID) KM_CALCOUTPUTS); 
+ 
+  fprintf (pfile, "} /* CalcOutputs */\n");
+
+} /* WriteCalcOutputs */
 
 
-/* IndexOneVar
+/* ----------------------------------------------------------------------------
+   IndexOneVar
    
    ORs the index passed through the info pointer, a PINT,
    and increments the value of the pointer.
@@ -680,18 +755,19 @@ void WriteCalcOutputs (PFILE pfile, PVM pvmGlo, PVM pvmCalcOut)
    
    Callback function for ForAllVar().
 */
-   
-int IndexOneVar (PFILE pfile, PVM pvm, PVOID pInfo)
+int IndexOneVar (PFILE pfile, PVMMAPSTRCT pvm, PVOID pInfo)
 {
-  if (pvm->szEqn != vszHasInitializer) { /*-- Handled later */
+  if (pvm->szEqn != vszHasInitializer) { /* Handled later */
     pvm->hType |= (*((PINT) pInfo))++;
     return 1;
   }  /* if */
   return 0;
-}  /* IndexOneVar */
+
+} /* IndexOneVar */
 
 
-/* IndexVariables
+/* ----------------------------------------------------------------------------
+   IndexVariables
 
    Creates indices and counts all the model variables.
    
@@ -702,21 +778,18 @@ int IndexOneVar (PFILE pfile, PVM pvm, PVOID pInfo)
    
    The indices to inputs are into the array of inputs.
 */
-
-void IndexVariables (PVM pvmGlo)
+void IndexVariables (PVMMAPSTRCT pvmGlo)
 {
   int iIndex, iMax = MAX_VARS;
   
-    /* Get counts */
-  
+  /* Get counts */
   vnStates  = ForAllVar (NULL, pvmGlo, &CountOneDecl, ID_STATE, NULL);
   vnOutputs = ForAllVar (NULL, pvmGlo, &CountOneDecl, ID_OUTPUT, NULL);
   vnInputs  = ForAllVar (NULL, pvmGlo, &CountOneDecl, ID_INPUT, NULL);
   vnParms   = ForAllVar (NULL, pvmGlo, &CountOneDecl, ID_PARM, NULL);
   vnModelVars = vnStates + vnOutputs;
   
-        /* Report all errors */
-
+  /* Report all errors */
   if (vnStates > MAX_VARS)
     ReportError (NULL, RE_TOOMANYVARS, "state", (PSTR) &iMax);
   if (vnOutputs > MAX_VARS)
@@ -730,10 +803,9 @@ void IndexVariables (PVM pvmGlo)
       || vnInputs > MAX_VARS
       || vnOutputs > MAX_VARS
       || vnParms > iMax)
-    ReportError (NULL, RE_FATAL, NULL, NULL);   /* Abort generation */
+    ReportError (NULL, RE_FATAL, NULL, NULL); /* Abort generation */
 
-    /* Set indices */
-  
+  /* Set indices */
   iIndex = 0;
   ForAllVar (NULL, pvmGlo, &IndexOneVar, ID_STATE, (PVOID) &iIndex);
   ForAllVar (NULL, pvmGlo, &IndexOneVar, ID_OUTPUT, (PVOID) &iIndex);
@@ -744,24 +816,24 @@ void IndexVariables (PVM pvmGlo)
   iIndex = vnStates + vnOutputs + vnInputs;
   ForAllVar (NULL, pvmGlo, &IndexOneVar, ID_PARM, (PVOID) &iIndex);
   
-}  /* IndexVariables */
+} /* IndexVariables */
 
 
-/* AdjustOneVar
+/* ----------------------------------------------------------------------------
+   AdjustOneVar
    
    Increments the dependent parameter handles of an input by the
    iOffset given through the info pointer.
    
    Callback function for ForAllVar().
 */
-   
-int AdjustOneVar (PFILE pfile, PVM pvm, PVOID pInfo)
+int AdjustOneVar (PFILE pfile, PVMMAPSTRCT pvm, PVOID pInfo)
 {
   PIFN pifn = (PIFN) pvm->szEqn;  
   WORD wOffset = *(PWORD) pInfo;
 
   if (!pifn)
-    return 1;       /*-- No eqn!  No dependent parm! */
+    return 1; /* No eqn!  No dependent parm! */
   
   if (pifn->hMag)
     pifn->hMag += wOffset;
@@ -775,25 +847,28 @@ int AdjustOneVar (PFILE pfile, PVM pvm, PVOID pInfo)
     pifn->hDecay += wOffset;
 
   return 1;
-}  /* AdjustOneVar */
+
+} /* AdjustOneVar */
 
 
-/* AdjustVarHandles
+/* ----------------------------------------------------------------------------
+   AdjustVarHandles
    
    Adjusts the variable handles on input definitions.  They must be
    incremented by the beginning offset of the parameter section of
    the global variable map.
 */
-
-void AdjustVarHandles (PVM pvmGlo)
+void AdjustVarHandles (PVMMAPSTRCT pvmGlo)
 {
   WORD wOffset = (WORD) vnInputs + vnStates + vnOutputs;
+
+  ForAllVar (NULL, pvmGlo, &AdjustOneVar, ID_INPUT, (PVOID) &wOffset);
   
-  ForAllVar (NULL, pvmGlo, &AdjustOneVar, ID_INPUT, (PVOID) &wOffset);  
-}  /* AdjustVarHandles */
+} /* AdjustVarHandles */
 
 
-/* ReversePointers
+/* ----------------------------------------------------------------------------
+   ReversePointers
    
    Flips the pointer on the var-list so that the head is the tail
    and the tail is the head and things aren't what they seem to be.
@@ -801,13 +876,12 @@ void AdjustVarHandles (PVM pvmGlo)
    The dynamic equation list must be reversed so that the equations
    appear in the right order since they were created as a stack.
 */
-
-void ReversePointers (PVM *ppvm)
+void ReversePointers (PVMMAPSTRCT *ppvm)
 {
-  PVM pvmPrev, pvmNext;
+  PVMMAPSTRCT pvmPrev, pvmNext;
 
   if (!ppvm || !(*ppvm)
-      || !(*ppvm)->pvmNextVar)  /* List of one is already reversed! */
+      || !(*ppvm)->pvmNextVar) /* List of one is already reversed! */
     return;
 
   pvmPrev = NULL;  
@@ -815,13 +889,15 @@ void ReversePointers (PVM *ppvm)
     (*ppvm)->pvmNextVar = pvmPrev;
     pvmPrev = (*ppvm);
     *ppvm = pvmNext;
-  }  /* while */
+  } /* while */
 
-  (*ppvm)->pvmNextVar = pvmPrev;        /* Link new head to reversed list */
+  (*ppvm)->pvmNextVar = pvmPrev; /* Link new head to reversed list */
 
-}  /* ReversePointers */
+} /* ReversePointers */
 
-/* AssertExistsEqn
+
+/* ----------------------------------------------------------------------------
+   AssertExistsEqn
    
    Check that equation exists.
    
@@ -841,45 +917,49 @@ void ReversePointers (PVM *ppvm)
    The errors are not reported as fatal so that all errors can
    be discovered.  They will cause exit subsequently.
 */
-
-int AssertExistsEqn (PFILE pfile, PVM pvm, PVOID pInfo)
+int AssertExistsEqn (PFILE pfile, PVMMAPSTRCT pvm, PVOID pInfo)
 {
   int iReturn = 0;
-  PVM pvmDyn = (PVM) pInfo;
+  PVMMAPSTRCT pvmDyn = (PVMMAPSTRCT) pInfo;
 
-  if (pvm->szEqn != vszHasInitializer) { /*-- Don't count these! */
+  if (pvm->szEqn != vszHasInitializer) { /* Don't count these! */
     if (pvmDyn) {
       if (!(iReturn = (int) GetVarPTR (pvmDyn, pvm->szName)))
-  ReportError (NULL, RE_NODYNEQN, pvm->szName, NULL);
+        ReportError (NULL, RE_NODYNEQN, pvm->szName, NULL);
     }
     else
       if (!(iReturn = (int) pvm->szEqn))
-  ReportError (NULL, RE_NOINPDEF, pvm->szName, NULL);
-  }  /* if */
-  /*-- If HasInitializer, no error to report */
+        ReportError (NULL, RE_NOINPDEF, pvm->szName, NULL);
+  } /* if */
+
+  /* If HasInitializer, no error to report */
   return (iReturn ? 1 : 0);
-}  /* AssertExistsEqn */
+
+} /* AssertExistsEqn */
 
 
-void VerifyEqns (PVM pvmGlo, PVM pvmDyn)
+/* ----------------------------------------------------------------------------
+*/
+void VerifyEqns (PVMMAPSTRCT pvmGlo, PVMMAPSTRCT pvmDyn)
 {
-  BOOL bStatesOK;  /*, bInputsOK;*/
+  BOOL bStatesOK;
 
-  bStatesOK =
-    (vnStates == ForAllVar (NULL, pvmGlo, &AssertExistsEqn,
-                            ID_STATE, (PVOID) pvmDyn));
+  bStatesOK = (vnStates == ForAllVar (NULL, pvmGlo, &AssertExistsEqn,
+                                      ID_STATE, (PVOID) pvmDyn));
 
   if (!bStatesOK)
     ReportError (NULL, RE_FATAL, NULL, "State equations missing");
-}  /* VerifyEqns */
+
+} /* VerifyEqns */
 
 void MakeCodeFilename (PSTR szNewName, PSTR szName)
 {
   strcpy (szNewName, szName);
-#ifdef __DOS__
+
+#ifdef _DOS_
   /* In Dos, you can only have one '.' in a filename */
-  strtok (szNewName, ".");              /* Remove any extension */
-  strcat (szNewName, ".c");             /* Create ".c" filename */
+  strtok (szNewName, ".");  /* Remove any extension */
+  strcat (szNewName, ".c");  /* Create ".c" filename */
 
 #else
   {
@@ -887,29 +967,33 @@ void MakeCodeFilename (PSTR szNewName, PSTR szName)
 
   cchName = strlen (szNewName);
   if (!((cchName > 1)
-  && (szNewName[cchName-1] == 'c')
-  && (szNewName[cchName-2] == '.')))    /* Look for current extension */
-  strcat (szNewName, ".c");             /* Create ".c" filename */
+      && (szNewName[cchName-1] == 'c')
+      && (szNewName[cchName-2] == '.'))) /* Look for current extension */
+    strcat (szNewName, ".c"); /* Create ".c" filename */
   } /* block */
+
 #endif
+
 } /* MakeCodeFilename */
   
 
-/* WriteModel
+/* ----------------------------------------------------------------------------
+   WriteModel
    
    Writes the model calculation file szOutFilename for the parameters
    and dynamic equations given.
 */
-   
 void WriteModel (PINPUTINFO pinfo, int nArg, PSTR rgszArgs[])
 {
   PFILE pfile;
   char szFileWithExt[FILENAME_MAX];
-static char vszFilenameDefault[] = "model.c";
+  static char vszFilenameDefault[] = "model.c";
 
-  if (!pinfo->pvmGloVars || !pinfo->pvmDynEqns)
+  if (!pinfo->pvmGloVars || (!pinfo->pvmDynEqns && !pinfo->pvmCalcOutEqns)) {
+    printf ("Error: No Dynamics, no outputs or no global variables defined\n");
     return;
-
+  }
+  
   ReversePointers (&pinfo->pvmGloVars);
   ReversePointers (&pinfo->pvmDynEqns);
   ReversePointers (&pinfo->pvmScaleEqns);
@@ -921,15 +1005,19 @@ static char vszFilenameDefault[] = "model.c";
   AdjustVarHandles (pinfo->pvmGloVars);
   VerifyEqns (pinfo->pvmGloVars, pinfo->pvmDynEqns);
 
-  MakeCodeFilename (szFileWithExt,
+  MakeCodeFilename (szFileWithExt, 
                     (nArg > 2 ? rgszArgs[2] : vszFilenameDefault));
 
   if ((pfile = fopen (szFileWithExt, "w"))) {
 
-  /* Keep track of the model description file and generator name */
+    /* Keep track of the model description file and generator name */
     vszModelFilename = pinfo->szInputFilename;
-    vszModGenName = "PPC Mod"; /* rgszArgs[0]; */
-
+#ifdef _MACOS_
+    vszModGenName = "Mod";
+#else
+    vszModGenName = rgszArgs[0];
+#endif
+    
     WriteHeader (pfile, szFileWithExt, pinfo->pvmGloVars);
 
     WriteIncludes (pfile);
@@ -937,6 +1025,7 @@ static char vszFilenameDefault[] = "model.c";
     WriteVarMap (pfile, pinfo->pvmGloVars);
 
     WriteInitModel (pfile, pinfo->pvmGloVars);
+/*  WriteInitExpt (pfile, pinfo->pvmGloVars); FB 7/12/96 commented out */
     WriteCalcDeriv (pfile, pinfo->pvmGloVars, pinfo->pvmDynEqns);
     WriteScale (pfile, pinfo->pvmGloVars, pinfo->pvmScaleEqns);
     WriteCalcJacob (pfile, pinfo->pvmGloVars, pinfo->pvmJacobEqns);
@@ -945,10 +1034,11 @@ static char vszFilenameDefault[] = "model.c";
     fclose (pfile);
     printf ("\n* Created model file '%s'\n", szFileWithExt);
 
-  }  /* if */
+  } /* if */
 
   if (!pfile)
     ReportError (NULL, RE_CANNOTOPEN | RE_FATAL,
-           szFileWithExt, "...in WriteModel ()");
-}  /* WriteModel */
+                 szFileWithExt, "...in WriteModel ()");
+
+} /* WriteModel */
 
