@@ -5,11 +5,33 @@
 
    Copyright (c) 1993.  Don Maszle, Frederic Bois.  All rights reserved.
 
+   This program is free software; you can redistribute it and/or
+   modify it under the terms of the GNU General Public License
+   as published by the Free Software Foundation; either version 2
+   of the License, or (at your option) any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+
+   contact
+     Frdric Bois / Don Maszle
+     BEHS, School of Public Health
+     University of California at Berkeley
+     Berkeley, CA 94720
+
+     fbois@diana.lbl.gov
+
    -- Revisions -----
-     Logfile:  %F%
-    Revision:  %I%
-        Date:  %G%
-     Modtime:  %U%
+     Logfile:  SCCS/s.lex.c
+    Revision:  1.14
+        Date:  29 Jul 1997
+     Modtime:  08:08:44
       Author:  @a
    -- SCCS  ---------
 
@@ -151,13 +173,14 @@ BOOL InitBuffer (PINPUTBUF pibIn, PSTR szFullPathname)
   pibIn->pInfo = NULL;
   pibIn->pbufCur = NULL;
 
-  if ((pibIn->pfileIn = fopen (szFullPathname, "r"))
-      && (pibIn->pbufOrg = (PBUF) malloc (BUFFER_SIZE))) {
-    bReturn = FillBuffer (pibIn);
-  } /* if */
+  if ((pibIn->pfileIn = fopen (szFullPathname, "r"))) {
+    if ((pibIn->pbufOrg = (PBUF) malloc (BUFFER_SIZE)))
+      bReturn = FillBuffer (pibIn);
+    else
+      ReportError (pibIn, RE_OUTOFMEM | RE_FATAL, "InitBuffer", NULL);
+  }
   else
-    ReportError (pibIn,
-                 RE_FILENOTFOUND | RE_FATAL, szFullPathname, NULL);
+    ReportError (pibIn, RE_FILENOTFOUND | RE_FATAL, szFullPathname, NULL);
 
   return (bReturn);
 
