@@ -5,15 +5,40 @@
 
    Copyright (c) 1993.  Don Maszle, Frederic Bois.  All rights reserved.
 
+   This program is free software; you can redistribute it and/or
+   modify it under the terms of the GNU General Public License
+   as published by the Free Software Foundation; either version 2
+   of the License, or (at your option) any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+
+   contact
+     Frdric Bois / Don Maszle
+     BEHS, School of Public Health
+     University of California at Berkeley
+     Berkeley, CA 94720
+
+     fbois@diana.lbl.gov
+
    -- Revisions -----
-     Logfile:  %F%
-    Revision:  %I%
-        Date:  %G%
-     Modtime:  %U%
+     Logfile:  SCCS/s.lex.h
+    Revision:  1.14
+        Date:  02 Jul 1997
+     Modtime:  08:36:57
       Author:  @a
    -- SCCS  ---------
 
    Header file for Lexical parsing routines.
+
+   Modified 21/3/97 by FB set MAX_ERRORS to zero.
+
 */
 
 #ifndef _LEX_H_
@@ -59,7 +84,7 @@
 
 /* Report Error constants -- Lex errors */
 
-#define MAX_ERRORS 20
+#define MAX_ERRORS 0
 
 #define RE_FATAL        0x8000  /* Can be ORd to iCode to cause exit(1) */
 #define RE_WARNING      0x4000  /* can be ORd to issue Warning instead */
@@ -106,6 +131,7 @@
 #define RE_UNEQUALTIMES     (RE_MODERROR + 20) /* Model variables in 'Distrib' statement have
                                                   != times in 'Print' statements */
 #define RE_DUPVARINPRINT    (RE_MODERROR + 21) /* Same var appears in 2 or more 'Print' statements */
+#define RE_NODATASTATEMENT  (RE_MODERROR + 22) /* No 'Data' for model var in 'Distrib' */
 
 #define RE_SIMERROR         0x0200             /* Simulation error prefix */
 #define RE_ERRORSINEXP      (RE_SIMERROR + 1)  /* Errors reported, skipping exp */
