@@ -5,15 +5,40 @@
 
    Copyright (c) 1993.  Don Maszle, Frederic Bois.  All rights reserved.
 
+   This program is free software; you can redistribute it and/or
+   modify it under the terms of the GNU General Public License
+   as published by the Free Software Foundation; either version 2
+   of the License, or (at your option) any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+
+   contact
+     Frdric Bois / Don Maszle
+     BEHS, School of Public Health
+     University of California at Berkeley
+     Berkeley, CA 94720
+
+     fbois@diana.lbl.gov
+
    -- Revisions -----
-     Logfile:  %F%
-    Revision:  %I%
-        Date:  %G%
-     Modtime:  %U%
+     Logfile:  SCCS/s.lexerr.c
+    Revision:  1.16
+        Date:  26 Jul 1997
+     Modtime:  02:54:25
       Author:  @a
    -- SCCS  ---------
 
    Reports errors and exits program if fatal.
+
+   Modified 18/3/97 by FB: check for validity of pibIn before FreeLevels
+     in ReportError.
 */
 
 #include <stdlib.h>
@@ -226,16 +251,16 @@ void ReportError (PINPUTBUF pibIn, WORD wCode, PSTR szMsg, PSTR szAltMsg)
     printf ("%s\n", szAltMsg);
 
   if (bFatal) {
-    if(pibIn->pInfo != NULL)
+    if ((pibIn != NULL) && (pibIn->pInfo != NULL))
       FreeLevels((PANALYSIS)pibIn->pInfo);
-    printf ("\nFatal errors.  Exiting.\n");
 
+    printf ("\nFatal errors.  Exiting.\n\n");
 #ifdef _MACOS_
     getchar();
 #endif
-
     exit (wCode);
-  } /* if */
+
+  } /* if bFatal */
 
 } /* ReportError */
 
@@ -244,7 +269,7 @@ void ReportError (PINPUTBUF pibIn, WORD wCode, PSTR szMsg, PSTR szAltMsg)
    ReportRunTimeError
 
    Reports error iCode to terminal (one of RE_) and optional
-   szMessage.   If iSeverity is set to RE_FATAL, exits program.
+   szMessage. If iSeverity is set to RE_FATAL, exits program.
 */
 
 void ReportRunTimeError (PANALYSIS panal, WORD wCode, ...)
@@ -332,6 +357,11 @@ void ReportRunTimeError (PANALYSIS panal, WORD wCode, ...)
     printf ("Bad value in output; model is not computable\n");
     break;
 
+  case RE_NODATASTATEMENT:
+    printf ("Found model variable in 'Distrib' statement without\n");
+    printf ("  corresponding 'Data' statement\n");
+    break;
+
   case RE_NOPRINTSTATEMENT:
     printf ("Found model variable in 'Distrib' statement without\n");
     printf ("  corresponding 'Print' statement\n");
@@ -356,14 +386,16 @@ void ReportRunTimeError (PANALYSIS panal, WORD wCode, ...)
 
   va_end(ap);
 
-  if (bFatal && panal != NULL) {
-    FreeLevels(panal);
+  if (bFatal) {
+    if (panal != NULL)
+      FreeLevels(panal);
 
+    printf ("\nFatal errors.  Exiting.\n\n");
 #ifdef _MACOS_
     getchar();
 #endif
-
     exit (wCode);
-  } /* if */
+
+  } /* if bFatal */
 
 } /* ReportRunTimeError */
