@@ -5,11 +5,33 @@
 
    Copyright (c) 1993.  Don Maszle, Frederic Bois.  All rights reserved.
 
+   This program is free software; you can redistribute it and/or
+   modify it under the terms of the GNU General Public License
+   as published by the Free Software Foundation; either version 2
+   of the License, or (at your option) any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+
+   contact
+     Frdric Bois / Don Maszle
+     BEHS, School of Public Health
+     University of California at Berkeley
+     Berkeley, CA 94720
+
+     fbois@diana.lbl.gov
+
    -- Revisions -----
-     Logfile:  %F%
-    Revision:  %I%
-        Date:  %G%
-     Modtime:  %U%
+     Logfile:  SCCS/s.lexfn.c
+    Revision:  1.15
+        Date:  02 Jul 1997
+     Modtime:  08:36:59
       Author:  @a
    -- SCCS  ---------
 
@@ -306,19 +328,19 @@ BOOL GetNDoses (PINPUTBUF pibIn, PSTR szLex, PIFN pifn)
     ReportError (pibIn, RE_OUTOFMEM | RE_FATAL, "GetNDoses", NULL);
 
   if (!(pifn->rghT0s =
-       (HANDLE *) malloc ((pifn->nDoses+1) * sizeof(unsigned int)))
-      || !(pifn->rghMags =
-          (HANDLE *) malloc (pifn->nDoses * sizeof(unsigned int))))
+       (HANDLE *) malloc ((pifn->nDoses+1) * sizeof(unsigned int))) || 
+      !(pifn->rghMags =
+       (HANDLE *) malloc (pifn->nDoses * sizeof(unsigned int))))
     ReportError (pibIn, RE_OUTOFMEM | RE_FATAL, "GetNDoses", NULL);
 
   /* Try to get doses list: n Mag's, n T0's */
 
-  for (i = 0; i < iDoseArg && bOK; i++ ) {
+  for (i = 0; i < iDoseArg && bOK; i++) {
 
     rgiTypes[i] = LX_INTEGER | LX_FLOAT | LX_IDENTIFIER;
 
     if (!(bOK = GetOptPunct (pibIn, rgszLex[i], ','))) {
-      *( rgszLex[i] + 1 )  = ',';
+      *(rgszLex[i] + 1)  = ',';
       ReportError(pibIn, RE_EXPECTED, rgszLex[i], NULL);
       break; /* Error: Stop getting args */
     }
@@ -354,7 +376,7 @@ BOOL GetNDoses (PINPUTBUF pibIn, PSTR szLex, PIFN pifn)
 Exit_GetNDoses:
 
   if (bErr)
-    printf ("Syntax: NDoses (nDoses, <n Magnitudes>, <n T0's>)\n\n");
+    printf ("Syntax: NDoses (nInputs, <n Magnitudes>, <n T0's>)\n\n");
 
   return (!bErr);
 
@@ -370,20 +392,84 @@ Exit_GetNDoses:
 
 BOOL GetSpikes (PINPUTBUF pibIn, PSTR szLex, PIFN pifn)
 {
+  PSTRLEX *rgszLex;
+  int *rgiTypes, iType;
+  int i, iDoseArg;
+  BOOL bOK = TRUE;
   BOOL bErr = FALSE;    /* Return value flags error condition */
 
   if ((bErr = EGetPunct (pibIn, szLex, CH_LPAREN)))
     goto Exit_GetSpikes;
 
-    /* Get positive integer number of spikes */
+  /* Get positive integer number of spikes */
 
   if ((bErr = ENextLex (pibIn, szLex, LX_INTEGER)))
     goto Exit_GetSpikes;
+
   pifn->nDoses = atoi(szLex);
+
   if ((bErr = (pifn->nDoses <= 0))) {
     ReportError (pibIn, RE_LEXEXPECTED, "positive-integer", szLex);
     goto Exit_GetSpikes;
-  } /* if */
+  }
+
+  /* iDoseArg is the number of arguments (doses and starting times)
+     in Spikes after the first integer, number of doses */
+  iDoseArg = 2 * pifn->nDoses;
+
+  if ( !(rgiTypes = InitiVector (iDoseArg)))
+    ReportError (pibIn, RE_OUTOFMEM | RE_FATAL, "GetNDoses", NULL);
+
+  if ( !(rgszLex = (PSTRLEX *) malloc (iDoseArg * sizeof(PSTRLEX))))
+    ReportError (pibIn, RE_OUTOFMEM | RE_FATAL, "GetNDoses", NULL);
+
+  if ( !(pifn->rgT0s = InitdVector (pifn->nDoses)) ||
+       !(pifn->rgMags = InitdVector (pifn->nDoses)))
+    ReportError (pibIn, RE_OUTOFMEM | RE_FATAL, "GetNDoses", NULL);
+
+  if (!(pifn->rghT0s =
+       (HANDLE *) malloc ((pifn->nDoses) * sizeof(unsigned int))) || 
+      !(pifn->rghMags =
+       (HANDLE *) malloc (pifn->nDoses * sizeof(unsigned int))))
+    ReportError (pibIn, RE_OUTOFMEM | RE_FATAL, "GetNDoses", NULL);
+
+  /* Try to get list: n Mag's, n T0's */
+
+  for (i = 0; i < iDoseArg && bOK; i++) {
+
+    rgiTypes[i] = LX_INTEGER | LX_FLOAT | LX_IDENTIFIER;
+
+    if (!(bOK = GetOptPunct (pibIn, rgszLex[i], ','))) {
+      *(rgszLex[i] + 1)  = ',';
+      ReportError(pibIn, RE_EXPECTED | RE_FATAL, rgszLex[i], NULL);
+      break; /* Error: Stop getting args */
+    }
+
+    NextLex (pibIn, rgszLex[i], &iType);
+    if (!(bOK &= (iType & rgiTypes[i]) > 0))
+      ReportError(pibIn, RE_LEXEXPECTED, vrgszLexTypes[rgiTypes[i]],
+                  rgszLex[i]);
+  } /* for */
+
+  if ((bErr = EGetPunct (pibIn, szLex, CH_RPAREN)))
+    goto Exit_GetSpikes;
+
+  /* Try to get each parm to show all errors */
+
+  bOK = TRUE;
+
+  /* magnitudes */
+  for (i = 0; i < pifn->nDoses; i++)
+    bOK &= DefDepParm (rgszLex[i], pifn->rgMags + i, pifn->rghMags + i);
+
+  /* times */
+  for (i = 0; i < pifn->nDoses; i++)
+    bOK &= DefDepParm (rgszLex[i+pifn->nDoses], pifn->rgT0s+i, pifn->rghT0s+i);
+
+  if (!bOK) ReportError (pibIn, RE_EXPECTED, "input-spec", NULL);
+
+#ifdef ndef 
+  /* obsolete code FB 11/6/97 */
 
   if ( !(pifn->rgT0s  = InitdVector (pifn->nDoses)) ||
        !(pifn->rgMags = InitdVector (pifn->nDoses)))
@@ -401,10 +487,12 @@ BOOL GetSpikes (PINPUTBUF pibIn, PSTR szLex, PIFN pifn)
   if (!bErr)
     bErr = EGetPunct (pibIn, szLex, CH_RPAREN);
 
+#endif
+
 Exit_GetSpikes:
 
   if (bErr)
-    printf ("Syntax: Spikes (nDoses, <n Magnitudes>, <n Times>)\n\n");
+    printf ("Syntax: Spikes (nInputs, <n Magnitudes>, <n Times>)\n\n");
 
   return (!bErr);
 
@@ -470,23 +558,23 @@ BOOL GetInputFn (PINPUTBUF pibIn, PSTR sz, PIFN pifn)
       switch (pifn->iType) {
 
       case IFN_NDOSES:
-    bReturn = GetNDoses (pibDum, szLex, pifn);
-    break;
+        bReturn = GetNDoses (pibDum, szLex, pifn);
+        break;
 
       case IFN_SPIKES:
-    bReturn = GetSpikes (pibDum, szLex, pifn);
-    break;
+        bReturn = GetSpikes (pibDum, szLex, pifn);
+        break;
 
       default:
-    pifn->iType = IFN_NULL;
-    ReportError (pibIn, RE_LEXEXPECTED, "input-spec", szLex);
-    break;
+        pifn->iType = IFN_NULL;
+        ReportError (pibIn, RE_LEXEXPECTED, "input-spec", szLex);
+        break;
 
       case IFN_PERDOSE:
       case IFN_PERRATE:
       case IFN_PEREXP:
-    bReturn = GetInputArgs (pibDum, pifn);
-    break;
+        bReturn = GetInputArgs (pibDum, pifn);
+        break;
 
       } /* switch */
     } /* if identifier */
