@@ -5,11 +5,33 @@
 
    Copyright (c) 1993.  Don Maszle, Frederic Bois.  All rights reserved.
 
+   This program is free software; you can redistribute it and/or
+   modify it under the terms of the GNU General Public License
+   as published by the Free Software Foundation; either version 2
+   of the License, or (at your option) any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+
+   contact
+     Frdric Bois / Don Maszle
+     BEHS, School of Public Health
+     University of California at Berkeley
+     Berkeley, CA 94720
+
+     fbois@diana.lbl.gov
+
    -- Revisions -----
      Logfile:  SCCS/s.lexfn.h
-    Revision:  1.2
-        Date:  2/6/94
-     Modtime:  18:21:43
+    Revision:  1.10
+        Date:  02 Jul 1997
+     Modtime:  08:36:59
       Author:  @a
    -- SCCS  ---------
 
@@ -26,17 +48,17 @@
 /* -----------------------------------------------------------------------------
    Constants  */
 
-#define N_TAU_EXPOSE    40    /* Number of Tau's to expose exp() inputs */
+#define N_TAU_EXPOSE 40 /* Number of Tau's to expose exp() inputs */
 
 /* Input Function constants */
 
-#define IFN_NULL    0
-#define IFN_CONSTANT    1
-#define IFN_PERDOSE    2
-#define IFN_PERRATE    3
-#define IFN_PEREXP    4
-#define IFN_NDOSES    5
-#define IFN_SPIKES      6
+#define IFN_NULL     0
+#define IFN_CONSTANT 1
+#define IFN_PERDOSE  2
+#define IFN_PERRATE  3
+#define IFN_PEREXP   4
+#define IFN_NDOSES   5
+#define IFN_SPIKES   6
 
 /* -----------------------------------------------------------------------------
    Typedefs  */
@@ -98,7 +120,7 @@ typedef struct tagIFN {
 
   /* Multiple dose inputs */
 
-  int nDoses;              /* Number of doses of spikes */
+  int nDoses;              /* Number of doses of Spikes */
   int iDoseCur;            /* Current Dose */
 
   /* For value input */
