@@ -5,11 +5,33 @@
 
    Copyright (c) 1993.  Don Maszle, Frederic Bois.  All rights reserved.
 
+   This program is free software; you can redistribute it and/or
+   modify it under the terms of the GNU General Public License
+   as published by the Free Software Foundation; either version 2
+   of the License, or (at your option) any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+
+   contact
+     Frdric Bois / Don Maszle
+     BEHS, School of Public Health
+     University of California at Berkeley
+     Berkeley, CA 94720
+
+     fbois@diana.lbl.gov
+
    -- Revisions -----
-     Logfile:  %F%
-    Revision:  %I%
-        Date:  %G%
-     Modtime:  %U%
+     Logfile:  SCCS/s.list.c
+    Revision:  1.12
+        Date:  28 Jul 1997
+     Modtime:  03:33:52
       Author:  @a
    -- SCCS  ---------
 
@@ -60,8 +82,7 @@ PLIST InitList ()
    If the callback is NULL and the boolean bAndData is set, free() is
    called with each user data pointer before freeing the list element.
 
-   NOTE:  ppList is a pointer to a LIST structure which is itself a
-          pointer.
+   NOTE: ppList is a pointer to a LIST structure which is itself a pointer.
 */
 
 void FreeList (PLIST *pplist, PFV_FREELISTCALLBACK pfvFreeData, BOOL bAndData)
@@ -103,9 +124,8 @@ void QueueListItem (PLIST plist, PVOID pData)
   if (!plist)
     return;
 
-  if (!(pNewElem = (PLISTELEM) malloc (sizeof(LISTELEM)))) {
+  if (!(pNewElem = (PLISTELEM) malloc (sizeof(LISTELEM))))
     ReportError (NULL, RE_OUTOFMEM | RE_FATAL, "QueueListItem", NULL);
-  } /* if */
 
   pNewElem->pData = pData; /* Init new element */
   pNewElem->pleNext = NULL;
