@@ -6,11 +6,33 @@ lsodes.h
 
    Copyright (c) 1993.  Don Maszle, Frederic Bois.  All rights reserved.
 
+   This program is free software; you can redistribute it and/or
+   modify it under the terms of the GNU General Public License
+   as published by the Free Software Foundation; either version 2
+   of the License, or (at your option) any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+
+   contact
+     Frdric Bois / Don Maszle
+     BEHS, School of Public Health
+     University of California at Berkeley
+     Berkeley, CA 94720
+
+     fbois@diana.lbl.gov
+
    -- Revisions -----
-     Logfile:  %F%
-    Revision:  %I%
-        Date:  %G%
-     Modtime:  %U%
+     Logfile:  SCCS/s.lsodes.h
+    Revision:  1.14
+        Date:  02 Jul 1997
+     Modtime:  08:37:01
       Author:  @a
    -- SCCS  ---------
 
@@ -38,10 +60,10 @@ This is the header file for the two parts
 /* ----------------------------------------------------------------------------
    macros */
 
-#define min(a,b) ((a) <= (b) ? (a) : (b))
-#define max(a,b) ((a) >= (b) ? (a) : (b))
-#define dmin(a,b) (double)min(a,b)
-#define dmax(a,b) (double)max(a,b)
+#define mymin(a,b) ((a) <= (b) ? (a) : (b))
+#define mymax(a,b) ((a) >= (b) ? (a) : (b))
+#define mydmin(a,b) (double)mymin(a,b)
+#define mydmax(a,b) (double)mymax(a,b)
 
 
 /* ----------------------------------------------------------------------------
