@@ -5,14 +5,49 @@
 
    Copyright (c) 1996.  Frederic Bois.  All rights reserved.
 
+   This program is free software; you can redistribute it and/or
+   modify it under the terms of the GNU General Public License
+   as published by the Free Software Foundation; either version 2
+   of the License, or (at your option) any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+
+   contact
+     Frdric Bois / Don Maszle
+     BEHS, School of Public Health
+     University of California at Berkeley
+     Berkeley, CA 94720
+
+     fbois@diana.lbl.gov
+
    -- Revisions -----
-     Logfile:  %F%
-    Revision:  %I%
-        Date:  %G%
-     Modtime:  %U%
+     Logfile:  SCCS/s.mh.c
+    Revision:  1.30
+        Date:  30 Jul 1997
+     Modtime:  06:53:16
       Author:  @a
    -- SCCS  ---------
 
+   Modified on 18/03/97 by FB: in LnDensity, improved error message for 
+   exponential density.
+
+   Modified on 12/05/97 by FB: in SampleThetas, improved the kernel adjustment.
+  
+   Modified on 21/05/97 by FB: in CheckPrintStatements check that a likelihood
+   is defined (via panal->nModelVars).
+
+   Modified on 22/06/97 by FB: in DoMarkov used pgd->nInitIter to force
+   printing of the times, data, and predictions for the last line of the 
+   restart file
+
+   Modified on 02/07/97 by FB: upgraded SetKernel and SetMCVars.
 */
 
 #include <stdio.h>
@@ -88,6 +123,8 @@ void CheckForFixed (PLEVEL plevel, char **args)
    the data likelihood for var1. Here we check these times.
    We also check for two print statements for the same variable, which can
    affect the output under some circumstances.
+   Checks also that data statements match print statements, at least for 
+   the variables var1 whose likelihood is written. 
    Called from TraverseLevels
 */
 
@@ -103,12 +140,19 @@ void CheckPrintStatements (PLEVEL plevel, char **args)
 
   pos = &(plevel->pexpt->os);
 
+  /* check that the same var does not appear in 2 or more print statements */
   for (n = 0; n < pos->nOutputs; ++n)
     for (m = n+1; m < pos->nOutputs; ++m)
       if (pos->phvar[n] == pos->phvar[m])
         ReportRunTimeError 
           (panal, RE_DUPVARINPRINT | RE_FATAL, "CheckPrintStatements");
 
+  if (panal->nModelVars == 0) {
+    printf ("\nError: you must specify a likelihood function for MCMC "
+            "sampling - Exiting.\n\n");
+    exit(0);
+  }
+
   for (n = 0; n < panal->nModelVars; ++n) {
     pMCVar = panal->rgpModelVars[n];
 
@@ -131,6 +175,10 @@ void CheckPrintStatements (PLEVEL plevel, char **args)
         ReportRunTimeError
           (panal, RE_UNEQUALNUMTIMES | RE_FATAL, "CheckPrintStatements");
 
+      if (!pos->prgdDataVals[VarIndex]) /* FB 01/07/97 */
+        ReportRunTimeError 
+          (panal, RE_NODATASTATEMENT | RE_FATAL, "CheckPrintStatements");
+
       for (m = 0; m < pos->pcOutputTimes[VarIndex]; ++m)
         if (pos->prgdOutputTimes[VarIndex][m] !=
             pos->prgdOutputTimes[SDIndex][m])
@@ -161,20 +209,24 @@ void CloneMCVars (PLEVEL plevel, char **args)
   for (n = 0; n < plevel->iInstances; ++n) {
     pLower = plevel->pLevels[n];
     pLower->nMCVars = nMCVars;
-    pLower->rgpMCVars = (PMCVAR*) malloc (nMCVars * sizeof(PMCVAR));
+    if (nMCVars != 0) {
+      if ((pLower->rgpMCVars = (PMCVAR*) malloc (nMCVars * sizeof(PMCVAR))) ==
+          NULL)
+      ReportError (NULL, RE_OUTOFMEM | RE_FATAL, "CloneMCVars", NULL);
+    }
   }
 
   nMCVars = 0;
   ForAllList3 (plevel->plistMCVars, &CloneMCVarsL, plevel, &nMCVars, NULL);
-
+  
 } /* CloneMCVars */
 
 
 void CloneMCVarsL (PVOID pData, PVOID pUser1, PVOID pUser2, PVOID pUser3)
 {
-  PMCVAR pMCVar = (PMCVAR)pData;
-  PLEVEL plevel = (PLEVEL)pUser1;
-  int *pnMCVars = (int*)pUser2;
+  PMCVAR pMCVar = (PMCVAR) pData;
+  PLEVEL plevel = (PLEVEL) pUser1;
+  int *pnMCVars = (int*) pUser2;
   int n;
   PLEVEL pLower;
   PMCVAR pClone;
@@ -182,7 +234,8 @@ void CloneMCVarsL (PVOID pData, PVOID pUser1, PVOID pUser2, PVOID pUser3)
   ++pMCVar->iDepth;
   for (n = 0; n < plevel->iInstances; ++n) {
     pLower = plevel->pLevels[n];
-    pClone = (PMCVAR) malloc (sizeof (MCVAR));
+    if ((pClone = (PMCVAR) malloc (sizeof (MCVAR))) == NULL)
+      ReportError (NULL, RE_OUTOFMEM | RE_FATAL, "CloneMCVarsL", NULL);
     memcpy (pClone, pMCVar, sizeof (MCVAR));
     pClone->plistDependents = InitList ();
     pLower->rgpMCVars[*pnMCVars] = pClone;
@@ -203,8 +256,8 @@ void CloneMCVarsL (PVOID pData, PVOID pUser1, PVOID pUser2, PVOID pUser3)
 
 void CloseMarkovFiles (PANALYSIS panal)
 {
-  if (panal->gd.pfileGibOut) {
-    fclose (panal->gd.pfileGibOut);
+  if (panal->gd.pfileOut) {
+    fclose (panal->gd.pfileOut);
     printf ("\nWrote results to \"%s\"\n", panal->gd.szGout);
   }
 
@@ -221,8 +274,8 @@ void ConvertLists(PLEVEL plevel, char **args)
   int n;
   PMCVAR pMCVar;
 
-  ListToPMCArray (panal, panal->plistModelVars,
-                  &panal->nModelVars, &panal->rgpModelVars);
+  ListToPMCArray (panal, panal->plistModelVars, &panal->nModelVars, 
+                  &panal->rgpModelVars);
 
   if (plevel->pexpt == NULL)
     ListToPVArray (panal, plevel->plistVars, &plevel->nFixedVars,
@@ -256,7 +309,7 @@ void DoMarkov (PANALYSIS panal)
   int        nThetas, nUpdateAt;
   long       iter = 0;
   long       nIter = pgd->nMaxIter; /* scheduled iterations of the sampler */
-  double     *pdMCVarVals;          /* initial values of thetas */
+  double     *pdMCVarVals = NULL;   /* read in values of thetas */
   double     dLnPrior = 0, dLnData = 0;
 
   InitRandom (panal->dSeed, TRUE);
@@ -280,12 +333,14 @@ void DoMarkov (PANALYSIS panal)
   TraverseLevels (pLevel0, FindMCParents, panal, NULL);
   TraverseLevels (pLevel0, FindMCDependents, panal, NULL);
 
-  /* Now that we have the MC vars right, write the output file header */
-  fprintf (pgd->pfileGibOut, "iter\t");
-  TraverseLevels (pLevel0, WriteHeader, panal, pgd->pfileGibOut, NULL);
-  fprintf (pgd->pfileGibOut,
-           "LnPrior\tLnData\tLnPosterior\n");
-  fflush (pgd->pfileGibOut);
+  /* Now that we have the MC vars right, write the output file header 
+     unless it's just a run for fit checking */
+  if (nIter != 0) {
+    fprintf (pgd->pfileOut, "iter\t");
+    TraverseLevels (pLevel0, WriteHeader, panal, pgd->pfileOut, NULL);
+    fprintf (pgd->pfileOut, "LnPrior\tLnData\tLnPosterior\n");
+    fflush (pgd->pfileOut);
+  }
 
   /* Convert the rest of the lists to arrays */
   TraverseLevels (pLevel0, ConvertLists, panal, NULL);
@@ -301,7 +356,8 @@ void DoMarkov (PANALYSIS panal)
   /* Print out the structure for checking */
   if (panal->bDependents) {
     TraverseLevels (pLevel0, PrintDeps, NULL);
-	  exit(0);
+    CloseMarkovFiles (panal);
+    return;
   }
 
   /* Change the MC vars hvar pointers from pointing to model parameters to 
@@ -315,35 +371,50 @@ void DoMarkov (PANALYSIS panal)
     nThetas = 0;
     TraverseLevels (pLevel0, GetNumberOfMCVars, &nThetas, NULL);
 
-    if ((pdMCVarVals = (double*) malloc (nThetas * sizeof (double))) == NULL)
+    if ((pdMCVarVals = (double *) malloc (nThetas * sizeof (double))) == NULL)
       ReportRunTimeError (panal, RE_OUTOFMEM | RE_FATAL, "DoMarkov");
 
     /* Read the starting values in the order they are printed and
        close the file when finished */
     ReadRestart (pgd->pfileRestart, nThetas, pdMCVarVals, &iter);
-    nThetas = 0;
-    TraverseLevels (pLevel0, SetMCVars, &nThetas, pdMCVarVals, NULL);
+
+    /* Set the dVals of the variables to the values read in */
+    TraverseLevels (pLevel0, SetMCVars, pdMCVarVals, NULL);
+
+    /* If nInitIter is 1 print just the predictions and data and exit */
+    if (pgd->nInitIter == 1) {
+      printf ("\nPrinting data and predictions for the last line "
+              "of the restart file\n");
+      PrintAllExpts (pLevel0, 0, panal, pgd->pfileOut);
+      CloseMarkovFiles (panal);
+      return;
+    }
 
     /* Set the jumping kernel's SD */
-    TraverseLevels (pLevel0, SetKernel, NULL);
+    TraverseLevels (pLevel0, SetKernel, 1, pdMCVarVals, NULL);
 
-    /* The MCVars have been altered by SetKernel, reset them */
-    nThetas = 0;
-    TraverseLevels (pLevel0, SetMCVars, &nThetas, pdMCVarVals, NULL);
+    /* Free the now used array */
     free (pdMCVarVals);
 
-    /* Initilalize the predictions arrays by running all experiments and
-       save the output */
+    /* Initialize the predictions arrays by running all experiments and
+       save the likelihood */
     RunAllExpts (pLevel0, 0, panal, &dLnData);
   }
 
   else { 
+    /* If nInitIter is 1 print an error message and exit */
+    if (pgd->nInitIter == 1) {
+      printf ("\nError: a restart file must be given to print data and"
+              "         predictions - Exiting.\n\n");
+      exit (0);
+    }    
+
     /* Set the jumping kernel's SD */
-    TraverseLevels (pLevel0, SetKernel, NULL);
+    TraverseLevels (pLevel0, SetKernel, 2, pdMCVarVals, NULL);
 
     /* Initialize the thetas by sampling, write them out at the same time */
-    fprintf (pgd->pfileGibOut, "0\t");
-    TraverseLevels (pLevel0, InitMCVars, panal, pgd->pfileGibOut, NULL);
+    fprintf (pgd->pfileOut, "0\t");
+    TraverseLevels (pLevel0, InitMCVars, panal, pgd->pfileOut, NULL);
 
     /* Output log-densities */
     TraverseLevels(pLevel0, CalculateTotals, panal, &dLnPrior, NULL);
@@ -351,9 +422,9 @@ void DoMarkov (PANALYSIS panal)
        save the data likelihood output */
     RunAllExpts(pLevel0, 0, panal, &dLnData);
 
-    fprintf (pgd->pfileGibOut, "%e\t%e\t%e\n", dLnPrior, dLnData,
+    fprintf (pgd->pfileOut, "%e\t%e\t%e\n", dLnPrior, dLnData,
              dLnPrior + dLnData);
-    fflush (pgd->pfileGibOut);
+    fflush (pgd->pfileOut);
   }
 
   /* Save the data likelihoods */
@@ -371,7 +442,7 @@ void DoMarkov (PANALYSIS panal)
     /* Start output to file, eventually */
     if (((iter + 1) % pgd->nPrintFreq == 0) &&
         (iter >= pgd->nMaxIter - pgd->nPrintIter))
-      fprintf (pgd->pfileGibOut, "%ld\t", iter + 1);
+      fprintf (pgd->pfileOut, "%ld\t", iter + 1);
 
     TraverseLevels (pLevel0, SampleThetas, panal, pgd, &iter, &nUpdateAt, 
                     NULL);
@@ -383,9 +454,9 @@ void DoMarkov (PANALYSIS panal)
       TraverseLevels (pLevel0, CalculateTotals, panal, &dLnPrior, NULL);
       dLnData = 0.0;     
       TraverseLevels1 (pLevel0, SumAllExpts, &dLnData, NULL);
-      fprintf (pgd->pfileGibOut, "%e\t%e\t%e\n", dLnPrior, dLnData,
+      fprintf (pgd->pfileOut, "%e\t%e\t%e\n", dLnPrior, dLnData,
                dLnPrior + dLnData);
-      fflush (pgd->pfileGibOut);
+      fflush (pgd->pfileOut);
     }
 
     /* Adjust the update time eventually */
@@ -513,42 +584,6 @@ void InitMCVars(PLEVEL plevel, char **args)
 
 } /* InitMCVars */
 
-#ifdef ndef
-/* Function -------------------------------------------------------------------
-   L2
-
-   Sum of squares for one experiment
-*/
-
-double L2 (PEXPERIMENT pexpt, PANALYSIS panal) 
-{
-  POUTSPEC pos;
-  PMCVAR pMCVar;
-  int n, m, l;
-  double x = dSS = 0.0;
-
-  pos = &pexpt->os;
-  for(n = 0; n < panal->nModelVars; ++n) {
-    if(!IsOutput((pMCVar = panal->rgpModelVars[n])->hvar))
-      continue;
-    for(m = 0; m < pos->nOutputs; ++m) {
-      if(pos->phvar[m] == pMCVar->hvar) {
-        for(l = 0; l < pos->pcOutputTimes[m]; ++l) {
-          if(pos->prgdOutputVals[m][l] != MISSING_VALUE) {
-            x = pos->prgdDataVals[m][l] - pos->prgdOutputVals[m][l];
-            x *= x;
-            dSS += x;
-          }
-          else
-            dSS += BAD_VAL;
-        }
-      }
-    }
-  }
-  return (dSS);
-
-} /* L2 */
-#endif
 
 /* Function -------------------------------------------------------------------
    ListToPMCArray
@@ -626,7 +661,7 @@ void ListToPVArrayL (PVOID pData, PVOID pUser1, PVOID pUser2, PVOID pUser3)
 */
 #define LNSQRT2PI 9.189385332046E-01
 
-double LnDensity (MCVAR *pMCVar, PANALYSIS panal)
+double LnDensity (PMCVAR pMCVar, PANALYSIS panal)
 {
   double dTmp, density;
   double dParm1 = *(pMCVar->pdParm[0]);
@@ -637,28 +672,42 @@ double LnDensity (MCVAR *pMCVar, PANALYSIS panal)
   char str[10];
 
   /* This should take care of all dTheta checking */
-  if (dTheta > dMax || dTheta < dMin)
-    return (BAD_VAL);
+  if (pMCVar->iType == MCV_BINOMIALBETA) {
+    if (dTheta < 0) {
+      printf ("Error: variate out of bounds in LnDensity\n");
+      exit (0);
+    }
+  }
+  else {
+    if (dTheta > dMax || dTheta < dMin) {
+      printf ("Error: variate out of bounds in LnDensity\n");
+      exit (0);
+    }
+  }
 
   switch (pMCVar->iType) {
 
     case MCV_UNIFORM:
-      if (dMax <= dMin)
+      if (dTheta > dParm2 || dTheta < dParm1)
+        return (BAD_VAL);
+      if (dParm2 <= dParm1)
         ReportRunTimeError (panal, RE_BADUNIFORMDIST | RE_FATAL,
                             pMCVar->pszName, "LnDensity");
-      return -log(dMax - dMin);
+      return -log(dParm2 - dParm1);
 
     case MCV_LOGUNIFORM:
-      if (dMax <= dMin)
+      if (dTheta > dParm2 || dTheta < dParm1)
+        return (BAD_VAL);
+      if (dParm2 <= dParm1)
         ReportRunTimeError (panal, RE_BADUNIFORMDIST | RE_FATAL,
                             pMCVar->pszName, "LnDensity");
-      return -log (dTheta * (dMax - dMin));
+      return -log (dTheta * (dParm2 - dParm1));
 
     case MCV_NORMALV: dParm2 = sqrt (dParm2); /* fall thru */
     case MCV_NORMAL:
       return lnDFNormal (dTheta, dParm1, dParm2);
 
-    case MCV_LOGNORMALV: dParm2 = exp(sqrt(log(dParm2))); /* fall thru */
+    case MCV_LOGNORMALV: dParm2 = exp(sqrt(dParm2)); /* fall thru */
     case MCV_LOGNORMAL:
       if (dParm1 <= 0.0) {
         sprintf(str, "%5.2e", dParm1);
@@ -678,7 +727,7 @@ double LnDensity (MCVAR *pMCVar, PANALYSIS panal)
              (CDFNormal ((dMax - dParm1) / dParm2) - 
               CDFNormal ((dMin - dParm1) / dParm2));
 
-    case MCV_TRUNCLOGNORMALV: dParm2 = exp(sqrt(log(dParm2))); /* fall thru */
+    case MCV_TRUNCLOGNORMALV: dParm2 = exp(sqrt(dParm2)); /* fall thru */
     case MCV_TRUNCLOGNORMAL:
       if (dParm1 <= 0.0 ) {
         sprintf(str, "%5.2e", dParm1);
@@ -696,14 +745,7 @@ double LnDensity (MCVAR *pMCVar, PANALYSIS panal)
               CDFNormal (log (dMin / dParm1) / dTmp));
 
     case MCV_BETA:
-      if (dMax <= dMin) {
-        printf ("Error: bad range for beta variate in LnDensity\n");
-        exit (0);
-      }
-      return (dParm1 - 1) * log (dTheta) +
-             (dParm2 - 1) * log (1 - dTheta) +
-             lnGamma (dParm1 + dParm2) -
-             lnGamma (dParm1) - lnGamma(dParm2) - log (dMax - dMin);
+      return lnDFBeta (dTheta, dParm1, dParm2, dMin, dMax);
 
     case MCV_CHI2:
       dTmp = 0.5 * dParm1;
@@ -711,7 +753,7 @@ double LnDensity (MCVAR *pMCVar, PANALYSIS panal)
              dTmp * (-6.9314718056E-01) - lnGamma (dTmp);
 
     case MCV_BINOMIAL:
-      if (dParm1 <= 0) {
+      if ((dParm1 < 0) || (dParm1 > 1)) {
         printf ("Error: bad p for binomial variate in LnDensity\n");
         exit (0);
       }
@@ -721,9 +763,23 @@ double LnDensity (MCVAR *pMCVar, PANALYSIS panal)
       }
       /* log binomial coefficient n! / (x!(n-x)!) */
       dTmp = lnGamma (dParm2 + 1) - lnGamma (dTheta + 1) - 
-             lnGamma (dParm2 - dTheta + 1); 
-      return dTheta * log (dParm1) + (dParm2 - dTheta) * log (1 - dParm1) +
-             dTmp;
+             lnGamma (dParm2 - dTheta + 1);
+
+      if (dParm1 == 0) { /* revised FB 18/07/97 */
+        if (dTheta != 0)
+          return (-1e10); /* should be -INF */
+      }
+      else
+        dTmp = dTmp + dTheta * log (dParm1);
+      
+      if (dParm1 == 1) { /* revised FB 18/07/97 */
+        if ((dParm2 - dTheta) == 0) 
+          return dTmp; /* because log(0^0) is 0 */
+        else
+          return (-1e10); /* should be -INF */
+      }
+      else
+        return dTmp + (dParm2 - dTheta) * log (1 - dParm1);
 
     case MCV_PIECEWISE:
       density = 2 / (dMax + dParm2 - dParm1 - dMin);
@@ -739,7 +795,8 @@ double LnDensity (MCVAR *pMCVar, PANALYSIS panal)
 
     case MCV_EXPONENTIAL:
       if (dParm1 <= 0) {
-        printf ("Error: bad scale for exponential variate in LnDensity\n");
+        printf ("Error: negative or null inverse scale (%g) for exponential "
+                "variate in LnDensity\n", dParm1);
         exit (0);
       }
       return -dTheta * dParm1 * log (dParm1);
@@ -767,6 +824,37 @@ double LnDensity (MCVAR *pMCVar, PANALYSIS panal)
       }
       return dTheta * log (dParm1) - dParm1 - lnGamma (dTheta + 1);
 
+    case MCV_BINOMIALBETA:
+      if (dParm1 < 0) {
+        printf ("Error: bad expectation for BinomialBeta variate "
+                "in LnDensity\n");
+        exit (0);
+      }
+      if (dParm2 <= 0) {
+        printf ("Error: bad alpha for BinomialBeta variate in LnDensity\n");
+        exit (0);
+      }
+      if (dMin <= 0) {
+        printf ("Error: bad beta for BinomialBeta variate in LnDensity\n");
+        exit (0);
+      }
+      dTmp = floor (0.5 + dParm1 + dParm1 * dMin / dParm2); /* this is N */
+      if (dTheta > dTmp)
+        return (-1e10);
+      else {
+        if ((dParm2 == 0.5) && (dMin == 0.5)) 
+          dTmp = lnGamma (0.5 + dTheta) + 
+                 lnGamma (0.5 + dTmp - dTheta) -
+                 lnGamma (dTheta + 1) - lnGamma (dTmp - dTheta + 1);
+        else
+          dTmp = lnGamma (dParm2 + dMin) + lnGamma (dTmp + 1) +
+                 lnGamma (dParm2 + dTheta) +
+                 lnGamma (dMin + dTmp - dTheta) -
+                 lnGamma (dTheta + 1) - lnGamma (dTmp - dTheta + 1) -
+                 lnGamma (dParm2) - lnGamma(dMin) - 
+                 lnGamma (dParm2 + dMin + dTmp);
+        return dTmp;
+      }
 
     default:
       ReportRunTimeError(panal, RE_UNKNOWNDIST | RE_FATAL, "LnDensity");
@@ -782,7 +870,8 @@ double LnDensity (MCVAR *pMCVar, PANALYSIS panal)
 /* Function -------------------------------------------------------------------
    LnLike
 
-   returns the log-likelihood of a parameter
+   returns the log-likelihood of the dependents given the parameter specified
+   by pMCVar
 */
 
 double LnLike (PMCVAR pMCVar, PANALYSIS panal) 
@@ -813,8 +902,10 @@ double LnLikeData (PEXPERIMENT pexpt, PANALYSIS panal) {
   PMCVAR pMCVar;
   int n, m, l;
   double dLnLike = 0.0;
+  double dTmp;
 
   pos = &pexpt->os;
+
   for (n = 0; n < panal->nModelVars; ++n) {
     pMCVar = panal->rgpModelVars[n];
 
@@ -836,7 +927,11 @@ double LnLikeData (PEXPERIMENT pexpt, PANALYSIS panal) {
               else
                 pMCVar->dParm[1] = pos->prgdOutputVals[pMCVar->iParmIndex][l];
             
-            dLnLike += LnDensity (pMCVar, panal);
+            dTmp = LnDensity (pMCVar, panal);
+            if (dTmp == (BAD_VAL))
+              return (BAD_VAL);
+            else
+              dLnLike = dLnLike + dTmp;
           }
           else
             ReportRunTimeError (panal, RE_BADMODEL | RE_FATAL, "LnLikeData");
@@ -865,7 +960,7 @@ void OpenMarkovFiles (PANALYSIS panal)
 
   /* Default if none given */
   else if (!(panal->gd.szGout))
-    panal->gd.szGout = "simgb.out";
+    panal->gd.szGout = "MCMC.default.out";
 
   /* eventually open the restart file before crushing the output file */
   if (panal->gd.szGrestart)
@@ -874,14 +969,89 @@ void OpenMarkovFiles (PANALYSIS panal)
       ReportRunTimeError(panal, RE_FATAL | RE_CANNOTOPEN,
                          panal->gd.szGrestart, "OpenMarkovFiles");
 
-  if (!(panal->gd.pfileGibOut)
-      && !(panal->gd.pfileGibOut = fopen (panal->gd.szGout, "w")))
+  if (!(panal->gd.pfileOut)
+      && !(panal->gd.pfileOut = fopen (panal->gd.szGout, "w")))
     ReportRunTimeError(panal, RE_FATAL | RE_CANNOTOPEN,
                        panal->gd.szGout, "OpenMarkovFiles");
 
 } /* OpenMarkovFiles */
 
 
+/* Function -------------------------------------------------------------------
+   PrintAllExpts
+
+   Run all experiments at level below `plevel'
+*/
+
+void PrintAllExpts (PLEVEL plevel, int nVar, PANALYSIS panal, PFILE pOutFile)
+{
+  int n;
+
+  for (n = 0; n < plevel->iInstances; ++n)
+    TraverseLevels1 (plevel->pLevels[n], PrintExpt, panal, pOutFile, NULL);
+
+} /* PrintAllExpts */
+
+
+/* Function -------------------------------------------------------------------
+   PrintExpt
+
+   If `plevel' has no experiments, modify the variables using its two
+   lists; if it has experiments, run them. Return 1 on success and 0 if
+   failure
+
+   Called from TraverseLevels1
+*/
+
+int PrintExpt (PLEVEL plevel, char **args)
+{
+  PANALYSIS   panal = (PANALYSIS)args[0];
+  PFILE       pOutFile = (PFILE)args[1];
+  int         l, m, n;
+  PEXPERIMENT pExpt = plevel->pexpt;
+  POUTSPEC    pos;
+  static int  printed_head = 0;
+
+  if (!printed_head) {
+    fprintf (pOutFile, "Time\tData\tPrediction\n");
+    printed_head = 1;
+  }
+
+  /* Set level sequence */
+  panal->pCurrentLevel[plevel->iDepth] = plevel;
+
+  if (pExpt != NULL) {
+    InitModel ();
+
+    /* Set the model vars that have been sampled in this experiment and 
+       above levels */
+    for (n = 0; n <= plevel->iDepth; ++n) {
+      SetModelVars (panal->pCurrentLevel[n]);
+      SetFixedVars (panal->pCurrentLevel[n]);
+    }
+
+    if (!DoOneExperiment (pExpt)) {
+      /* Error */
+      printf ("Warning: DoOneExperiment failed\n");
+      return 0;
+    }
+    else {
+      pos = &pExpt->os;
+      for (m = 0; m < pos->nOutputs; ++m) {
+        for (l = 0; l < pos->pcOutputTimes[m]; ++l) {
+          fprintf (pOutFile, "%g\t%g\t%g\n", pos->prgdOutputTimes[m][l], 
+                   pos->prgdDataVals[m][l], pos->prgdOutputVals[m][l]);
+        }
+      }
+    }
+
+  } /* if */
+
+  return (1);
+
+} /* PrintExpt */
+
+
 /* Function -------------------------------------------------------------------
    PrintDeps
 
@@ -901,21 +1071,21 @@ void PrintDeps (PLEVEL plevel, char **args)
   for (n = 0; n < plevel->nMCVars; ++n) {
     pMCVar = plevel->rgpMCVars[n];
 
-    fprintf(stderr, "Variable %s (%d) [%x]\n",
-            pMCVar->pszName, pMCVar->iDepth, (unsigned int) pMCVar);
+    fprintf(stderr, "Variable %s (%d) [%lx]\n",
+            pMCVar->pszName, pMCVar->iDepth, (unsigned long) pMCVar);
 
     for (m = 0; m < 4; ++m)
       if (pMCVar->pMCVParent[m] != NULL)
-        fprintf (stderr, "  Parent %d: %s (%d) [%x]\n", m,
+        fprintf (stderr, "  Parent %d: %s (%d) [%lx]\n", m,
                  pMCVar->pMCVParent[m]->pszName,
                  pMCVar->pMCVParent[m]->iDepth, 
-                 (unsigned int) pMCVar->pMCVParent[m]);
+                 (unsigned long) pMCVar->pMCVParent[m]);
 
     for (m = 0; m < pMCVar->nDependents; ++m)
-      fprintf (stderr, "  Dependent: %s (%d) [%x]\n",
+      fprintf (stderr, "  Dependent: %s (%d) [%lx]\n",
                pMCVar->rgpDependents[m]->pszName,
                pMCVar->rgpDependents[m]->iDepth, 
-               (unsigned int) pMCVar->rgpDependents[m]);
+               (unsigned long) pMCVar->rgpDependents[m]);
 
     if (pMCVar->bExptIsDep)
       fprintf(stderr, "  This variable influences experiments directly\n");
@@ -934,8 +1104,7 @@ void PrintDeps (PLEVEL plevel, char **args)
 void ReadRestart (FILE *pfileRestart, long nThetas,
                   double *pdTheta, long *pIter) 
 {
-  int n;
-  long lDummy;
+  register int n;
 
   *pIter = -1;
 
@@ -944,12 +1113,12 @@ void ReadRestart (FILE *pfileRestart, long nThetas,
   fscanf (pfileRestart,  "%*[^\n]");  getc(pfileRestart);
 
   /* as long as we have not reached the end of the file we keep reading lines
-     and overwriting the thetas and sigmas, they keep only their
-     last value. We also keep incrementing the global iteration counter
-     iter:
+     and overwriting the thetas, they keep only their last value. 
+     We throw away first field, and we keep incrementing the global iteration 
+     counter iter:
    */
   while (!(feof (pfileRestart) ||
-          (fscanf (pfileRestart, "%ld", &lDummy) == EOF))) {
+          (fscanf (pfileRestart, "%*s") == EOF))) {
     for (n = 0; n < nThetas; ++n) {
       if (fscanf(pfileRestart, "%lg", &(pdTheta[n])) == EOF) {
         printf ("Error: incorrect length for restart file - Exiting\n");
@@ -996,9 +1165,6 @@ int RestoreLikelihoods (PLEVEL plevel, char **args)
    RunAllExpts
 
    Run all experiments at level below `plevel'
-
-Do we need this one really ? Who calls it ?
-
 */
 
 void RunAllExpts (PLEVEL plevel, int nVar, PANALYSIS panal, double *pdLnData)
@@ -1064,18 +1230,22 @@ int RunExpt (PLEVEL plevel, char **args)
 
 double SampleTheta (PMCVAR pMCVar) {
 
-  double dMin = *(pMCVar->pdParm[2]);
-  double dMax = *(pMCVar->pdParm[3]);
-  double dpropSD;
-
-  if (pMCVar->iType == MCV_LOGNORMAL || 
-      pMCVar->iType == MCV_TRUNCLOGNORMAL ||
-      pMCVar->iType == MCV_LOGUNIFORM)
-    dpropSD = /* log */ (pMCVar->dKernelSD);
-  else
-    dpropSD = pMCVar->dKernelSD;
-
-  return TruncNormalRandom (pMCVar->dVal, dpropSD, dMin, dMax);
+  /* if the parameter has a discrete distribution, round it - FB 12/06/97 */
+  if (pMCVar->iType == MCV_BINOMIAL || pMCVar->iType == MCV_POISSON ) {
+   return floor (0.5 + TruncNormalRandom (pMCVar->dVal, pMCVar->dKernelSD,
+                                          *(pMCVar->pdParm[2]), 
+                                          *(pMCVar->pdParm[3])));
+  }
+  else { /* FB fixed the uniform case - FB 30/06/97 */
+    if (pMCVar->iType == MCV_UNIFORM || pMCVar->iType == MCV_LOGUNIFORM ) {
+      return TruncNormalRandom (pMCVar->dVal, pMCVar->dKernelSD, 
+                                *(pMCVar->pdParm[0]), *(pMCVar->pdParm[1]));
+    }
+    else {
+      return TruncNormalRandom (pMCVar->dVal, pMCVar->dKernelSD, 
+                                *(pMCVar->pdParm[2]), *(pMCVar->pdParm[3]));
+    }
+  }
 
 } /* SampleTheta */
 
@@ -1097,9 +1267,10 @@ void SampleThetas (PLEVEL plevel, char **args)
 
   double dLnPrior, dLnLike, dLnData;
   double dLnPriorNew, dLnLikeNew, dLnDataNew;
-  double dTheta, dJumps;
+  double dTheta, dJumps, dSum, dVar;
   PMCVAR pMCVar;
-  int n;
+  static MCVAR tmpMCVar;
+  int i, n;
 
   /* For all MC vars at this level */
   for (n = 0; n < plevel->nMCVars; ++n) {
@@ -1110,76 +1281,105 @@ void SampleThetas (PLEVEL plevel, char **args)
     if (pMCVar->bIsFixed)
       goto WriteIt;
 
-    /* Compute prior and likelihood */
-    dLnPrior = LnDensity (pMCVar, panal);
-    dLnLike = LnLike (pMCVar, panal);
+    if (pMCVar->bGibbs) {
+      /* we can try Gibbs sampling if all dependent are distributed 
+         normally around pMCVar. Only the normal case is currently
+         implemented */
 
-    dLnData = 0.0;
+      /* form the sum of dependents */
+      dSum = 0;
+      for (i = 0; i < pMCVar->nDependents; ++i) 
+        dSum = dSum + pMCVar->rgpDependents[i]->dVal;
 
-    /* If data are dependent compute the data likelihood */
-    if (pMCVar->bExptIsDep) {
-      /* Form the likelihood of all experiments at this level or beneath.*/
-      TraverseLevels1 (plevel, SumAllExpts, &dLnData, NULL);
-    }
-
-    /* Save current value */
-    dTheta = pMCVar->dVal;
-
-    /* Adjust the jumping kernel SD, depending on acceptance rates, 
-       make sure it does not exceed DBL_MAX */
-    if ((*pnIter % *pnUpdateAt == 0) && (*pnIter > 0)) {
-      if ((dJumps = (double) pMCVar->lJumps / (double) *pnUpdateAt) > 0.3)
-        pMCVar->dKernelSD = pMCVar->dKernelSD * 2;
-      else 
-        if ((dJumps < 0.15) && (pMCVar->dKernelSD < (DBL_MAX / 2)))
-          pMCVar->dKernelSD = pMCVar->dKernelSD * 0.5;
-
-      pMCVar->lJumps = 0;
-    }
-
-    /* Sample a new value */
-    pMCVar->dVal = SampleTheta (pMCVar);
-
-    /* recompute prior and likelihood */
-    dLnPriorNew = LnDensity (pMCVar, panal);
-    dLnLikeNew = LnLike (pMCVar, panal);
-
-    dLnDataNew = 0.0;
-
-    /* If data are dependent compute the data likelihood */
-    if (pMCVar->bExptIsDep) {
-      /* Run all experiments at this level or beneath.
-         We should in fact run only the dependent experiments ! */
-      
-      if (!TraverseLevels1 (plevel, RunExpt, panal, &dLnDataNew, NULL)) {
-        /* If running experiments fails, do not jump */
-        pMCVar->dVal = dTheta;
-        TraverseLevels1 (plevel, RestoreLikelihoods, NULL);
-        goto WriteIt;
+      if (pMCVar->iType == MCV_NORMAL) { /* we have to form the variance */
+        /* all dependents are supposed to be distributed with the same 
+           variance, so use the first one */
+        dVar = pow(*(pMCVar->rgpDependents[0]->pdParm[1]), 2);
       }
-    }      
+      else dVar = *(pMCVar->rgpDependents[0]->pdParm[1]);
 
-    /* Test the results and act accordingly */
-    if (!Test (pMCVar->bExptIsDep, *pnIter, pgd->nInitIter,
-               dLnPrior, dLnPriorNew, dLnLike, dLnLikeNew,
-               dLnData, dLnDataNew)) {
-      pMCVar->dVal = dTheta;
-
-      if(pMCVar->bExptIsDep)
-        TraverseLevels1 (plevel, RestoreLikelihoods, NULL);
-    }
-    else {
-      ++pMCVar->lJumps;
-
-      if(pMCVar->bExptIsDep)
-        TraverseLevels1 (plevel, SaveLikelihoods, NULL);
+      tmpMCVar.dParm[0] = (dVar * *(pMCVar->pdParm[0]) + 
+                            *(pMCVar->pdParm[1]) * dSum) / 
+                           (*(pMCVar->pdParm[1]) * pMCVar->nDependents + dVar);
+      tmpMCVar.dParm[1] = 0;
+      CalculateOneMCParm (&tmpMCVar);
     }
+    else { /* Metropolis-Hastings */
 
+      /* Compute prior and likelihood */
+      dLnPrior = LnDensity (pMCVar, panal);
+      dLnLike = LnLike (pMCVar, panal);
+  
+      dLnData = 0.0;
+  
+      /* If data are dependent compute the data likelihood */
+      if (pMCVar->bExptIsDep) {
+        /* Form the likelihood of all experiments at this level or beneath.*/
+        TraverseLevels1 (plevel, SumAllExpts, &dLnData, NULL);
+      }
+  
+      /* Save current value */
+      dTheta = pMCVar->dVal;
+  
+      /* Adjust the jumping kernel SD, depending on acceptance rates, 
+         make sure it does not exceed DBL_MAX or a third of the range */
+      if ((*pnIter % *pnUpdateAt == 0) && (*pnIter > 0)) {
+        if (((dJumps = (double)pMCVar->lJumps / (double)*pnUpdateAt) > 0.3) &&
+            (pMCVar->dKernelSD < ((*(pMCVar->pdParm[3]) / 6) - 
+                                  (*(pMCVar->pdParm[2]) / 6))) &&
+            (pMCVar->dKernelSD < (DBL_MAX / 2)))
+          pMCVar->dKernelSD = pMCVar->dKernelSD * 2;
+        else 
+          if (dJumps < 0.15)
+            pMCVar->dKernelSD = pMCVar->dKernelSD * 0.5;
+  
+        pMCVar->lJumps = 0;
+      }
+  
+      /* Sample a new value */
+      pMCVar->dVal = SampleTheta (pMCVar);
+  
+      /* recompute prior and likelihood */
+      dLnPriorNew = LnDensity (pMCVar, panal);
+      dLnLikeNew = LnLike (pMCVar, panal);
+  
+      dLnDataNew = 0.0;
+  
+      /* If data are dependent compute the data likelihood */
+      if (pMCVar->bExptIsDep) {
+        /* Run all experiments at this level or beneath.
+           We should in fact run only the dependent experiments ! */
+        
+        if (!TraverseLevels1 (plevel, RunExpt, panal, &dLnDataNew, NULL)) {
+          /* If running experiments fails, do not jump */
+          pMCVar->dVal = dTheta;
+          TraverseLevels1 (plevel, RestoreLikelihoods, NULL);
+          goto WriteIt;
+        }
+      }      
+  
+      /* Test the results and act accordingly */
+      if (!Test (pMCVar->bExptIsDep, *pnIter, pgd->nInitIter,
+                 dLnPrior, dLnPriorNew, dLnLike, dLnLikeNew,
+                 dLnData, dLnDataNew)) {
+        pMCVar->dVal = dTheta;
+  
+        if(pMCVar->bExptIsDep)
+          TraverseLevels1 (plevel, RestoreLikelihoods, NULL);
+      }
+      else {
+        ++pMCVar->lJumps;
+  
+        if(pMCVar->bExptIsDep)
+          TraverseLevels1 (plevel, SaveLikelihoods, NULL);
+      }
+      } /* else (Metrolopolis-Hastings */
+  
 WriteIt: /* Write the MC var value to output file */
 
     if (((*pnIter+1) % pgd->nPrintFreq == 0) &&
         (*pnIter >= pgd->nMaxIter - pgd->nPrintIter)) {
-      fprintf(pgd->pfileGibOut, "%5g\t", pMCVar->dVal);
+      fprintf(pgd->pfileOut, "%5g\t", pMCVar->dVal);
     }
   }
 
@@ -1235,26 +1435,56 @@ void SetFixedVars (PLEVEL plevel)
 
 void SetKernel (PLEVEL plevel, char **args)
 {
+  long useMCVarVals = (long) args[0]; /* 1 if dVal is to be restored, else 2 */
+  double *pdMCVarVals = (double *) args[1];
   double dMin, dMax, dTmp;
   long   n, m;
+  static long nThetas;
+  PMCVAR pMCVar;
 
   /* set the jumping kernel's SD: sample 4 variates and take the range */
-  for (n = 0; n < plevel->nMCVars; ++n)
+  for (n = 0; n < plevel->nMCVars; ++n) {
+
     if ( !(plevel->rgpMCVars[n]->bIsFixed)) {
-      CalculateOneMCParm (plevel->rgpMCVars[n]);
-      dMin = dMax = plevel->rgpMCVars[n]->dVal;
-      for (m = 0; m < 2; ++m) {
-        CalculateOneMCParm (plevel->rgpMCVars[n]);
-        dTmp = plevel->rgpMCVars[n]->dVal;
+
+      pMCVar = plevel->rgpMCVars[n];
+      CalculateOneMCParm (pMCVar);
+
+      dMin = dMax = pMCVar->dVal;
+      for (m = 0; m < 3; ++m) {
+        CalculateOneMCParm (pMCVar);
+        dTmp = pMCVar->dVal;
         if (dMin >= dTmp) dMin = dTmp;
         else if (dMax < dTmp) dMax = dTmp;
       }
-      /* The range can be at most 2 DBL_MAX, so we halve it in order to never
-         exceed computational bounds, and we never form the dangerous complete
-         range */
-      plevel->rgpMCVars[n]->dKernelSD = (0.5 * dMax) - (0.5 * dMin);
+
+      /* The range can be at most 2 DBL_MAX, so we halve it in that case 
+         to never exceed computational bounds, and we never form the
+         dangerous complete range */
+      if ((*(pMCVar->pdParm[2]) == -DBL_MAX) || 
+          (*(pMCVar->pdParm[3]) == DBL_MAX))
+        pMCVar->dKernelSD = (0.5 * dMax) - (0.5 * dMin);
+      else
+        pMCVar->dKernelSD = dMax - dMin;
+
+      /* but if the range is zero (can happens for discrete variables, then 
+         set it to half the prior range - FB 12/06/97 */
+      if (pMCVar->dKernelSD == 0) {
+        if ((*(pMCVar->pdParm[2]) == -DBL_MAX) || 
+            (*(pMCVar->pdParm[3]) == DBL_MAX))
+          pMCVar->dKernelSD = (0.5 * *(pMCVar->pdParm[3]) - 
+                               0.5 * *(pMCVar->pdParm[2]));
+        else 
+          pMCVar->dKernelSD = *(pMCVar->pdParm[3]) - *(pMCVar->pdParm[2]);
+      }   
     }
 
+    /* restore the value of the variable - FB 02/07/97 */
+    if (useMCVarVals == 1)
+      plevel->rgpMCVars[n]->dVal = pdMCVarVals[nThetas++];
+
+  }
+
 } /* SetKernel */
 
 
@@ -1289,12 +1519,12 @@ void SetModelVars(PLEVEL plevel)
 
 void SetMCVars (PLEVEL plevel, char **args)
 {
-  long   *pnThetas = (long*) args[0];
-  double *pdMCVarVals = (double*) args[1];
+  double *pdMCVarVals = (double*) args[0];
+  static long nThetas = 0;
   long   n;
 
   for (n = 0; n < plevel->nMCVars; ++n)
-    plevel->rgpMCVars[n]->dVal = pdMCVarVals[(*pnThetas)++];
+    plevel->rgpMCVars[n]->dVal = pdMCVarVals[nThetas++];
 
 } /* SetMCVars */
 
@@ -1361,23 +1591,17 @@ BOOL Test (BOOL bExptIsDep, long iter, long initIter, double dLnPrior,
            double dLnPriorNew, double dLnLike, double dLnLikeNew,
            double dLnData, double dLnDataNew) {
 
-  double dPjump, dTmp;
+  double dPjump;
 
   if (dLnPriorNew == BAD_VAL || dLnLikeNew == BAD_VAL)
     return FALSE;
 
   dPjump = dLnPriorNew - dLnPrior + dLnLikeNew - dLnLike;
+
   if (bExptIsDep)
     dPjump += dLnDataNew - dLnData;
 
-  /* Disabled: If we are doing an initialization iteration we jump each time
-     only if we improve (i.e. if dPjump > 1) */
-/*  if (iter >= initIter) */
-    dTmp = log(Randoms());
-/*  else
-    dTmp = 0;
-*/
-  return ((BOOL) (dTmp <= dPjump));
+  return ((BOOL) (log(Randoms()) <= dPjump));
 
 } /* Test */
 
@@ -1486,7 +1710,7 @@ void WriteHeader (PLEVEL plevel, char **args)
    Write the values of MC vars for one level to output file
 */
 
-void WriteMCVars (PLEVEL plevel, PANALYSIS panal, FILE* pOutFile)
+void WriteMCVars (PLEVEL plevel, PANALYSIS panal, PFILE pOutFile)
 {
   int n;
   PMCVAR pMCVar;
