@@ -4,11 +4,33 @@
    
    Copyright (c) 1993.  Don Maszle, Frederic Bois.  All rights reserved.
 
+   This program is free software; you can redistribute it and/or
+   modify it under the terms of the GNU General Public License
+   as published by the Free Software Foundation; either version 2
+   of the License, or (at your option) any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+
+   contact
+     Frdric Bois / Don Maszle
+     BEHS, School of Public Health
+     University of California at Berkeley
+     Berkeley, CA 94720
+
+     fbois@diana.lbl.gov
+
    -- Revisions -----
      Logfile:  SCCS/s.mh.h
-    Revision:  1.8
-        Date:  20 Jan 1997
-     Modtime:  02:33:49
+    Revision:  1.10
+        Date:  18 Jul 1997
+     Modtime:  09:16:21
       Author:  @a
    -- SCCS  ---------
 
@@ -43,8 +65,6 @@
 #define BAD_VAL               1.0E+100
 #define MISSING_VALUE         (-DBL_MAX)
 #define INPUT_MISSING_VALUE   -1
-#define PI                    3.1415926535897932384626433
-#define INV_SQRT_2PI          0.398942280401433
 
 /* ----------------------------------------------------------------------------
    Typedefs
@@ -88,11 +108,14 @@ void ListToPMCArrayL (PVOID pData, PVOID pUser1, PVOID pUser2, PVOID pUser3);
 void ListToPVArray (PANALYSIS panal, PLIST plist,
                     long *nFixedVars, PVARMOD **rgpFixedVars);
 void ListToPVArrayL (PVOID pData, PVOID pUser1, PVOID pUser2, PVOID pUser3);
-double LnDensity (PMCVAR pmcvar, PANALYSIS panal);
+double LnDensity (PMCVAR pMCVar, PANALYSIS panal);
 double LnLike (PMCVAR pMCVar, PANALYSIS panal);
 double LnLikeData (PEXPERIMENT pexpt, PANALYSIS panal);
 void   OpenMarkovFiles (PANALYSIS panal);
+void   PrintAllExpts (PLEVEL plevel, int nVar, PANALYSIS panal,
+                      PFILE pOutFile);
 void   PrintDeps (PLEVEL plevel, char **args);
+int    PrintExpt (PLEVEL plevel, char **args);
 void   ReadRestart (FILE *pfileRestart, long nThetas,
                     double *pdTheta, long *pIter);
 int    RestoreLikelihoods (PLEVEL plevel, char **args);
@@ -115,7 +138,7 @@ void TraverseLevels (PLEVEL plevel,
 int  TraverseLevels1 (PLEVEL plevel,
                       int (*routinePtr)(PLEVEL plevel, char **args), ...);
 void WriteHeader (PLEVEL plevel, char **args);
-void WriteMCVars (PLEVEL plevel, PANALYSIS panal, FILE *pOutFile);
+void WriteMCVars (PLEVEL plevel, PANALYSIS panal, PFILE pOutFile);
 
 #endif /* _MH_H_ */
 
