@@ -5,11 +5,33 @@
 
    Copyright (c) 1993.  Don Maszle, Frederic Bois.  All rights reserved.
 
+   This program is free software; you can redistribute it and/or
+   modify it under the terms of the GNU General Public License
+   as published by the Free Software Foundation; either version 2
+   of the License, or (at your option) any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+
+   contact
+     Frdric Bois / Don Maszle
+     BEHS, School of Public Health
+     University of California at Berkeley
+     Berkeley, CA 94720
+
+     fbois@diana.lbl.gov
+
    -- Revisions -----
-     Logfile:  %F%
-    Revision:  %I%
-        Date:  %G%
-     Modtime:  %U%
+     Logfile:  SCCS/s.modelu.c
+    Revision:  1.23
+        Date:  29 Jul 1997
+     Modtime:  08:08:46
       Author:  @a
    -- SCCS  ---------
 
@@ -65,11 +87,6 @@ extern VMMAPSTRCT vrgvmGlo[];     /* Global variable map */
 BOOL vbModelReinitd = FALSE;    /* Model has been initialized */
 
 
-#ifndef MIN
-#define MIN(n1, n2)    ((n1) < (n2) ? (n1) : (n2))
-#endif
-
-
 /* ----------------------------------------------------------------------------
    FixupDependentInputs
 
@@ -115,8 +132,9 @@ void FixupDependentInputs()
     if (vrgInputs[i].hDecay)
       vrgInputs[i].dDecay = GetVarValue (vrgInputs[i].hDecay);
 
-    /* NDoses dependencies */
-    if (vrgInputs[i].iType == IFN_NDOSES)  /* FB 12/10/96 */
+    /* NDoses or Spikes dependencies */
+    if ((vrgInputs[i].iType == IFN_NDOSES) || /* FB 12/10/96 */
+        (vrgInputs[i].iType == IFN_SPIKES))   /* FB 11/06/97 */
       for (j = 0; j < vrgInputs[i].nDoses; j++) {
         if (vrgInputs[i].rghMags[j])
           vrgInputs[i].rgMags[j] = GetVarValue (vrgInputs[i].rghMags[j]);
@@ -126,9 +144,10 @@ void FixupDependentInputs()
         /* check that the times are in order */
         if (j > 0) {
           if (vrgInputs[i].rgT0s[j] <= vrgInputs[i].rgT0s[j-1]) {
-            printf ("\nError: unordered pair of times (%g, %g) in NDoses "
-                    "statement - Exiting\n", vrgInputs[i].rgT0s[j-1], 
-                    vrgInputs[i].rgT0s[j]);
+            printf ("\nError: unordered pair of times (%g, %g) in %s "
+                    "statement - Exiting\n", 
+                    vrgInputs[i].rgT0s[j-1], vrgInputs[i].rgT0s[j],
+                    (vrgInputs[i].iType == IFN_NDOSES ? "NDoses" : "Spikes"));
             exit (0);
           }
         }
@@ -172,7 +191,7 @@ void FixupDependentInputs()
    GetStartPeriods
 
    Calculates the start of the current period for each input.  This
-   is 0.0 if *pdTime is 0.0, otherwise some expensive we have
+   is 0.0 if *pdTime is 0.0, otherwise we have
    to figure out where we are.  Note that this is only called
    once at the begining of a simulation and for most simulations, the
    start time will be 0.
@@ -187,15 +206,15 @@ void GetStartPeriods (PDOUBLE pdTime)
   if (*pdTime == 0.0)
     for (i = 0; i < vnInputs; i++) {
       vrgInputs[i].dTStartPeriod = 0.0;
-      if (vrgInputs[i].iType == IFN_NDOSES
-          || vrgInputs[i].iType == IFN_SPIKES)
+      if (vrgInputs[i].iType == IFN_NDOSES ||
+          vrgInputs[i].iType == IFN_SPIKES)
         vrgInputs[i].iDoseCur = 0;
     } /* for */
 
   else
     for (i = 0; i < vnInputs; i++) {
-      if (vrgInputs[i].iType == IFN_NDOSES
-          || vrgInputs[i].iType == IFN_SPIKES) {
+      if (vrgInputs[i].iType == IFN_NDOSES || 
+          vrgInputs[i].iType == IFN_SPIKES) {
         for (vrgInputs[i].iDoseCur = 0;
              vrgInputs[i].iDoseCur < vrgInputs[i].nDoses;
              vrgInputs[i].iDoseCur++)
@@ -266,7 +285,6 @@ void UpdateNDoses (PIFN pifn, PDOUBLE pdTnext, PDOUBLE pdTime)
 
    Update a Spikes type input.
 */
-
 BOOL UpdateSpikes (PIFN pifn, PDOUBLE pdTnext, PDOUBLE pdTime)
 {
   register double *rgT0s = pifn->rgT0s; /* The schedule times */
@@ -278,13 +296,14 @@ BOOL UpdateSpikes (PIFN pifn, PDOUBLE pdTnext, PDOUBLE pdTime)
       *pdTnext = rgT0s[j]; /* Time of upcoming spike */
 
     else if (*pdTime == rgT0s[j]) {
-      pifn->bOn = TRUE; /* At a spike--return the time */
+      /* At a spike, return the time of the following spike */
+      pifn->bOn = TRUE;
       if (j + 1 < pifn->nDoses)
-        *pdTnext = rgT0s[j + 1]; /* of the following spike */
-    } /* if */
+        *pdTnext = rgT0s[j + 1];
+    } /* else if */
 
     else /* Oops */
-      printf ("UpdateSpikes: Discontinuity was passed over\n");
+      printf ("\nUpdateSpikes: Discontinuity was passed over\n");
   } /* if */
 
   return (pifn->bOn); /* Flag for post-update of spikes */
@@ -292,6 +311,11 @@ BOOL UpdateSpikes (PIFN pifn, PDOUBLE pdTnext, PDOUBLE pdTime)
 } /* UpdateSpikes */
 
 
+/* -----------------------------------------------------------------------------
+   PostUpdateSpikes
+
+   Reupdate a Spikes type input.
+*/
 void PostUpdateSpikes (PDOUBLE pdTime)
 {
   static PDOUBLE rgDerivTemp; /* Derivatives are NOT affected */
@@ -318,7 +342,6 @@ void PostUpdateSpikes (PDOUBLE pdTime)
 
    Update default input types.  PerRate, PerDose, PerExp.
 */
-
 void UpdateDefaultInput (PIFN pifn, PDOUBLE pdTnext, PDOUBLE pdTime)
 {
   *pdTnext = pifn->dTStartPeriod + pifn->dT0;
@@ -364,7 +387,6 @@ void UpdateDefaultInput (PIFN pifn, PDOUBLE pdTnext, PDOUBLE pdTime)
    UpdateInputs() assumes that dTime > 0.0, always, and further,
    that time proceeds positively.
 */
-
 void UpdateInputs (PDOUBLE pdTime, PDOUBLE pdNextTransTime)
 {
   double dT; /* New time to try */
@@ -375,8 +397,7 @@ void UpdateInputs (PDOUBLE pdTime, PDOUBLE pdNextTransTime)
     FixupDependentInputs ();    /* Fixup input fields dependent on vars */
     ScaleModel();               /* ...then scale the model parms */
     GetStartPeriods (pdTime);   /* Initialize the current period */
-    vbModelReinitd = FALSE;
-  } /* if */
+  }
 
   dT = *pdNextTransTime = DBL_MAX;
   for (i = 0; i < vnInputs; i++) {
@@ -390,6 +411,10 @@ void UpdateInputs (PDOUBLE pdTime, PDOUBLE pdNextTransTime)
         break;
 
       case IFN_SPIKES:
+        /* FB 11/6/97: advance to next spike if we are not at initial time */
+        if (!vbModelReinitd) 
+          vrgInputs[i].iDoseCur++;
+
         bSpikeOn = UpdateSpikes (&vrgInputs[i], &dT, pdTime) || bSpikeOn;
         break;
 
@@ -409,7 +434,10 @@ void UpdateInputs (PDOUBLE pdTime, PDOUBLE pdNextTransTime)
 
   } /* for */
 
-  if (bSpikeOn) PostUpdateSpikes (pdTime);
+  /* if (bSpikeOn) PostUpdateSpikes (pdTime); FB 11/06/97: */
+  if (vbModelReinitd) { /* If model has just been initialized */
+    vbModelReinitd = FALSE; /* don't do it again */
+  }    
 
 } /* UpdateInputs */
 
@@ -737,25 +765,33 @@ void CalcInputs (PDOUBLE pdTime)
     case IFN_CONSTANT:
       break; /* dVal does not change */
 
-    case IFN_PERDOSE:
-    case IFN_PERRATE: /* Does not change step-wise */
-
-      break;
-
-    case IFN_PEREXP: /* Calc current exp() */
-      vrgInputs[i].dVal =
-          vrgInputs[i].dMag
-          * exp ((vrgInputs[i].dTStartPeriod - *pdTime)
-             * vrgInputs[i].dDecay);
-      break;
-
-        case IFN_SPIKES:
     case IFN_NDOSES:
       if (vrgInputs[i].iDoseCur < vrgInputs[i].nDoses)
         vrgInputs[i].dVal = vrgInputs[i].rgMags[vrgInputs[i].iDoseCur] *
                             vrgInputs[i].dMag; /* Scaled */
       break;
 
+    case IFN_PERDOSE:
+    case IFN_PERRATE: /* Does not change step-wise */
+      break;
+
+    case IFN_PEREXP: /* Calc current exp() */
+      vrgInputs[i].dVal = vrgInputs[i].dMag * 
+                          exp ((vrgInputs[i].dTStartPeriod - *pdTime) * 
+                               vrgInputs[i].dDecay);
+      break;
+
+    case IFN_SPIKES:
+      if ((*pdTime == vrgInputs[i].rgT0s[vrgInputs[i].iDoseCur]) &&
+          (vrgInputs[i].iDoseCur < vrgInputs[i].nDoses))
+        vrgInputs[i].dVal = vrgInputs[i].rgMags[vrgInputs[i].iDoseCur] *
+                            vrgInputs[i].dMag; /* Scaled */
+
+      else
+        vrgInputs[i].dVal = 0;
+
+      break;
+
       } /* switch */
     } /* if */
   } /* for all inputs */
@@ -819,9 +855,11 @@ void DumpSymbolTable (char *szFilename)
 void GetStateHandles (HVAR *phvar)
 {
   int i = 0;
+
   VMMAPSTRCT *pvm = vrgvmGlo;
-  while(pvm->szName) {
-    if(IsState(pvm->hvar))
+
+  while (pvm->szName) {
+    if (IsState(pvm->hvar))
       phvar[i++] = pvm->hvar;
     ++pvm;
   };
