@@ -6,11 +6,33 @@
 
    Copyright (c) 1993.  Don Maszle, Frederic Bois.  All rights reserved.
 
+   This program is free software; you can redistribute it and/or
+   modify it under the terms of the GNU General Public License
+   as published by the Free Software Foundation; either version 2
+   of the License, or (at your option) any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+
+   contact
+     Frdric Bois / Don Maszle
+     BEHS, School of Public Health
+     University of California at Berkeley
+     Berkeley, CA 94720
+
+     fbois@diana.lbl.gov
+
    -- Revisions -----
-     Logfile:  %F%
-    Revision:  %I%
-        Date:  %G%
-     Modtime:  %U%
+     Logfile:  SCCS/s.random.c
+    Revision:  1.27
+        Date:  18 Jul 1997
+     Modtime:  09:22:47
       Author:  @a
    -- SCCS  ---------
 
@@ -23,6 +45,7 @@
  * Also available are several other types of random variates:
 
    BetaRandom(alpha, beta, a, b)      -- Beta(alpha, beta) over [a, b]
+   BinomialBetaRandom(E, a, b)        -- BetaBinomial (n = E + E * a / b)
    BinomialRandom(p, n)               -- Binomial of n trials, P(each) = p
    Chi2Random(dof)                    -- Chi-squared w/dof degrees of freedom
    ExpRandom(beta)                    -- Exponential of inverse scale beta
@@ -48,6 +71,7 @@
    erfc(x)                                     -- Error function
    lnDFNormal(x, mu, sd)                       -- Log of the normal density
    lnGamma(x)                                  -- Natural log of gamma function
+   lnDFBeta (x, alpha, beta, min, max)         -- Log of beta density
    SetSeed(seed)                               -- Sets the seed of Randoms
    InitRandom(seed, f)                         -- Initializes the package
 
@@ -59,6 +83,10 @@
 
    If the random number generator is not initialized before any of its
    routines are called, it initializes itself with a default seed.
+
+   Modified on 18/3/97 by FB: write "inverse scale" in error message of 
+     ExpRandom 
+
 */
 
 #include <float.h>
@@ -82,10 +110,6 @@ typedef struct tagRANDREC {
 } RANDREC, *PRANDREC;
 
 
-#define SEED_MIN     1.0
-#define SEED_MAX     2147483646.0
-#define PI           3.1415926535897932384626433
-
 static vbNoSeed = TRUE;         /* Flag to prevent use without seed */
 static vbNotInitd = TRUE;       /* Flag to prevent use without initializing */
 static RANDREC vRandRec;        /* Global random informatino shared by */
@@ -219,7 +243,8 @@ double Randoms (void)
    The routine is initialized by calling InitRandom().  A check is
    done to assure that initialization is performed.
 
-   Adapted from Press et al., Numerical Recipes, by F.Bois, D.R.Maszle.
+   Adapted from the algorithm described in the book Numerical Recipes by
+   Press et al. 
 */
 
 double RandomShuffle (void)
@@ -243,7 +268,8 @@ double RandomShuffle (void)
 
    Returns a Normal random variate based on a unit variate,
    using a random generator as a source of uniform deviates.
-   From Press et al., Numerical Recipes.
+   Adapted from the algorithm described in the book Numerical Recipes by
+   Press et al. 
 
    Programs using Normal should initialize the random number
    generator with InitRandom().
@@ -251,7 +277,7 @@ double RandomShuffle (void)
 
 double NormalRandom (double dMean, double dStdDev)
 {
-  double factor, racin, tempo1, tempo2;
+  double dRacine, dTemp1, dTemp2, dTemp3;
 
   if (vbNotInitd)
     InitRandom (SEED_DEFAULT, TRUE);
@@ -262,15 +288,15 @@ double NormalRandom (double dMean, double dStdDev)
   } /* if */
 
   do {
-    tempo1 = 2 * RandomShuffle () - 1;
-    tempo2 = 2 * RandomShuffle () - 1;
-    racin = tempo1 * tempo1 + tempo2 * tempo2;
-  } while ((racin >= 1) || (racin == 0));
+    dTemp1 = 2 * RandomShuffle () - 1;
+    dTemp2 = 2 * RandomShuffle () - 1;
+    dRacine = dTemp1 * dTemp1 + dTemp2 * dTemp2;
+  } while ((dRacine >= 1) || (dRacine == 0));
 
-  factor = sqrt(-2 * log(racin) / racin);
-  vRandRec.memGauss = tempo1 * factor;
+  dTemp3 = sqrt(-2 * log(dRacine) / dRacine);
+  vRandRec.memGauss = dTemp1 * dTemp3;
   vRandRec.switchG = 1;
-  return (dMean + dStdDev * (tempo2 * factor));
+  return (dMean + dStdDev * (dTemp2 * dTemp3));
 
 } /* NormalRandom */
 
@@ -292,7 +318,7 @@ double LogNormalRandom (double dMean, double dStdDev)
                        "unknown", str, "LogNormalRandom");
   }
   else 
-    if (dStdDev <= 0) {
+    if (dStdDev < 1) {
       char str[10];
       sprintf(str, "%5.2e", dStdDev);
       ReportRunTimeError(NULL, RE_BADLOGNORMALSD | RE_FATAL,
@@ -374,7 +400,7 @@ double ExpRandom (double beta)
 {
 
   if (beta <= 0) {
-    printf ("Error: negative or null scale for an exponential variate "
+    printf ("Error: negative or null inverse scale for an exponential variate "
             "- Exiting\n\n");
     exit (0);
   }
@@ -594,101 +620,120 @@ long PoissonRandom(double mu)
    Return as a double floating-point number an integer value that is a random
    deviate drawn from a binomial distribution of n trials each of
    probability p, using Randoms () as a source of uniform random deviates.
-   From Press et al., Numerical Recipes.
+   Adapted from the algorithm described in the book Numerical Recipes by
+   Press et al. 
 */
 
-double BinomialRandom (double p, long n)
+double BinomialRandom (double p, long N)
 {
   long j;
-  static long nOld = (-1);
-  double mean, em, g, angle, pTemp, bnl, sq, t, y;
-  static double pOld = (-1.0), pc, pLog, pcLog, en, gOld;
+  static long iOldN = -1;
+  double dAngle, dDeviate, dMean, dPtemp, dSqrt, dTangent, dTemp1, dTemp2;
+  static double dLnFactN, dPold = -1, dLnP, dQ, dLnQ;
 
-  if (p < 0 || p > 1 || n < 0) {
+  if (p < 0 || p > 1 || N < 0) {
     printf ("Error: negative or null parameters for a binomial variate "
             "- Exiting\n\n");
     exit (0);
   }
 
-  pTemp = ( p <= 0.5 ? p : 1.0 - p);
-  mean = n * pTemp;  /* This is the mean of the deviate to be produced. */
+  dPtemp = ( p <= 0.5 ? p : 1 - p);
+  dMean = N * dPtemp;  /* mean of the deviate to be produced. */
 
-  /* Use the direct method while n is not too large.
+  /* Use the direct method if N is not too large.
      This can require up to 25 calls to random */
 
-  if (n < 25) {
-    bnl = 0.0;
-    for (j = 1; j <= n; j++)
-      if (Randoms () < pTemp )
-        bnl += 1.0;
+  if (N < 25) {
+    dDeviate = 0;
+    for (j = 0; j < N; j++)
+      if (Randoms () < dPtemp)
+        dDeviate = dDeviate + 1;
   }
   else
-    if (mean < 1.0) {
+    if (dMean < 1) {
       /* if less than one event is expected out of 25 or more trials,then the
          distribution is quite accurately Poisson. Use direct method. */
-      g = exp(-mean);
-      t = 1.0;
-      for (j = 0; j <= n; j++) {
-        t *= Randoms ();
-        if (t < g) break;
-      } /* for */
+      dTemp1 = exp(-dMean);
+      dTemp2 = 1.0;
+      for (j = 0; j <= N; j++) {
+        dTemp2 = dTemp2 * Randoms ();
+        if (dTemp2 < dTemp1) break;
+      }
 
-      bnl = (j <= n ? j : n);
-    } /* if */
+      dDeviate = (j <= N ? j : N);
+    }
+    else { /* Use rejection */
 
-    else {
-    /* Use the rejection method. */
+      if (N != iOldN) { 
+        /* if N has changed or it's the first call, initialize */
+        dLnFactN = lnGamma((double) N + 1);
+        iOldN = N;
+      }
 
-      if (n != nOld) {
-        /* if n has changed, compute useful quantities */
-
-        en = n;
-        gOld = lnGamma(en + 1.0);
-        nOld = n;
+      if (dPtemp != dPold) { 
+        /* if dPtemp has changed or it's the first call, initialize. */
+        dPold = dPtemp;
+        dQ = 1 - dPtemp;
+        dLnP = log(dPtemp);
+        dLnQ = log(dQ);
       } /* if */
 
-      if (pTemp != pOld) {
-        /* if pTemp has changed, compute useful quantities. */
-
-        pc = 1.0 - pTemp;
-        pLog = log(pTemp);
-        pcLog = log(pc);
-        pOld = pTemp;
-      } /* if */
-
-      sq = sqrt( 2.0 * mean * pc );
+      dSqrt = sqrt(2 * dMean * dQ);
 
       /* Rejection method with a Lorentzian comparison function. */
 
       do {
         do {
-          angle = 3.141592654 * Randoms ();
-          y = tan(angle);
-          em = sq * y + mean;
-        } while (em < 0.0 || em >= (en + 1.0)); /* Reject */
+          dAngle = PI * Randoms ();
+          dTangent = tan(dAngle);
+          dTemp1 = dSqrt * dTangent + dMean;
+        } while (dTemp1 < 0 || dTemp1 >= (N + 1)); /* Reject */
 
-        em = floor(em); /* Trick for integer-valued distribution */
+        dTemp1 = floor(dTemp1); /* discrete distribution */
 
-        t = 1.2 * sq * (1.0 + y * y) *
-            exp(gOld - lnGamma(em + 1.0) - lnGamma(en - em + 1.0) +
-                em * pLog + (en - em) * pcLog);
+        dTemp2 = 1.2 * dSqrt * (1 + dTangent * dTangent) *
+                 exp(dLnFactN - lnGamma(dTemp1 + 1) - 
+                     lnGamma(N - dTemp1 + 1) +
+                     dTemp1 * dLnP + (N - dTemp1) * dLnQ);
 
-      } while (Randoms () > t);
+      } while (Randoms () > dTemp2);
 
-      /* Reject. This happens about 1.5 time per deviate on average */
+      /* Reject on average about 1.5 time per deviate */
 
-      bnl = em;
+      dDeviate = dTemp1;
 
     } /* else */  /* end of rejection */
 
-  if ( pTemp != p )
-    bnl = n - bnl; /* Remember to undo the symmetry tranf */
+  if (dPtemp != p)
+    dDeviate = N - dDeviate; /* undo the symmetry tranformation */
 
-  return (bnl);
+  return (dDeviate);
 
 } /* BinomialRandom */
 
 
+/* ----------------------------------------------------------------------------
+   BinomialBetaRandom
+
+   Return as a double floating-point number an integer value that is a random
+   deviate drawn from a binomial distribution of n trials each of
+   probability p, p being beta distributed with parameters alpha and beta.
+   I use the expectation in input. The classical N is equal to 
+   E + E * beta / alpha.
+   See Bernardo & Smith "Bayesian Theory"
+*/
+
+double BinomialBetaRandom (double Expectation, double alpha, double beta)
+{
+
+  /* parameters will be checked in BinomialRandom */
+
+  return BinomialRandom (BetaRandom (alpha, beta, 0, 1),
+                         Expectation + Expectation * beta / alpha);
+
+} /* BinomialBetaRandom */
+
+
 /* -----------------------------------------------------------------------------
    Multinomial
 
@@ -905,26 +950,30 @@ double BetaRandom (double alpha, double beta, double a, double b)
 /* -----------------------------------------------------------------------------
    erfc
 
-   the error function of z (Press et al. Numerical Recipes)
+   the error function of z.
+
+   Adapted from the algorithm described in the book Numerical Recipes by
+   Press et al. 
+
 */
 
-double erfc(double x)
+double erfc (double x)
 {
-  double t, z, ans;
+  double dAbsX, t, dVal;
 
-  z = fabs(x);
-  t = 1.0 / (1.0 + 0.5 * z);
-  ans = t * exp(-z*z - 1.26551223 + t*(1.00002368 + t*(0.37409196 +
-        t*(0.09678418 + t*(-0.18628806 + t*(0.27886807 + t*(-1.13520398 +
-        t*(1.48851587 + t*(-0.82215223 + t*(0.17087277))))))))));
-  return ( x >= 0.0 ? ans : 2.0 - ans );
+  dAbsX = fabs(x);
+  t = 1 / (1 + 0.5 * dAbsX);
+  dVal = t * exp(-dAbsX*dAbsX - 1.26551223 + t*(1.00002368 + t*(0.37409196 +
+         t*(0.09678418 + t*(-0.18628806 + t*(0.27886807 + t*(-1.13520398 +
+         t*(1.48851587 + t*(-0.82215223 + t*(0.17087277))))))))));
+  return ( x >= 0 ? dVal : 2 - dVal );
 }
 
 
 /* -----------------------------------------------------------------------------
    CDFNormal
 
-   the probability for [-inf;z] under the normal distribution
+   the probability for [-inf;Z] under the normal distribution
 */
 
 double CDFNormal (double z)
@@ -950,33 +999,63 @@ double lnDFNormal (double x, double mu, double sd)
 }
 
 
+/* -----------------------------------------------------------------------------
+   lnDFBeta
+   the log of the beta density function
+   FB 08/07/97
+*/
+
+double lnDFBeta (double x, double alpha, double beta, double min, double max)
+{
+  if (max <= min) {
+    printf ("Error: bad range for beta variate in lnDFBeta\n");
+    exit (0);
+  }
+  if (alpha <= 0) {
+    printf ("Error: bad alpha for beta variate in LnDensity\n");
+    exit (0);
+  }
+  if (beta <= 0) {
+    printf ("Error: bad beta for beta variate in LnDensity\n");
+    exit (0);
+  }
+
+  x = (x - min) / (max - min); 
+  return (alpha - 1) * log (x) + (beta - 1) * log (1 - x) +
+         lnGamma (alpha + beta) - lnGamma (alpha) - lnGamma(beta) - 
+         log (max - min);
+}
+
+
 /* -----------------------------------------------------------------------------
    lnGamma
 
    A function to return the natural log of the Gamma function of x.
-   From Press et al., Numerical Recipes.
+   Adapted from the algorithm described in the book Numerical Recipes by
+   Press et al. 
    It can be used to compute factorials since ln(n!) = lnGamma(n + 1)
 */
 
 double lnGamma (double x)
 {
-  double tmp, ser;
+  double dSeries, dTemp;
 
   if (x <= 0.0) {
     printf ("Error: negative or null parameter for lnGamma function\n");
     exit (0);
   }
   
-  ser = 1.000000000190015 +
-        76.18009172947146   /  x      -
-        86.50532032141677   / (x + 1) +
-        24.01409824083091   / (x + 2) -
-        1.231739572450155   / (x + 3) +
-        1.20865097386617E-3 / (x + 4) -
-        5.39523938495E-6    / (x + 5);
+  dSeries = 1.000000000190015 +
+            76.18009172947146   /  x      -
+            86.50532032141677   / (x + 1) +
+            24.01409824083091   / (x + 2) -
+            1.231739572450155   / (x + 3) +
+            1.20865097386617E-3 / (x + 4) -
+            5.39523938495E-6    / (x + 5);
 
-  tmp = x + 4.5;
-  return -tmp + (x - 0.5) * log (tmp) + log (2.50662827465 * ser);
+  dTemp = x + 4.5;
+  dTemp = -dTemp + (x - 0.5) * log (dTemp) + log (2.50662827465 * dSeries);
+  return dTemp;
 
 } /* lnGamma */
 
