@@ -6,11 +6,33 @@
 
    Copyright (c) 1993.  Don Maszle, Frederic Bois.  All rights reserved.
 
+   This program is free software; you can redistribute it and/or
+   modify it under the terms of the GNU General Public License
+   as published by the Free Software Foundation; either version 2
+   of the License, or (at your option) any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+
+   contact
+     Frdric Bois / Don Maszle
+     BEHS, School of Public Health
+     University of California at Berkeley
+     Berkeley, CA 94720
+
+     fbois@diana.lbl.gov
+
    -- Revisions -----
-     Logfile:  %F%
-    Revision:  %I%
-        Date:  %G%
-     Modtime:  %U%
+     Logfile:  SCCS/s.random.h
+    Revision:  1.20
+        Date:  18 Jul 1997
+     Modtime:  09:22:48
       Author:  @a
    -- SCCS  ---------
 
@@ -27,7 +49,11 @@
 /* ----------------------------------------------------------------------------
    Constants  */
 
+#define SEED_MIN     1.0
+#define SEED_MAX     2147483646.0
 #define SEED_DEFAULT 314159265.3589793
+#define PI           3.1415926535897932384626433
+#define INV_SQRT_2PI 0.398942280401433
 
 
 /* ----------------------------------------------------------------------------
@@ -47,6 +73,7 @@ double Randoms (void);
 /* Several types of random variates */
 
 double BetaRandom (double alpha, double beta, double a, double b);
+double BinomialBetaRandom (double Expectation, double alpha, double beta);
 double BinomialRandom (double p, long n);
 double Chi2Random (double dof);
 double ExpRandom (double beta);
@@ -75,6 +102,7 @@ double InterpolateX (double rgX[], double rgY[], long lLower, double dY);
 double erfc(double x);
 double lnDFNormal(double x, double mu, double sd);
 double lnGamma (double x);
+double lnDFBeta (double x, double alpha, double beta, double min, double max);
 void   CalcCumulative (long cDim, double *rg_x, double *rg_pdf,
                        double *rg_Cdf, int  iOrder);
 void   SetSeed (double dSeed);
