@@ -2,11 +2,33 @@
 
    Copyright (c) 1993.  Don Maszle, Frederic Bois.  All rights reserved.
 
+   This program is free software; you can redistribute it and/or
+   modify it under the terms of the GNU General Public License
+   as published by the Free Software Foundation; either version 2
+   of the License, or (at your option) any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+
+   contact
+     Frdric Bois / Don Maszle
+     BEHS, School of Public Health
+     University of California at Berkeley
+     Berkeley, CA 94720
+
+     fbois@diana.lbl.gov
+
    -- Revisions -----
-     Logfile:  %F%
-    Revision:  %I%
-        Date:  %G%
-     Modtime:  %U%
+     Logfile:  SCCS/s.sim.c
+    Revision:  1.46
+        Date:  30 Jul 1997
+     Modtime:  02:13:26
       Author:  @a
    -- SCCS  ---------
 
@@ -14,6 +36,13 @@
 
    Zeng added lsodes_() in function DoOneExperiment ()
 
+   Modified 18/03/97 by FB in DoMonteCarlo corrected integration failure warning.
+
+   Modified by FB 21/05/1997: disabled simhelp function calls and command-line
+   options; removed dead CheckDependencies code. Removed command-lines options.
+
+   Modified by FB 15/07/97: re-enabled command-line options (but they are 
+   undeveloped) for GNU conformity.
 */
 
 #include <stdio.h>
@@ -26,11 +55,12 @@
 #include "yourcode.h"
 #include "getopt.h"
 #include "mh.h"
+#include "optdesign.h"
 #include "lexerr.h"
 #include "lsodes.h"
 #include "mac.h"
 #include "sim.h"
-#include "simhelp.h"
+/* #include "simhelp.h" */
 #include "simi.h"
 #include "siminit.h"
 #include "simo.h"
@@ -38,7 +68,7 @@
 #include "strutil.h"
 
 
-static char vszVersion[] = "v4.0"; /* Version of program */
+static char vszVersion[] = "v4.1"; /* Version of program */
 
 
 /* -----------------------------------------------------------------------------
@@ -198,8 +228,8 @@ int DoOneExperiment (PEXPERIMENT pexp)
 
   pexp->dTime = pexp->dT0;
 
-  /* set lsodes flag to 1 for first call */
-  pexp->is.iDSFlag = 1;
+  /* set lsodes return flag to 1 for first call */
+  pis->iDSFlag = 1;
 
   /* Iterate to final time */
   while (pexp->dTime < pexp->dTfinal) {
@@ -213,12 +243,14 @@ int DoOneExperiment (PEXPERIMENT pexp)
 
       if (pis->iAlgo == IAL_LSODES) { /* Lsodes algorithm */
 
+        pis->rwork[0] = dTup; /* do not overshoot dTup - FB 01/07/97 */
+
         lsodes_(&pmod->nStates, pmod->pdModelVars, &(pexp)->dTime,
                 &dTup, &pis->itol, &pis->dRtol, &pis->dAtol,
                 &pis->itask, &pis->iDSFlag, &pis->iopt, pis->rwork,
                 &pis->lrw, pis->iwork, &pis->liw, &pis->iMf);
 
-        /* Handle error returns : FB 25 nov 96 : */
+        /* Handle error returns : FB 25/11/96 : */
         if (pis->iDSFlag < 0) {
           /* We cannot guarantee the accuracy of the results, exit the routine
              with an error flag */
@@ -420,6 +452,16 @@ void DoMonteCarlo (PANALYSIS panal)
   if (!nRuns)
     printf ("0 runs specified for SetPoint().  Reading entire file.\n\n");
 
+  /* FB 21/07/97: dependencies between parameters are handled here. Pointers 
+     to the parent parameters are stored in hparm of the pmcvar structure for
+     each parameter. We now have to make pdParms point to the parent's dVals. */
+  if (panal->iType == AT_MONTECARLO) {
+    SetParents (&panal->mc, 0); /* start at 0, do them all */
+  }
+  else { /* panal->iType == AT_SETPOINTS */
+    SetParents (&panal->mc, panal->mc.nSetParms);
+  }
+
   panal->mc.lRun = 0; /* First run */
   bNotDone = TRUE;
 
@@ -440,14 +482,14 @@ void DoMonteCarlo (PANALYSIS panal)
         WriteMCOutput (panal, &mcpredout);
       }
       else
-        printf ("Warning: "
-                "Integration failed on iteration %ld, experiment %d:"
-                "         No output generated\n", panal->mc.lRun, i);
+        printf ("Warning: Integration failed on iteration %ld, experiment %d:\n"
+                "         No output generated\n", panal->mc.lRun+1, i+1);
     } /* if bNotDone */
 
     panal->mc.lRun++; /* Next run */
     if (nRuns) /* If a number of runs spec'd... */
       bNotDone = (panal->mc.lRun < nRuns);
+
   } /* while */
 
   CloseMCFiles (panal);
@@ -476,11 +518,14 @@ void DoAnalysis (PANALYSIS panal)
       DoMonteCarlo (panal);
       break;
 
-    case AT_GIBBS:
     case AT_MCMC:
       DoMarkov (panal);
       break;
 
+    case AT_OPTDESIGN:
+      DoOptimalDesign (panal);
+      break;
+
   } /* switch */
 
   if (panal->expGlobal.os.pfileOut) {
@@ -530,7 +575,7 @@ void PrepAnalysis (PANALYSIS panal)
      arrays anyway */
   if (pmc->nParms == 0) return;
   
-  pmc->rgdParms = (double *) malloc((pmc->nParms)*sizeof(double));
+  pmc->rgdParms = InitdVector (pmc->nParms);
   pmc->rgpMCVar = (MCVAR **) malloc((pmc->nParms)*sizeof(MCVAR *));
   if (!(pmc->rgdParms && pmc->rgpMCVar))
     ReportError (NULL, RE_OUTOFMEM | RE_FATAL, "PrepAnalysis", NULL);
@@ -555,50 +600,6 @@ void PrepAnalysis (PANALYSIS panal)
 } /* PrepAnalysis */
 
 
-/* ----------------------------------------------------------------------------
-   CheckDependencies
-
-   Sets the dependency level for MC variables
-   (those included in `Distrib' statements)
-*/
-
-void CheckDependencies (PANALYSIS panal) {
-/*  MONTECARLO *pmc = &panal->mc;
-  MCVAR *pmcvar;
-  int n, m, l;
-  char *level, lv, prevLevel, varParm;
-  char s[4];
-  BOOL changed;
-  pmc->nLevels = 0;
-  for(n = 0; n < MAX_LEVELS; ++n) {
-    ++pmc->nLevels;
-    changed = FALSE;
-    for(m = 0; m < pmc->nParms; ++m) {
-      for(l = 0; l < pmc->nParms; ++l) {
-        if(l == m) continue;
-        pmcvar = pmc->rgpMCVar[l];
-        if((varParm = pmcvar->cVarParm) == 0) continue;
-        level = &pmcvar->cLevel;
-        if(((varParm & 1) && (pmcvar->uParm1.hvar == pmc->rgpMCVar[m]->hvar)) ||
-           ((varParm & 2) && (pmcvar->uParm2.hvar == pmc->rgpMCVar[m]->hvar)) ||
-           ((varParm & 4) && (pmcvar->uMin.hvar == pmc->rgpMCVar[m]->hvar)) ||
-           ((varParm & 8) && (pmcvar->uMax.hvar == pmc->rgpMCVar[m]->hvar))) {
-            prevLevel = *level;
-            if((lv = pmc->rgpMCVar[m]->cLevel + 1) == MAX_LEVELS) {
-              sprintf(s, "%d", lv);
-              ReportError(NULL, RE_TOOMANYLEVELS | RE_FATAL, s, NULL);
-            }
-            if(lv > prevLevel) {
-              *level = lv;
-              changed = TRUE;
-            }
-        }
-      }
-    }
-    if(!changed) break;
-  }*/
-} /* CheckDependencies */
-
 /* Get the command line argument stuff */
 
 /* -----------------------------------------------------------------------------
@@ -606,7 +607,7 @@ void CheckDependencies (PANALYSIS panal) {
 
    returns a pointer to just the filename of a full path.
 */
-
+/*
 char *SansPath (char *szFullPathname)
 {
   register char *szFile;
@@ -616,11 +617,11 @@ char *SansPath (char *szFullPathname)
       if (*szFullPathname == '/')
         szFile = szFullPathname+1;
       szFullPathname++;
-    } /* while */
+    }
 
   return szFile;
 
-} /* SansPath */
+} */ /* SansPath */
 
 
 /* -----------------------------------------------------------------------------
@@ -671,8 +672,10 @@ void PromptFilenames (PSTR *pszFileIn, PSTR *pszFileOut)
 
 /* -----------------------------------------------------------------------------
    Command Line Parsing
+
 */
 
+/*
 #define WarnArgumentReqd(szOption, szArg) \
   printf ("* Command-line option \"%s\" requires an argument \"%s\"\n",\
           szOption, szArg);
@@ -683,52 +686,26 @@ void PromptFilenames (PSTR *pszFileIn, PSTR *pszFileOut)
 
 void GetOutputFlagOption (PANALYSIS panal, char *optarg)
 {
-  if (!strcmp (optarg, "MCOutputs"))       panal->fCmdOptions |= OF_MCOUTPUTS;
+  WarnUnknownArg ("-O", optarg);
 
-  else if (!strcmp (optarg,"NoMCOutputs")) panal->fNotOptions |= OF_MCOUTPUTS;
-
-  else if (!strcmp (optarg, "MCResult"))   panal->fCmdOptions |= OF_MCRESULT;
-
-  else if (!strcmp (optarg, "NoMCResult")) panal->fNotOptions |= OF_MCRESULT;
-
-  else if (!strcmp (optarg, "ParmList"))   panal->fCmdOptions |= OF_PARMLIST;
-
-  else if (!strcmp (optarg, "NoParmList")) panal->fNotOptions |= OF_PARMLIST;
-
-  else if (!strcmp (optarg, "VarNames"))   panal->fCmdOptions |= OF_VARNAMES;
-
-  else if (!strcmp (optarg, "NoVarNames")) panal->fNotOptions |= OF_VARNAMES;
-
-  else WarnUnknownArg ("-O", optarg);
-
-} /* GetOutputFlagOption */
+} */ /* GetOutputFlagOption */
 
 
 /* -----------------------------------------------------------------------------
    GetFilenames
 
-   retrieves the filenames from the command line arguments passed to
+   retrieves options and filenames from the command line arguments passed to
    the program.
 
    The command line syntax is:
 
-     pksim [-options] [input-file [output-file | 4-file-names]]
+     mcsim [-options | --options] [input-file [output-file]]
 
-   If the output filename is not given a poorly chosen default is
-   used.  If neither the input, nor output filenames are given, the
+   If the output filename is not given a default is used.
+   If neither the input, nor output filenames are given, the
    program prompts for them both.
 
-   If the optional four extra filenames are included instead, they are
-   interpreted as files for Monte Carlo output in this manner:
-
-     pass-file fail-file behavior-file summary-file
-
    The options can appear anywhere in the line and in any order.
-   Where conflicts occur, the option specified first SHOULD be used,
-   but probably the last is used.  Also, inibitory options override
-   requested options, e.g. -ONoParmList -OParmList does not print the
-   parmlist.  This is considered acceptable because if you are
-   braindead enough to put this on the command-line, you lose.
 
    The options are parsed with _getopt(). After _getopt() is called,
    the args in rgszArg have been permuted so that non-option args are
@@ -740,121 +717,83 @@ void GetOutputFlagOption (PANALYSIS panal, char *optarg)
      int   optind;    -- Index in ARGV of the next elem to be scanned
      char *nextchar;  -- The next char to be scanned in the option-element
      int   opterr;    -- 0 value flags to inhibit GNU error messages
+
 */
 
-static char vszOptions[] = "h:H:O:D:n:S:";
+static char vszOptions[] = "h:H:D:";
 
-BOOL GetCmdLineArgs (int cArg,
-                     char *const *rgszArg,
-                     PSTR *pszFileIn, PSTR *pszFileOut,
-                     PANALYSIS panal)
+void GetCmdLineArgs (int cArg, char *const *rgszArg, PSTR *pszFileIn, 
+                     PSTR *pszFileOut, PANALYSIS panal)
 {
   int c;
 
   *pszFileIn = *pszFileOut = (PSTR) NULL;
 
-  /* Process command-line arguments */
+  while (1) {
 
-  if (cArg == 0)
-    PromptFilenames (pszFileIn, pszFileOut);
-  else {
-    while (1) {
+    c = _getopt (cArg, rgszArg, vszOptions);
+    if (c == EOF) /* Finish with option args */
+      break;
 
-      c = _getopt (cArg, rgszArg, vszOptions);
-      if (c == EOF) /* Finish with option args */
-        break;
+    switch (c) {
+      case '?':
+        optarg = 0;
+        /* Fall through! */
 
-      switch (c) {
-        case '?':
-          optarg = 0;
-          /* Fall through! */
-
-        case 'H':
-        case 'h':
-          if (optarg && *optarg)
-            ShowHelp (optarg);
-          else
-            ShowHelpMessage (SansPath (rgszArg[0]));
-          exit (-1);
-          break;
-
-        case 'D':
-          printf (">> Debug mode: Using option '%s'\n", optarg);
-          /* Can setup to run with certain debug flags */
-          if(!strcmp(optarg, "d"))
-            panal->bDependents = TRUE;
-          else if(!strcmp(optarg, "i"))
-            panal->bPrintIter = TRUE;
-          else if(!strcmp(optarg, "p"))
-            panal->bParams = TRUE;
-          break;
-
-        case 'n':
-          break;
-
-        case 'O':
-          if (!optarg) {
-            WarnArgumentReqd ("-O", "output format");
-            break;
-          } /* if */
-          GetOutputFlagOption (panal, optarg);
-          break;
-
-        case 'S':
-          if (optarg) {
-            if (!(panal->fCmdOptions & OF_CMDLSEED))
-              panal->dSeed = atof (optarg);
-            panal->fCmdOptions |= OF_CMDLSEED;
-          } /* if */
-          else
-            WarnArgumentReqd ("-S", "random seed");
-          break;
-
-        default:
-          printf ("Unknown option in command-line, %c = code 0%o ?\n", c, c);
-          break;
-
-      } /* switch */
-
-    } /* while */
-
-    switch (cArg - optind) { /* Remaining args are  filenames */
-      default:
-        ShowHelp ("Usage");
+      case 'H':
+      case 'h':
+        /* if (optarg && *optarg) disabled for now, not up to date
+          ShowHelp (optarg);
+        else
+          ShowHelpMessage (SansPath (rgszArg[0])); */
         exit (-1);
         break;
 
-      case 5: /* Input and 2 output files spec'd */
-        printf ("Error: the 2 output files option is now obsolete - "
-                "Exiting\n\n");
+      case 'D':
+        printf (">> Debug mode: Using option '%s'\n", optarg);
+        /* Could setup to run with certain debug flags, not enabled  */
         break;
 
-      case 2: /* Output and input file specificed */
-        *pszFileOut = rgszArg[optind + 1];
-
-        /* Fall through! */
-
-      case 1: /* Input file specificed */
-        *pszFileIn = rgszArg[optind];
-        break;
-
-      case 0: /* No file names specified */
-        PromptFilenames (pszFileIn, pszFileOut);
+      default:
+        printf ("Unknown option in command-line, %c = code 0%o ?\n", c, c);
         break;
 
     } /* switch */
 
-  } /* else */
+  } /* while */
 
-  while (*pszFileIn && (*pszFileIn)[0]        /* Files specified   */
-     && !MyStrcmp(*pszFileIn, *pszFileOut)) { /* and not different */
+  switch (cArg - optind) { /* Remaining args are  filenames */
+    case 2: /* Output and input file specificed */
+      *pszFileOut = rgszArg[optind + 1];
 
-     printf ("\n** Input and output filename must be different.\n");
+      /* Fall through! */
+
+    case 1: /* Input file specificed */
+      *pszFileIn = rgszArg[optind];
+      break;
+
+    case 0: /* No file names specified */
+      PromptFilenames (pszFileIn, pszFileOut);
+      break;
+
+    default:
+      /* ShowHelp ("Usage"); */ /* disabled for now, not updated */
+      exit (-1);
+      break;
+  } /* switch */
+
+  while (*pszFileIn && (*pszFileIn)[0] &&      /* Files specified   */
+         !MyStrcmp(*pszFileIn, *pszFileOut)) { /* and not different */
+
+    printf ("\n** Input and output filename must be different.\n");
     PromptFilenames (pszFileIn, pszFileOut);
 
   } /* while */
 
-  return (*pszFileIn && (*pszFileIn)[0]); /* Input name given */
+  if (!(*pszFileIn && (*pszFileIn)[0])) { /* no input name given is an error */
+    printf ("Error: an input file name must be specified - Exiting\n\n");
+    exit (-1);
+  }
 
 } /* GetCmdLineArgs */
 
@@ -864,8 +803,13 @@ BOOL GetCmdLineArgs (int cArg,
 void AnnounceProgram (void)
 {
   printf ("\n________________________________________\n");
-  printf ("\nMCSim simulation program: %s\n\n", vszVersion);
-  printf ("All rights reserved.\n\n");
+  printf ("\nMCSim %s\n\n", vszVersion);
+  printf ("Copyright (c) 1993-1997 by F. Bois & D. Maszle. "
+          "All rights reserved.\n\n");
+
+  printf ("MCSim comes with ABSOLUTELY NO WARRANTY;\n"
+          "This is free software, and you are welcome to redistribute it\n"
+          "under certain conditions; see the GNU General Public License.\n\n");
 
   printf ("* Using `%s' model in file \"%s\" created by %s\n\n",
           szModelDescFilename, szModelSourceFilename, szModelGenAndVersion);
@@ -894,14 +838,12 @@ int main (int nArg, char *const *rgszArg)
 
   AnnounceProgram ();
 
-
   if (!panal)
     ReportError (NULL, RE_OUTOFMEM | RE_FATAL,
                  "ANALYSIS specification too large", NULL);
 
   InitAnalysis (panal);
-  if (!(GetCmdLineArgs (nArg, rgszArg, &szFileIn, &szFileOut, panal)))
-    exit (-1);
+  GetCmdLineArgs (nArg, rgszArg, &szFileIn, &szFileOut, panal);
 
   /* Define the output file as the global experiment default  */
   panal->expGlobal.os.szOutfilename = szFileOut;
@@ -914,9 +856,7 @@ int main (int nArg, char *const *rgszArg)
   ibIn.pInfo = (PVOID) panal; /* Attach analysis specification to input */
 
   if (ReadAnalysis (&ibIn)) {
-    SetOptions (panal);
     PrepAnalysis (panal);
-    CheckDependencies (panal);
     DoAnalysis (panal);
   } /* if */
   FreeLevels(panal);
