@@ -4,15 +4,39 @@
 
    Copyright (c) 1993.  Don Maszle, Frederic Bois.  All rights reserved.
 
+   This program is free software; you can redistribute it and/or
+   modify it under the terms of the GNU General Public License
+   as published by the Free Software Foundation; either version 2
+   of the License, or (at your option) any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+
+   contact
+     Frdric Bois / Don Maszle
+     BEHS, School of Public Health
+     University of California at Berkeley
+     Berkeley, CA 94720
+
+     fbois@diana.lbl.gov
+
    -- Revisions -----
-     Logfile:  %F%
-    Revision:  %I%
-        Date:  %G%
-     Modtime:  %U%
+     Logfile:  SCCS/s.sim.h
+    Revision:  1.33
+        Date:  02 Jul 1997
+     Modtime:  08:37:16
       Author:  @a
    -- SCCS  ---------
 
    Header file for simulation
+
+   Modified the obsolete "Options Flags"
 */
 
 #ifndef _SIM_H_
@@ -36,11 +60,11 @@
    Other maximum sizes are MAX_EQN and MAX_LEX in lex.h */
 
 #define MAX_LEVELS            10   /* for now; actual is 255 */
-#define MAX_INSTANCES         100  /* for now; allowable instances of `Level'
+#define MAX_INSTANCES         200  /* for now; allowable instances of `Level'
                                       at any depth */
-#define MAX_EXPERIMENTS       100
+#define MAX_EXPERIMENTS       200
 #define LSODES_IWORKSIZE      300  /* Lsodes tells if this is not big enough */
-#define LSODES_RWORKSIZE      700  /* Idem */
+#define LSODES_RWORKSIZE      1200 /* Idem */
 #define ARGS_MAX              8    /* Maximum number of args to lex */
                                    /* Used in allocating argument strings */
 #define MAX_PRINT_VARS        5    /* Arbitrary */
@@ -61,9 +85,9 @@
 #define KM_MCVARY          12
 #define KM_SETPOINTS       13
 #define KM_OUTPUTFILE      14
-#define KM_GIBBS           15
+#define KM_MCMC            15
 #define KM_LEVEL           16
-#define KM_MCMC            17
+#define KM_OPTDESIGN       17
 
 #define KM_END             100
 
@@ -90,12 +114,16 @@
 #define KM_LOGNORMALV      225
 #define KM_TRUNCNORMALV    226
 #define KM_TRUNCLOGNORMALV 227
+#define KM_BINOMIALBETA    228
 
 #define KM_PREDICTION      300
 
 #define KM_LSODES          600
 #define KM_EULER           601
 
+#define KM_FORWARD         700
+#define KM_BACKWARD        701
+
 /* Context bit flags */
 
 #define CN_END             0x0000    /* No context */
@@ -112,25 +140,8 @@
 #define AT_DEFAULTSIM   1    /* Normal simulation */
 #define AT_MONTECARLO   2    /* Monte Carlo variations */
 #define AT_SETPOINTS    3    /* Set points simulation */
-#define AT_GIBBS        4    /* Gibbs estimation */
-#define AT_MCMC         5    /* Metropolis estimation */
-
-/* Options Flags -- bit flags */
-
-#define OF_PARMLIST    0x0001   /* Print parameter list */
-#define OF_MCOUTPUTS   0x0002   /* Print MCOutData array -- output values    */
-#define OF_MCRESULT    0x0004   /* Print result of MC or SetPoints analysis  */
-#define OF_PASSFAIL    0x0008   /* Print result of MC as Pass/Fail [1 or 0]  */
-#define OF_MONTEBITS   0x000F
-
-#define OF_COMMENT     0x0010   /* Print a comment (no comment) to outfile */
-#define OF_VARNAMES    0x0020   /* Print the variable names (no names)     */
-#define OF_VERT_OUTPUT 0x0040   /* Print variables in columns (rows) */
-#define OF_NORMALBITS  0x00F0
-
-#define OF_CMDLSEED    0x1000   /* Seed given on command-line */
-#define OF_CMDLOUTFILE 0x2000   /* Output filename given on command-line */
-#define OF_CMDLBITS    0xF000
+#define AT_MCMC         4    /* Metropolis or Gibbs estimation */
+#define AT_OPTDESIGN    5    /* Metropolis or Gibbs estimation */
 
 /* Monte Carlo Variation types */
 
@@ -153,6 +164,7 @@
 #define MCV_LOGNORMALV      15
 #define MCV_TRUNCNORMALV    16
 #define MCV_TRUNCLOGNORMALV 17
+#define MCV_BINOMIALBETA    18
 
 /* Integration Method types */
 
@@ -164,10 +176,10 @@
 #define IAL_DEFAULT     IAL_LSODES
 #define IOPT_DEFAULT    (0)
 #define ITOL_DEFAULT    (1)
-#define ITASK_DEFAULT   (1)
+#define ITASK_DEFAULT   (4)   /* do not overshoot - FB 01/07/97 */
 #define RTOL_DEFAULT    (1.0e-5)
 #define ATOL_DEFAULT    (1.0e-7)
-#define IMF_DEFAULT     (1) /* stiff */
+#define IMF_DEFAULT     (222) /* stiff */
 #define TSTEP_DEFAULT   (1)
 
 /* Simulation specification defaults */
@@ -263,7 +275,6 @@ typedef struct tagOUTSPEC {
   PINT    piSigmas;         /* Array of SDs for each var */
   PDOUBLE *prgdOutputTimes; /* Array of output times for each var */
   PDOUBLE *prgdOutputVals;  /* Array of output values for each var */
-  PDOUBLE *prgdSavedVals;   /* Array of saved output values for each var */
   PDOUBLE *prgdDataVals;    /* Array of data values for each var */
 
   int cDistinctTimes;       /* Count of distinct output times */
@@ -277,21 +288,22 @@ typedef struct tagOUTSPEC {
 typedef struct tagMCVAR {
   PSTR    pszName;          /* Model variableariable name */
   HVAR    hvar;             /* Handle to the model variable to be modified */
-  int     iDepth;           /* Level (to distinguish among vars with same hvar) */
+  int     iDepth;           /* Level (to distinguish vars with same hvar) */
   int     iType;            /* One of MCV_ distribution types */
   char    cVarParm;         /* Flag for variable parameters */
-  HVAR    hParm[4];         /* pointers to model vars for 4 distribution parameters */
+  HVAR    hParm[4];         /* pointers to model vars for 4 distrib. params */
   double  dParm[4];         /* values of distribution parameters */
   PDOUBLE pdParm[4];        /* pointers to distribution parameters */
   double  dVal;             /* Value for this run */
-  struct tagMCVAR *pMCVParent[4]; /* Pointers to parents of this var, i.e., vars on
-                                     which this var depends */
-  PLIST   plistDependents;  /* List of MC vars that depend directly on this one */
+  struct  tagMCVAR *pMCVParent[4]; /* Pointers to parents of this var (vars on
+                                      which this var depends) */
+  PLIST   plistDependents;  /* List of MCvars depending directly on this one */
   long    nDependents;
-  struct tagMCVAR **rgpDependents;
+  struct  tagMCVAR **rgpDependents;
   BOOL    bExptIsDep;       /* True if experiment is dependent on this var */
   int     iParmIndex;       /* Index in OUTSPEC when Parm[1] is ModelVar */
   BOOL    bIsFixed;         /* True if var is fixed */
+  BOOL    bGibbs;           /* True if its conditional distrib. is known */
   long    lJumps;           /* Number of MH jumps for this param */
   double  dKernelSD;        /* MCMC jumping kernel SD */
 
@@ -304,18 +316,19 @@ typedef struct tagGIBBSDATA {
   long nPrintFreq;      /* To request output every nPrintFreq iterations */
   long nPrintIter;      /* Number of final iterations to print */
 
-  PSTR szGout;          /* Filename for Gibbs output */
-  PFILE pfileGibOut;    /* File for Gibbs output */
+  PSTR szGout;          /* Filename for output */
+  PFILE pfileOut;       /* File pointer for output */
 
-  PSTR szGrestart;      /* Parm file  for restarting Gibbs */
-  PFILE pfileRestart;   /* File for Gibbs restart */
+  PSTR szGrestart;      /* Filename for restart parameter vectors */
+  PFILE pfileRestart;   /* File pointer for restart */
 
-  PSTR szGdata;         /* Filename for Gibbs input data */
+  PSTR szGdata;         /* Filename for input data */
 
 } GIBBSDATA, *PGIBBSDATA; /* tagGIBBSDATA */
 
 
 /* Specification for Monte Carlo type experiment */
+enum {forward, backward};
 
 typedef struct tagMONTECARLO {
   long nRuns;               /* Number of Monte Carlo runs */
@@ -337,6 +350,8 @@ typedef struct tagMONTECARLO {
 
   long nSetParms;           /* Count of setpoint parameters */
 
+  int  style;               /* either forward or backward for optimal design */
+
 } MONTECARLO, *PMONTECARLO; /* tagMONTECARLO */
 
 
@@ -384,19 +399,19 @@ typedef struct tagEXPERIMENT {
 typedef struct tagLEVEL {
   int    iDepth;                           /* Depth of this level */
   int    iSequence;                        /* Instance # of this level */
-  int    iInstances;                       /* Number of instances of next level */
+  int    iInstances;                       /* # of instances of next level */
   struct tagLEVEL *pLevels[MAX_INSTANCES]; /* Pointers to sublevels */
 
-  PLIST  plistVars;                        /* Vars, other than those in Distrib */
+  PLIST  plistVars;    /* Vars, other than those in Distrib */
   long   nFixedVars;
   PVARMOD *rgpFixedVars;
 
-  PLIST   plistMCVars;                     /* List of MCVAR record, variation specs */
-  long    nMCVars;                         /* Count of parameters */
-  PMCVAR *rgpMCVars;                       /* A priori distributions for each */
+  PLIST   plistMCVars; /* List of MCVAR record, variation specs */
+  long    nMCVars;     /* Count of parameters */
+  PMCVAR *rgpMCVars;   /* A priori distributions for each */
 
-  PEXPERIMENT pexpt;                       /* Ptr to expt struct, NULL if not expt
-				                                      EXPERIMENT is used for compatibility */ 
+  PEXPERIMENT pexpt;   /* Ptr to expt struct, NULL if not expt
+				                  EXPERIMENT is used for compatibility */ 
 } LEVEL, *PLEVEL; /* tagLEVEL */
 
 
@@ -409,10 +424,6 @@ typedef struct tagANALYSIS {
   BOOL bPrintIter;	    /* Debug flag for printing iteration numbers */
 
   int iType;            /* Type of analysis. One of AT_ types */
-  int fOptions;         /* Options flags -- combination of OF_ above */
-
-  int fCmdOptions;      /* Command line opts, overwrite defaults */
-  int fNotOptions;      /* Command line opts, inhibits these options */
 
   WORD wContext;        /* Context flag used during input processing */
   double dSeed;         /* Random seed used for all analyses */
@@ -458,7 +469,6 @@ extern PSTRLEX vrgszlexArgs[];
 */
 
 void AnnounceProgram (void);
-void CheckDependencies (PANALYSIS panal);
 void CorrectInputToTransition (PEXPERIMENT, PDOUBLE);
 
 int  DoOneExperiment (PEXPERIMENT pexp);
@@ -473,7 +483,7 @@ int  Euler (long neq, double *y, double *t, double tout, double dTStep);
 
 void FreeVarMod (PVOID pData);
 
-BOOL GetCmdLineArgs (int cArg, char *const *rgszArg, PSTR *pszFileIn,
+void GetCmdLineArgs (int cArg, char *const *rgszArg, PSTR *pszFileIn,
                      PSTR *pszFileOut, PANALYSIS panal);
 void GetOutputFlagOption (PANALYSIS panal, char *optarg);
 
