@@ -5,15 +5,43 @@
 
    Copyright (c) 1993.  Don Maszle, Frederic Bois.  All rights reserved.
 
+   This program is free software; you can redistribute it and/or
+   modify it under the terms of the GNU General Public License
+   as published by the Free Software Foundation; either version 2
+   of the License, or (at your option) any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+
+   contact
+     Frdric Bois / Don Maszle
+     BEHS, School of Public Health
+     University of California at Berkeley
+     Berkeley, CA 94720
+
+     fbois@diana.lbl.gov
+
    -- Revisions -----
-     Logfile:  %F%
-    Revision:  %I%
-        Date:  %G%
-     Modtime:  %U%
+     Logfile:  SCCS/s.simi.c
+    Revision:  1.46
+        Date:  29 Jul 1997
+     Modtime:  08:08:47
       Author:  @a
    -- SCCS  ---------
 
    Contains input routines for simulation.
+
+   Modified on 18/3/97 by FB: improved comment for exponential density in 
+     GetMCVarySpec.
+
+   Modified on 21/3/97 by FB: made undefined identifier a fatal error, 
+     and collapsed two memory allocation checking in one, in GetPrint.
 */
 
 #include <assert.h>
@@ -48,8 +76,9 @@ KM vrgkmKeywordMap[] = { /* Global Keyword - code map */
 
   {"OutputFile",    KM_OUTPUTFILE,  CN_GLOBAL},    /* Regular out file */
 
-  {"Gibbs",         KM_GIBBS,       CN_GLOBAL},    /* Gibbs estim spec */
+  {"Gibbs",         KM_MCMC,        CN_GLOBAL},    /* Gibbs estim spec */
   {"MCMC",          KM_MCMC,        CN_GLOBAL},    /* Metrop estim spec */
+  {"OptimDesign",   KM_OPTDESIGN,   CN_GLOBAL},    /* Optimal design spec */
   {"MonteCarlo",    KM_MONTECARLO,  CN_GLOBAL},    /* Monte Carlo spec */
   {"Distrib",       KM_MCVARY,      CN_GLOBAL},    /* MC Variable mod */
   {"MCVary",        KM_MCVARY,      CN_GLOBAL},    /* Obsolete!! */
@@ -94,12 +123,15 @@ KM vrgkmKeywordMap[] = { /* Global Keyword - code map */
   {"LogNormal_v",     KM_LOGNORMALV,      CN_FUNCARG},
   {"TruncNormal_v",   KM_TRUNCNORMALV,    CN_FUNCARG},
   {"TruncLogNormal_v",KM_TRUNCLOGNORMALV, CN_FUNCARG},
+  {"BinomialBeta",    KM_BINOMIALBETA,    CN_FUNCARG},
   
   {"Prediction",    KM_PREDICTION,        CN_FUNCARG},
 
   {"Euler",         KM_EULER,             CN_FUNCARG},
   {"Lsodes",        KM_LSODES,            CN_FUNCARG},
 
+  {"Forward",       KM_FORWARD,           CN_FUNCARG},
+  {"Backward",      KM_BACKWARD,          CN_FUNCARG},
 
   /* Variables names valid in all CN_ */
 
@@ -231,7 +263,8 @@ int McvFromLex (PSTR szLex)
              : ikwcode == KM_NORMALV        ? MCV_NORMALV
              : ikwcode == KM_LOGNORMALV     ? MCV_LOGNORMALV
              : ikwcode == KM_TRUNCNORMALV   ? MCV_TRUNCNORMALV
-             : ikwcode == KM_TRUNCLOGNORMALV ? MCV_TRUNCLOGNORMALV
+             : ikwcode == KM_TRUNCLOGNORMALV? MCV_TRUNCLOGNORMALV
+             : ikwcode == KM_BINOMIALBETA   ? MCV_BINOMIALBETA
              : (-1));
 
   return iReturn;
@@ -286,14 +319,14 @@ BOOL GetSimType (PINPUTBUF pibIn)
       iAT = AT_SETPOINTS;
       break;
 
-    case KM_GIBBS:
-      iAT = AT_GIBBS;
-      break;
-
     case KM_MCMC:
       iAT = AT_MCMC;
       break;
 
+    case KM_OPTDESIGN:
+      iAT = AT_OPTDESIGN;
+      break;
+
     case KM_DEFAULTSIM:
       iAT = AT_DEFAULTSIM;
       break;
@@ -533,11 +566,10 @@ BOOL GetPrint (PINPUTBUF pibIn, PSTR szLex, POUTSPEC pos)
       if(nVars == MAX_PRINT_VARS)
         ReportError(pibIn, RE_TOOMANYPVARS | RE_FATAL, "GetPrint", NULL);
       if ((bErr = !(hvar = GetVarHandle (szLex))))
-        ReportError (pibIn, RE_UNDEFINED, szLex, NULL);
+        ReportError (pibIn, RE_UNDEFINED | RE_FATAL, szLex, NULL);
       else {
-        if ( !(pprintrec[nVars] = (PPRINTREC) malloc (sizeof(PRINTREC))))
-          ReportError (pibIn, RE_OUTOFMEM | RE_FATAL, "GetPrint", NULL);
-        if ( !(pprintrec[nVars]->szOutputName =
+        if ( !(pprintrec[nVars] = (PPRINTREC) malloc (sizeof(PRINTREC))) ||
+             !(pprintrec[nVars]->szOutputName =
               (PSTR) malloc (MyStrlen(szLex)+1)))
           ReportError (pibIn, RE_OUTOFMEM | RE_FATAL, "GetPrint", NULL);
         MyStrcpy (pprintrec[nVars]->szOutputName, szLex);
@@ -558,7 +590,7 @@ BOOL GetPrint (PINPUTBUF pibIn, PSTR szLex, POUTSPEC pos)
         free (pprintrec[n]->szOutputName);
         free (pprintrec[n]);
       }
-    } /* if */
+    }
     else
       for(n = 0; n < nVars; ++n)
         QueueListItem (pos->plistPrintRecs, (PVOID) pprintrec[n]);
@@ -812,7 +844,7 @@ static int vrgiSimArgTypes[NSIM_ARGS] = {LX_NUMBER};
    get the Gibbs specification.
 */
 
-BOOL GetGibbsSpec (PINPUTBUF pibIn, PEXPERIMENT pexp, int iKwCode)
+BOOL GetGibbsSpec (PINPUTBUF pibIn, PEXPERIMENT pexp)
 {
 #define NGIBBS_ARGS 8 /* # Func args to gibbs spec */
 
@@ -826,7 +858,7 @@ static int vrgiGibbsArgTypes[NGIBBS_ARGS] = { LX_STRING, LX_STRING, LX_STRING,
   BOOL bErr= !GetFuncArgs(pibIn, NGIBBS_ARGS,
                           vrgiGibbsArgTypes,vrgszlexArgs[0]);
 
-  static char vszGibbsOutDefault[] = "Gibbs.Out.Default";
+  static char vszGibbsOutDefault[] = "MCMC.default.out";
 
   if (!bErr) {
     if (*vrgszlexArgs[0]) { /* Get output Filename */
@@ -860,24 +892,132 @@ static int vrgiGibbsArgTypes[NGIBBS_ARGS] = { LX_STRING, LX_STRING, LX_STRING,
     panal->gd.nPrintFreq = atol(vrgszlexArgs[5]);
     panal->gd.nPrintIter = atol(vrgszlexArgs[6]);
 
-    if (!(panal->fCmdOptions & OF_CMDLSEED))
-      panal->dSeed = atof(vrgszlexArgs[7]);
+    panal->dSeed = atof(vrgszlexArgs[7]);
 
+    if (panal->gd.nInitIter && (panal->gd.szGrestart == NULL)) {
+      printf ("Error: if printPredFlag is not zero a restart file must be "
+              "given - Exiting\n\n");
+      exit (0);
+    }
   } /* if */
   else
     printf ("Syntax: %s (szOut, szRestart, szDat, \n"
-            "nMaxIters, nInitIter, nPrintFreq, dSeed)\n\n",
-            iKwCode == KM_GIBBS ? GetKeyword (KM_GIBBS) : 
+            "nMaxIters, [0,1], nPrintFreq, nIterToPrint, dSeed)\n\n",
             GetKeyword (KM_MCMC));
 
   if (!bErr && !panal->iType)
-   iKwCode == KM_GIBBS ? (panal->iType = AT_GIBBS) : (panal->iType = AT_MCMC);
+   panal->iType = AT_MCMC;
 
   return (!bErr);
 
 } /* GetGibbsSpec */
 
 
+/* -----------------------------------------------------------------------------
+   GetOptDSpec
+
+   get the optimal design specification. It is based on the GetSetPointsSpec
+   routine. 
+   The modification list is kept in MCVAR variation records, although this is 
+   not really a Monte Carlo analysis.  This structure should eventually be 
+   changed to reflect a more general variation specification.
+*/
+BOOL GetOptDSpec (PINPUTBUF pibIn, PANALYSIS  panal, PSTR szLex)
+{
+  PMCVAR pmcvar;
+  HVAR hvar;
+  int iErr = 0;
+  int iNLI;
+  int ikwcode;
+
+  /* Try to get open paren and filenames */
+  if ((iErr = EGetPunct (pibIn, szLex, CH_LPAREN)                   || 
+              GetStringArg (pibIn, &panal->gd.szGout, szLex, FALSE) ||
+              GetStringArg (pibIn, &panal->gd.szGrestart, szLex, TRUE))) {
+    goto Exit_GetOptDSpec;
+  }
+
+  /* There has to be a restart file */
+  if (!panal->gd.szGrestart)
+    ReportError (pibIn, RE_SPECERR | RE_FATAL, "Missing restart file", NULL);
+
+  /* Try to get number of parameter samples to read in */
+  GetOptPunct (pibIn, szLex, ',');
+  if ((iErr = ENextLex (pibIn, szLex, LX_INTEGER)))
+    goto Exit_GetOptDSpec;
+  panal->mc.nRuns = atol (szLex);
+
+  /* Try to get the random seed */
+  GetOptPunct (pibIn, szLex, ',');
+  if ((iErr = ENextLex (pibIn, szLex, LX_NUMBER)))
+    goto Exit_GetOptDSpec;
+  panal->dSeed = atof (szLex);
+
+  /* Try to get the style (Forward or Backward) */
+  GetOptPunct (pibIn, szLex, ',');
+  if ((iErr = ENextLex (pibIn, szLex, LX_IDENTIFIER)))
+    goto Exit_GetOptDSpec;
+  ikwcode = GetKeywordCode (szLex, NULL);
+  if (ikwcode == KM_FORWARD)
+    panal->mc.style = forward;
+  else if (ikwcode == KM_BACKWARD)
+         panal->mc.style = backward;
+       else {
+         iErr = TRUE;
+         goto Exit_GetOptDSpec;
+       }
+
+  /* Try to get identifier list */
+  while ((iNLI=NextListItem (pibIn, szLex, LX_IDENTIFIER, 1, CH_RPAREN)) > 0) {
+    hvar = GetVarHandle(szLex);
+    if ((iErr = (!hvar || IsInput(hvar))))
+      break; /* Is this reported ? */
+
+    if ( !(pmcvar = (PMCVAR) malloc (sizeof(MCVAR))))
+      ReportError (pibIn, RE_OUTOFMEM | RE_FATAL, "GetOptDSpec", NULL);
+
+    pmcvar->hvar = hvar;
+    pmcvar->iType = MCV_SETPOINTS;
+    pmcvar->dParm[2] = pmcvar->dParm[3] = 0.0;
+
+    QueueListItem (panal->mc.plistMCVars, pmcvar);
+
+  } /* while */
+
+  panal->mc.nSetParms = ListLength (panal->mc.plistMCVars);
+
+  if (panal->mc.nSetParms == 0) {
+    iErr = TRUE;
+    printf (
+    "\nError: you must specify a list of parameters to read.\n\n");
+    goto Exit_GetOptDSpec;
+  }
+  
+  if (!iNLI) /* List terminator */
+    iErr = EGetPunct (pibIn, szLex, CH_RPAREN);
+  else {
+    iErr = TRUE;
+    ReportError (pibIn, RE_LEXEXPECTED, "identifier", szLex);
+  } /* else */
+
+Exit_GetOptDSpec: ;
+
+  if (iErr) {
+    printf ("Syntax:\n"
+            "%s (\"Output_File\", \"Param_Sample_File\", nSamples, "
+            "random_seed, <Forward or Backward>, "
+            "<param-id-list...>)\n\n", GetKeyword (KM_OPTDESIGN));
+    printf ("Exiting...\n");
+    exit (0);
+  }
+  else if (!panal->iType)
+    panal->iType = AT_OPTDESIGN; /* Flag SetPoints anal if not chosen */
+
+  return (iErr);
+
+} /* GetOptDSpec */
+
+
 /* ----------------------------------------------------------------------------
    GetMCVarySpec
 
@@ -902,7 +1042,6 @@ int GetMCVarySpec (PINPUTBUF pibIn, PANALYSIS panal, PSTR szLex)
 
   if (panal->iType && !((panal->iType == AT_MONTECARLO) ||
                         (panal->iType == AT_SETPOINTS)  ||
-                        (panal->iType == AT_GIBBS)      ||
                         (panal->iType == AT_MCMC))) {
     EatStatement (pibIn);  /* Ignore this Distrib() stmt */
     goto Exit_MCVarySpec;
@@ -923,11 +1062,11 @@ int GetMCVarySpec (PINPUTBUF pibIn, PANALYSIS panal, PSTR szLex)
   } /* if */
 
   /* This conditions the list in which pmcvar will be queued */
-  if(IsModelVar(hvar))
+  if (IsModelVar (hvar))         /* Distrib is for a likelihood definition */
     plist = panal->plistModelVars;
-  else if(panal->iCurrentDepth == 0)
+  else if (panal->iCurrentDepth == 0)            /* not an MCMC simulation */ 
     plist = panal->mc.plistMCVars;
-  else
+  else                                               /* an MCMC simulation */
     plist = panal->pCurrentLevel[panal->iCurrentDepth-1]->plistMCVars;
 
   if (!(pmcvar = (PMCVAR) malloc (sizeof(MCVAR))))
@@ -946,6 +1085,7 @@ int GetMCVarySpec (PINPUTBUF pibIn, PANALYSIS panal, PSTR szLex)
   pmcvar->bIsFixed = FALSE;
   pmcvar->lJumps = 0;
   pmcvar->dKernelSD = INIT_KERNELSD;
+  pmcvar->bGibbs = FALSE;
   for (n = 0; n < 4; n++) {
     pmcvar->hParm[n] = 0;
     pmcvar->pMCVParent[n] = NULL;
@@ -970,20 +1110,14 @@ int GetMCVarySpec (PINPUTBUF pibIn, PANALYSIS panal, PSTR szLex)
     case MCV_UNIFORM:
     case MCV_LOGUNIFORM: /* 2 parameters: min and max */
 
-      if((iErr=GetMCVaryParam(pibIn, szLex, plist, 2, pmcvar)))
+      if((iErr=GetMCVaryParam(pibIn, szLex, plist, 0, pmcvar)))
         goto Done_GetMCVary;
 
-      if((iErr=GetMCVaryParam(pibIn, szLex, plist, 3, pmcvar)))
+      if((iErr=GetMCVaryParam(pibIn, szLex, plist, 1, pmcvar)))
         goto Done_GetMCVary;
 
-      if (pmcvar->cVarParm == 0) { /* no symbolic parameters */
-        pmcvar->dParm[0] = (pmcvar->dParm[2] + pmcvar->dParm[3]) / 2; 
-        pmcvar->dParm[1] = (pmcvar->dParm[3] - pmcvar->dParm[2]) / sqrt (12);
-      }
-      else { /* arbitrary values */
-        pmcvar->dParm[0] = 0;
-        pmcvar->dParm[1] = 1;
-      }
+      pmcvar->dParm[2] = -DBL_MAX;
+      pmcvar->dParm[3] = DBL_MAX;
 
       break;
 
@@ -1077,7 +1211,7 @@ int GetMCVarySpec (PINPUTBUF pibIn, PANALYSIS panal, PSTR szLex)
       if (pmcvar->cVarParm < 2)
         pmcvar->dParm[3] = pmcvar->dParm[1];
       else
-        pmcvar->dParm[3] = DBL_MAX;
+        pmcvar->dParm[3] = DBL_MAX; /* FB 18/07/97 */
 
 
       break;
@@ -1100,7 +1234,7 @@ int GetMCVarySpec (PINPUTBUF pibIn, PANALYSIS panal, PSTR szLex)
       break;
 
     /* ----------------------------------------------------------------------*/
-    case MCV_EXPONENTIAL: /* only one parameter: scale */
+    case MCV_EXPONENTIAL: /* only one parameter: inverse scale */
 
       if ((iErr = GetMCVaryParam (pibIn, szLex, plist, 0, pmcvar)))
         goto Done_GetMCVary;
@@ -1139,6 +1273,23 @@ int GetMCVarySpec (PINPUTBUF pibIn, PANALYSIS panal, PSTR szLex)
 
       break;
 
+    /* ----------------------------------------------------------------------*/
+    case MCV_BINOMIALBETA: /* 3 parameter: mean, alpha, beta */
+
+      if ((iErr = GetMCVaryParam (pibIn, szLex, plist, 0, pmcvar)))
+        goto Done_GetMCVary;
+
+      if ((iErr = GetMCVaryParam (pibIn, szLex, plist, 1, pmcvar)))
+        goto Done_GetMCVary;
+
+      if ((iErr = GetMCVaryParam (pibIn, szLex, plist, 2, pmcvar)))
+        goto Done_GetMCVary;
+
+      /* set the last parameter, unused */
+      pmcvar->dParm[3] = DBL_MAX;
+
+      break;
+
     /* ----------------------------------------------------------------------*/
     default:
         ReportRunTimeError(panal, RE_UNKNOWNDIST | RE_FATAL, "GetMCVarySpec");
@@ -1312,13 +1463,11 @@ BOOL CheckDistribParam(PLIST plist, HVAR hvar1, HVAR hvar2) {
 /* ----------------------------------------------------------------------------
    GetSetPointsSpec
 
-   Reads the SetPoints() arguments.  The modification list is kept
+   Reads the SetPoints() arguments. The modification list is kept
    in MCVAR variation records, although this is not really a Monte
    Carlo analysis.  This structure should eventually be changed to
-   reflect a more general variation specification.  The optimize
-   routines will need to use a similar thing.
+   reflect a more general variation specification.
 */
-
 int GetSetPointsSpec (PINPUTBUF pibIn, PANALYSIS  panal, PSTR szLex)
 {
   PMCVAR pmcvar;
@@ -1326,38 +1475,27 @@ int GetSetPointsSpec (PINPUTBUF pibIn, PANALYSIS  panal, PSTR szLex)
   int iErr = 0;
   int iNLI;
 
-  /* If there is an analysis type specified, don't bother to get this spec.
-     Actually it is disasterous to do so in some cases so let's just not,
-     so that we don't nuke info.
-   */
-  if (panal->iType && panal->iType != AT_SETPOINTS) {
-    EatStatement (pibIn);
-    goto Exit_GetSetPointsSpec;
-  } /* if */
-
+  /* MonteCarlo sampling can be mixed with SetPoints sampling if Distrib
+     specs appear after the SetPoint spec */
   if (ListLength (panal->mc.plistMCVars) > 0) {
-    ReportError (pibIn, RE_WARNING | RE_DEFOTHER,
-         "Monte Carlo variations", NULL);
-    printf (
-         "** Oops.  If you have MCVary() specs before the SetPoints() spec"
-         "\n   move them all *after* the SetPoints() spec.\n");
-    exit (-1);
-
-  } /* if */
+    printf ("Error: Distrib() statements can only appear after the SetPoints()"
+            "specification, not before - Exiting\n\n");
+    exit (0);
+  }
 
   /* Try to get open paren and filenames */
-
-  if ((iErr = EGetPunct (pibIn, szLex, CH_LPAREN)
-              || GetStringArg (pibIn, &panal->mc.szMCOutfilename, szLex, FALSE)
-              || GetStringArg (pibIn, &panal->mc.szSetPointsFilename, szLex, TRUE)))
-
+  if ((iErr = EGetPunct (pibIn, szLex, CH_LPAREN) || 
+              GetStringArg (pibIn, &panal->mc.szMCOutfilename, szLex, FALSE) ||
+              GetStringArg (pibIn, &panal->mc.szSetPointsFilename, szLex, 
+                            TRUE))) {
     goto Exit_GetSetPointsSpec;
+  }
 
+  /* There has to be a restart file */
   if (!panal->mc.szSetPointsFilename)
-    ReportError (pibIn, RE_SPECERR, "No set points file", NULL);
+    ReportError (pibIn, RE_SPECERR | RE_FATAL, "Missing setpoints file", NULL);
 
   /* Try to get number of runs */
-
   GetOptPunct (pibIn, szLex, ',');
   if ((iErr = ENextLex (pibIn, szLex, LX_INTEGER)))
     goto Exit_GetSetPointsSpec;
@@ -1368,8 +1506,7 @@ int GetSetPointsSpec (PINPUTBUF pibIn, PANALYSIS  panal, PSTR szLex)
   /* FB 20/11/96 removed this because it gives errors if only a comma
      separates the number of runs from the list of params :
      GetOptPunct (pibIn, szLex, ','); */
-  while ((iNLI = NextListItem (pibIn, szLex, LX_IDENTIFIER, 1, CH_RPAREN))
-         > 0) {
+  while ((iNLI=NextListItem (pibIn, szLex, LX_IDENTIFIER, 1, CH_RPAREN)) > 0) {
     hvar = GetVarHandle(szLex);
     if ((iErr = (!hvar || IsInput(hvar))))
       break; /* Is this reported ? */
@@ -1391,7 +1528,7 @@ int GetSetPointsSpec (PINPUTBUF pibIn, PANALYSIS  panal, PSTR szLex)
   if (panal->mc.nSetParms == 0) {
     iErr = TRUE;
     printf (
-    "Error: you must specify a list of parameters to for SetPoints to read\n");
+    "\nError: you must specify a list of parameters to read.\n\n");
     goto Exit_GetSetPointsSpec;
   }
   
@@ -1402,13 +1539,12 @@ int GetSetPointsSpec (PINPUTBUF pibIn, PANALYSIS  panal, PSTR szLex)
     ReportError (pibIn, RE_LEXEXPECTED, "identifier", szLex);
   } /* else */
 
-Exit_GetSetPointsSpec:
-  ;
+Exit_GetSetPointsSpec: ;
 
   if (iErr) {
     printf ("Syntax:\n"
-             " %s (\"OutputFile\", \"SetPtsFile\", "
-             "nRuns, <parameter-id-list...>)\n\n", GetKeyword (KM_SETPOINTS));
+             "%s (\"OutputFile\", \"SetPtsFile\", nRuns, "
+             "<param-id-list...>)\n\n", GetKeyword (KM_SETPOINTS));
     printf ("Exiting...\n");
     exit (0);
   }
@@ -1421,8 +1557,8 @@ Exit_GetSetPointsSpec:
 
 
 /* -----------------------------------------------------------------------------
+   GetMonteCarloSpec
 */
-
 int GetMonteCarloSpec (PINPUTBUF pibIn, PANALYSIS panal, PSTR szLex)
 {
 #define NMC_ARGS 3     /* 3 MonteCarlo Args */
@@ -1452,8 +1588,7 @@ static int vrgiMCArgTypes[NMC_ARGS] = {LX_STRING, LX_INTEGER, LX_NUMBER};
     } /* if */
 
     panal->mc.nRuns = atol (vrgszlexArgs[1]);
-    if (!(panal->fCmdOptions & OF_CMDLSEED))
-      panal->dSeed = atof(vrgszlexArgs[2]);
+    panal->dSeed = atof(vrgszlexArgs[2]);
   } /* if */
   else
     printf ("Syntax: %s (szOutfilename, nRuns, dSeed)\n\n", 
@@ -1470,6 +1605,7 @@ Exit_GetMonteCarloSpec: ;
 
 
 /* -----------------------------------------------------------------------------
+   GetParmMod
 */
 
 BOOL GetParmMod (PINPUTBUF pibIn, PSTRLEX szLex, PSTREQN szeqn)
@@ -1597,7 +1733,6 @@ void NewExperiment (PINPUTBUF pibIn)
 
    cleans up at the end of defining a new experiment section.
 */
-
 BOOL EndExperiment (PINPUTBUF pibIn, PANALYSIS panal)
 {
   BOOL bReturn;
@@ -1630,62 +1765,81 @@ BOOL EndExperiment (PINPUTBUF pibIn, PANALYSIS panal)
 
 } /* EndExperiment */
 
+
 /* -----------------------------------------------------------------------------
    SetLevel
 
    Encountered `Level' keyword, increments number of levels, allocates
    structure, initializes
 */
-
-int SetLevel(PINPUTBUF pibIn) {
+int SetLevel(PINPUTBUF pibIn) 
+{
   PSTRLEX szPunct;
   PANALYSIS panal = (PANALYSIS)pibIn->pInfo;
   PLEVEL plevel;
   BYTE n;
 
-  if(panal->iType != AT_MCMC)
+  if (panal->iType != AT_MCMC)
     ReportError(pibIn, RE_TYPENOTMCMC | RE_FATAL, "SetLevel", NULL);
 
-  if(panal->iCurrentDepth == MAX_LEVELS)
+  if (panal->iCurrentDepth == MAX_LEVELS)
     ReportError(pibIn, RE_TOOMANYLEVELS | RE_FATAL, "SetLevel", NULL);
 
-  if(panal->wContext == CN_EXPERIMENT)
+  if (panal->wContext == CN_EXPERIMENT)
     ReportError(pibIn, RE_LEVINEXPT | RE_FATAL, "SetLevel", NULL);
 
-  if(EGetPunct(pibIn, szPunct, CH_LBRACE))
+  if (EGetPunct(pibIn, szPunct, CH_LBRACE))
     return 1;
-  if(panal->iCurrentDepth == 0) {
-    if(panal->iInstances == MAX_INSTANCES - 1)
+
+  if (panal->iCurrentDepth == 0) {
+
+    if (panal->iInstances == MAX_INSTANCES - 1)
       ReportError(pibIn, RE_TOOMANYINST | RE_FATAL, "SetLevel", NULL);
-    plevel = panal->pLevels[panal->iInstances++] =
-                                          (PLEVEL)malloc(sizeof(LEVEL));
+
+    plevel = panal->pLevels[panal->iInstances++] 
+           = (PLEVEL) malloc (sizeof (LEVEL));
+    if (plevel == NULL)
+      ReportError(pibIn, RE_OUTOFMEM | RE_FATAL, "SetLevel", NULL);
+
     plevel->iSequence = panal->iInstances;
-    printf("New level - depth 1, instance %d\n", panal->iInstances);
+    printf ("New level - depth 1, instance %d\n", panal->iInstances);
+
   }
   else {
+
     plevel = panal->pLevels[panal->iInstances-1];
-    for(n = 0; n < panal->iCurrentDepth-1; ++n) {
+
+    for (n = 0; n < panal->iCurrentDepth-1; ++n)
       plevel = plevel->pLevels[plevel->iInstances-1];
-    }
-    if(plevel->iInstances == MAX_INSTANCES - 1)
+
+    if (plevel->iInstances == MAX_INSTANCES - 1)
       ReportError(pibIn, RE_TOOMANYINST | RE_FATAL, "SetLevel", NULL);
+
     n = panal->pCurrentLevel[panal->iCurrentDepth-1]->iInstances++;
-    plevel = plevel->pLevels[n] = (PLEVEL)malloc(sizeof(LEVEL));
+
+    plevel = plevel->pLevels[n] 
+           = (PLEVEL)malloc(sizeof(LEVEL));
+
+    if (plevel == NULL)
+      ReportError(pibIn, RE_OUTOFMEM | RE_FATAL, "SetLevel", NULL);
+
     plevel->iSequence = n + 1;
     printf("New level - depth %d, instance %d\n", panal->iCurrentDepth+1,
            panal->pCurrentLevel[panal->iCurrentDepth-1]->iInstances);
   }
-  if(plevel == NULL)
-    ReportError(pibIn, RE_OUTOFMEM | RE_FATAL, "SetLevel", NULL);
+
   plevel->iInstances = 0;
   plevel->iDepth = panal->iCurrentDepth;
   panal->pCurrentLevel[panal->iCurrentDepth++] = plevel;
-  if(panal->iDepth < panal->iCurrentDepth)
+
+  if (panal->iDepth < panal->iCurrentDepth)
     panal->iDepth = panal->iCurrentDepth;
+
   plevel->nMCVars = plevel->nFixedVars = 0;
   plevel->plistVars = InitList();
   plevel->plistMCVars = InitList();
   plevel->pexpt = NULL;
+
   return 0;
   
 } /* SetLevel */
@@ -1694,31 +1848,33 @@ int SetLevel(PINPUTBUF pibIn) {
 /* -----------------------------------------------------------------------------
    EndLevel
 */
-
-BOOL EndLevel(PANALYSIS panal) {
+BOOL EndLevel (PANALYSIS panal)
+{
   if(panal->iCurrentDepth-- == 0)
     return FALSE;
   return TRUE;
-}
+} /* EndLevel */
 
 
 /* -----------------------------------------------------------------------------
    FreeLevels
 */
-
-void FreeLevels(PANALYSIS panal) {
+void FreeLevels (PANALYSIS panal)
+{
   BYTE n;
-  for(n = 0; n < panal->iInstances; ++n)
-    FreeOneLevel(panal->pLevels[n]);
+  for (n = 0; n < panal->iInstances; n++)
+    if (panal->pLevels[n] != NULL)
+      FreeOneLevel (panal->pLevels[n]);
 } /* FreeLevels */
 
 
 /* -----------------------------------------------------------------------------
    FreeMCLists
 */
-int FreeMCLists(PVOID pData, PVOID pUserInfo) {
-  PMCVAR pMCVar = (PMCVAR)pData;
-  FreeList(&pMCVar->plistDependents, NULL, TRUE);
+int FreeMCLists (PVOID pData, PVOID pUserInfo)
+{
+  PMCVAR pMCVar = (PMCVAR) pData;
+  FreeList (&pMCVar->plistDependents, NULL, TRUE);
   /* dummy */
   return 0;
 }
@@ -1727,21 +1883,25 @@ int FreeMCLists(PVOID pData, PVOID pUserInfo) {
 /* -----------------------------------------------------------------------------
    FreeOneLevel (recursive)
 */
-
-void FreeOneLevel(PLEVEL plevel) {
+void FreeOneLevel (PLEVEL plevel)
+{
   BYTE n;
-  for(n = 0; n < plevel->iInstances; ++n)
-    FreeOneLevel(plevel->pLevels[n]);
-  FreeList(&plevel->plistVars, NULL, TRUE);
-  ForAllList(plevel->plistMCVars, &FreeMCLists, NULL);
-  FreeList(&plevel->plistMCVars, NULL, TRUE);
-  if(plevel->pexpt != NULL)
-    free(plevel->pexpt);
-  if(plevel->nMCVars > 0)
-    free(plevel->rgpMCVars);
-  if(plevel->nFixedVars > 0)
-    free(plevel->rgpFixedVars);
+
+  for (n = 0; n < plevel->iInstances; n++)
+    if (plevel->pLevels[n] != NULL)
+      FreeOneLevel (plevel->pLevels[n]);
+
+  FreeList (&plevel->plistVars, NULL, TRUE);
+  ForAllList (plevel->plistMCVars, &FreeMCLists, NULL);
+  FreeList (&plevel->plistMCVars, NULL, TRUE);
+  if (plevel->pexpt != NULL)
+    free (plevel->pexpt);
+  if (plevel->nMCVars > 0)
+    free (plevel->rgpMCVars);
+  if (plevel->nFixedVars > 0)
+    free (plevel->rgpFixedVars);
   free(plevel);
+
 } /* FreeOneLevel */
 
 
@@ -1758,7 +1918,6 @@ void FreeOneLevel(PLEVEL plevel) {
    If the word szLex is not a keyword, ProcessWord() attempts to
    define a parameter specification.
 */
-
 void ProcessWord (PINPUTBUF pibIn, PSTR szLex, PSTR szEqn)
 {
   int iErr = 0;
@@ -1784,14 +1943,18 @@ void ProcessWord (PINPUTBUF pibIn, PSTR szLex, PSTR szEqn)
     switch (iKWCode) {
 
     default:
-      /* If a keyword is not found, try to get a parmeter assignment */
+      /* If a keyword is not found, try to get a parameter assignment */
       iErr = GetParmMod (pibIn, szLex, szEqn);
       break;
 
     /*Process the following keywords */
 
-    case KM_END:
-      panal->wContext = CN_END;
+    case KM_PRINT:
+      iErr = GetPrint (pibIn, szLex, &panal->pexpCurrent->os);
+      break;
+
+    case KM_PRINTSTEP:
+      iErr = GetPrintStep (pibIn, szLex, &panal->pexpCurrent->os);
       break;
 
     case KM_EXPERIMENT:
@@ -1799,14 +1962,6 @@ void ProcessWord (PINPUTBUF pibIn, PSTR szLex, PSTR szEqn)
         NewExperiment (pibIn);
       break;
 
-    case KM_GIBBS: case KM_MCMC:
-      iErr = GetGibbsSpec (pibIn, panal->pexpCurrent, iKWCode);
-      break;
-
-    case KM_INTEGRATE:
-      iErr = GetIntegrate (pibIn, &panal->pexpCurrent->is);
-      break;
-
     case KM_LEVEL:
       iErr = SetLevel (pibIn);
       break;
@@ -1815,10 +1970,6 @@ void ProcessWord (PINPUTBUF pibIn, PSTR szLex, PSTR szEqn)
       iErr = GetMCVarySpec (pibIn, panal, szLex);
       break;
 
-    case KM_MONTECARLO:
-      iErr = GetMonteCarloSpec (pibIn, panal, szLex);
-      break;
-
     case KM_OUTPUTFILE:
       if (panal->pexpCurrent->os.szOutfilename) {
         ReportError (pibIn, RE_REDEF| RE_WARNING, "OutputFile", "* Ignoring");
@@ -1827,18 +1978,26 @@ void ProcessWord (PINPUTBUF pibIn, PSTR szLex, PSTR szEqn)
       else iErr = GetOutputFile (pibIn, szLex, &panal->pexpCurrent->os);
       break;
 
-    case KM_PRINT:
-      iErr = GetPrint (pibIn, szLex, &panal->pexpCurrent->os);
-      break;
-
-    case KM_PRINTSTEP:
-      iErr = GetPrintStep (pibIn, szLex, &panal->pexpCurrent->os);
-      break;
-
     case KM_DATA:
       iErr = GetData (pibIn, szLex, &panal->pexpCurrent->os);
       break;
 
+    case KM_INTEGRATE:
+      iErr = GetIntegrate (pibIn, &panal->pexpCurrent->is);
+      break;
+
+    case KM_MCMC:
+      iErr = GetGibbsSpec (pibIn, panal->pexpCurrent);
+      break;
+
+    case KM_OPTDESIGN:
+      iErr = GetOptDSpec (pibIn, panal, szLex);
+      break;
+
+    case KM_MONTECARLO:
+      iErr = GetMonteCarloSpec (pibIn, panal, szLex);
+      break;
+
     case KM_SETPOINTS:
       iErr = GetSetPointsSpec (pibIn, panal, szLex);
       break;
@@ -1850,6 +2009,11 @@ void ProcessWord (PINPUTBUF pibIn, PSTR szLex, PSTR szEqn)
     case KM_SIMTYPE:
       iErr = GetSimType (pibIn);
       break;
+
+    case KM_END:
+      panal->wContext = CN_END;
+      break;
+
     } /* switch */
   } /* else */
 
