@@ -4,11 +4,33 @@
 
    Copyright (c) 1993.  Don Maszle, Frederic Bois.  All rights reserved.
 
+   This program is free software; you can redistribute it and/or
+   modify it under the terms of the GNU General Public License
+   as published by the Free Software Foundation; either version 2
+   of the License, or (at your option) any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+
+   contact
+     Frdric Bois / Don Maszle
+     BEHS, School of Public Health
+     University of California at Berkeley
+     Berkeley, CA 94720
+
+     fbois@diana.lbl.gov
+
    -- Revisions -----
      Logfile:  SCCS/s.simi.h
-    Revision:  1.7
-        Date:  10 Jan 1997
-     Modtime:  03:05:32
+    Revision:  1.9
+        Date:  02 Jul 1997
+     Modtime:  08:37:19
       Author:  @a
    -- SCCS  ---------
 
@@ -40,7 +62,7 @@ void FreeLevels (PANALYSIS panal);
 int  FreeMCLists (PVOID pData, PVOID pUserInfo);
 void FreeOneLevel (PLEVEL plevel);
 BOOL GetData (PINPUTBUF pibIn, PSTR szLex, POUTSPEC pos);
-BOOL GetGibbsSpec (PINPUTBUF pibIn, PEXPERIMENT pexp, int iKwCode);
+BOOL GetGibbsSpec (PINPUTBUF pibIn, PEXPERIMENT pexp);
 BOOL GetIntegrate (PINPUTBUF pibIn, PINTSPEC pis);
 PSTR GetKeyword (int iKWCode);
 int  GetKeywordCode (PSTR szKeyword, PINT pfContext);
@@ -50,6 +72,7 @@ int  GetMCVaryParam(PINPUTBUF pibIn, PSTR szLex,
                     PLIST plist, int n, PMCVAR pmcvar);
 int  GetMCVarySpec (PINPUTBUF pibIn, PANALYSIS panal, PSTR szLex);
 int  GetMonteCarloSpec (PINPUTBUF pibIn, PANALYSIS panal, PSTR szLex);
+BOOL GetOptDSpec (PINPUTBUF pibIn, PANALYSIS  panal, PSTR szLex);
 BOOL GetOutputFile (PINPUTBUF pibIn, PSTR szLex, POUTSPEC pos);
 BOOL GetParmMod (PINPUTBUF pibIn, PSTRLEX szLex, PSTREQN szeqn);
 BOOL GetPrint (PINPUTBUF pibIn, PSTR szLex, POUTSPEC pos);
