@@ -5,11 +5,33 @@
 
    Copyright (c) 1993.  Don Maszle, Frederic Bois.  All rights reserved.
 
+   This program is free software; you can redistribute it and/or
+   modify it under the terms of the GNU General Public License
+   as published by the Free Software Foundation; either version 2
+   of the License, or (at your option) any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+
+   contact
+     Frdric Bois / Don Maszle
+     BEHS, School of Public Health
+     University of California at Berkeley
+     Berkeley, CA 94720
+
+     fbois@diana.lbl.gov
+
    -- Revisions -----
-     Logfile:  %F%
-    Revision:  %I%
-        Date:  %G%
-     Modtime:  %U%
+     Logfile:  SCCS/s.siminit.c
+    Revision:  1.16
+        Date:  29 Jul 1997
+     Modtime:  08:08:48
       Author:  @a
    -- SCCS  ---------
 
@@ -33,13 +55,15 @@
 
 void GetModelInfo (PMODELINFO pmi)
 {
-  if((pmi->nStates = (long) GetNStates()) != 0)
-    pmi->pStateHvar = (HVAR*)malloc(pmi->nStates * sizeof(HVAR));
-  pmi->nModelVars = (long) GetNModelVars();
+  pmi->nModelVars = (long) GetNModelVars ();
   pmi->pdModelVars = GetModelVector ();
 
-  GetStateHandles(pmi->pStateHvar);
+  if ((pmi->nStates = (long) GetNStates()) != 0) { /* FB improved 28/07/97 */
+    if (!(pmi->pStateHvar = (HVAR*) malloc (pmi->nStates * sizeof(HVAR))))
+      ReportError(NULL, RE_OUTOFMEM | RE_FATAL, "GetModelInfo", NULL);
 
+    GetStateHandles (pmi->pStateHvar);
+  }
 } /* GetModelInfo */
 
 
@@ -90,7 +114,6 @@ void InitOutputSpec (POUTSPEC pos)
   pos->piCurrentOut     = NULL;    /* Index to current output for each var */
   pos->prgdOutputTimes  = NULL;    /* Array of output times for each var */
   pos->prgdOutputVals   = NULL;    /* Array of output values for each var */
-  pos->prgdSavedVals    = NULL;    /* Array of saved output vals for each var */
   pos->prgdDataVals     = NULL;    /* Array of data values for each var */
 
   pos->cDistinctTimes   = 0;       /* Init output times list to empty */
@@ -155,7 +178,7 @@ void InitGibbs (PGIBBSDATA pgd)
   pgd->nPrintFreq = 1;      /* Print every iteration */
 
   pgd->szGout = NULL;       /* Filename for Gibbs output */
-  pgd->pfileGibOut = NULL;  /* File for Gibbs output */
+  pgd->pfileOut = NULL;     /* File for Gibbs output */
 
   pgd->szGrestart = NULL;   /* Parm file  for restarting Gibbs */
   pgd->pfileRestart = NULL; /* File for Gibbs restart */
@@ -165,52 +188,6 @@ void InitGibbs (PGIBBSDATA pgd)
 } /* InitGibbs */
 
 
-/* -----------------------------------------------------------------------------
-   SetCmdLineOptions
-
-   overrides the default options with those from the command-line.
-   First set the desired options from the command line, then
-   inhibits those that are asked to be turned off.
-*/
-
-void SetCmdLineOptions (PANALYSIS panal)
-{
-  panal->fOptions |= panal->fCmdOptions;  /* Use these */
-  panal->fOptions &= ~panal->fNotOptions; /* Inhibit these */
-} /* SetCmdLineOptions */
-
-
-/* -----------------------------------------------------------------------------
-   SetOptions
-
-   chooses various default options for the analysis, overriding with
-   any values that were given on the command line.
-*/
-
-void SetOptions (PANALYSIS panal)
-{
-  panal->fOptions = 0;
-
-  switch (panal->iType) {
-    case AT_DEFAULTSIM:
-      panal->fOptions |= (OF_VARNAMES | OF_VERT_OUTPUT);
-      break;
-
-    case AT_SETPOINTS:
-      panal->fOptions |= (OF_PARMLIST | OF_MCRESULT);
-      break;
-
-    case AT_MONTECARLO:
-      panal->fOptions |= (OF_PARMLIST | OF_MCRESULT);
-      break;
-
-  } /* switch */
-
-  SetCmdLineOptions (panal);        /* Maybe override the defaults */
-
-} /* SetOptions */
-
-
 /* -----------------------------------------------------------------------------
    InitAnalysis
 
@@ -228,9 +205,6 @@ void InitAnalysis (PANALYSIS panal)
   panal->bDependents = panal->bParams = FALSE;
 
   panal->iType = AT_NOTSPECD; /* Type of analysis */
-  panal->fOptions = 0; /* Options will be set by SetOptions */
-  panal->fCmdOptions = 0; /* CmdLine opts: overwrites defaults */
-  panal->fNotOptions = 0; /* CmdLine opts: inhibits defaults */
 
   panal->dSeed  = SEED_DEFAULT; /* The seed for everybody */
 
@@ -302,10 +276,7 @@ int InitOneOutVar (PVOID pData, PVOID pInfo)
 
   pos->prgdOutputTimes[i] = ppr->pdTimes;
   pos->prgdOutputVals[i]  = InitdVector (ppr->cTimes);
-  pos->prgdSavedVals[i]   = InitdVector (ppr->cTimes);
-
-  if (pos->prgdOutputTimes[i] == NULL || pos->prgdOutputVals[i] == NULL ||
-                                             pos->prgdSavedVals[i] == NULL)
+  if (pos->prgdOutputTimes[i] == NULL || pos->prgdOutputVals[i] == NULL)
     ReportError (NULL, RE_OUTOFMEM | RE_FATAL, "InitOneOutVar()", NULL);
 
   return 0;
@@ -340,9 +311,9 @@ int InitOneDataVar (PVOID pData, PVOID pInfo)
     if (pos->pcOutputTimes[i] == pda->cData)
       pos->prgdDataVals[i] = pda->pdData;
     else {
-      printf ("Error: There must be as many data in the Data ");
-      printf ("statement as times \n");
-      printf ("       in the Print or PrintStep statement for %s - Exiting.\n",
+      printf ("Error: There must be as many data in the Data " 
+              "statement as times \n" 
+              "       in the Print or PrintStep statement for %s - Exiting.\n",
               pda->szDataName);
       exit(0);
     }
@@ -461,19 +432,18 @@ BOOL PrepareOutSpec (PEXPERIMENT pexp)
     ReportError (NULL, RE_NOOUTPUTS, (PSTR) &pexp->iExp, NULL);
 
   else {
-    pos->pszOutputNames = (PSTR *) malloc(cOut * sizeof(PSTR));
+    pos->pszOutputNames = (PSTR *) malloc (cOut * sizeof(PSTR));
     pos->phvar          = (HVAR *) malloc(cOut * sizeof(HVAR));
     pos->pcOutputTimes  = InitiVector (cOut);
     pos->piCurrentOut   = InitiVector (cOut);
     pos->prgdOutputTimes= InitpdVector (cOut);
     pos->prgdOutputVals = InitpdVector (cOut);
-    pos->prgdSavedVals  = InitpdVector (cOut);
     pos->prgdDataVals   = InitpdVector (cOut);
 
-    if (pos->pszOutputNames == NULL     || pos->phvar == NULL
-        || pos->pcOutputTimes == NULL   || pos->piCurrentOut == NULL    
-        || pos->prgdOutputTimes == NULL || pos->prgdOutputVals == NULL  
-        || pos->prgdSavedVals == NULL   || pos->prgdDataVals == NULL)
+    if (pos->pszOutputNames  == NULL || pos->phvar == NULL          || 
+        pos->pcOutputTimes   == NULL || pos->piCurrentOut == NULL   ||    
+        pos->prgdOutputTimes == NULL || pos->prgdOutputVals == NULL ||  
+        pos->prgdDataVals    == NULL)
       ReportError (NULL, RE_OUTOFMEM | RE_FATAL, "PrepareOutSpec()", NULL);
 
     else {
@@ -496,6 +466,12 @@ BOOL PrepareOutSpec (PEXPERIMENT pexp)
       /* Adjust the final time to match the last output time */
       pexp->dTfinal = pos->rgdDistinctTimes[pos->cDistinctTimes - 1];
 
+      if (pexp->dTfinal == pexp->dT0) {
+        printf ("\nError: starting and final times are equal in Experiment %d "
+                "- Exiting.\n\n", pexp->iExp);
+        exit(0);
+      }
+
       bReturn = TRUE;
     } /* else */
   } /* else */
