@@ -4,11 +4,33 @@
 
    Copyright (c) 1993.  Don Maszle, Frederic Bois.  All rights reserved.
 
+   This program is free software; you can redistribute it and/or
+   modify it under the terms of the GNU General Public License
+   as published by the Free Software Foundation; either version 2
+   of the License, or (at your option) any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+
+   contact
+     Frdric Bois / Don Maszle
+     BEHS, School of Public Health
+     University of California at Berkeley
+     Berkeley, CA 94720
+
+     fbois@diana.lbl.gov
+
    -- Revisions -----
      Logfile:  SCCS/s.siminit.h
-    Revision:  1.6
-        Date:  23 Nov 1996
-     Modtime:  12:39:47
+    Revision:  1.8
+        Date:  02 Jul 1997
+     Modtime:  08:37:20
       Author:  @a
    -- SCCS  ---------
 
@@ -43,9 +65,6 @@ BOOL InitOutputs (PEXPERIMENT pexp, PINT piOut, PDOUBLE pdTout);
 void InitOutputSpec (POUTSPEC pos);
 BOOL PrepareOutSpec (PEXPERIMENT pexp);
 BOOL PrintOutSpec (PEXPERIMENT pexp);
-void SetCmdLineOptions (PANALYSIS panal);
-void SetOptions (PANALYSIS panal);
-
 
 #endif /* _SIMINIT_H_ */
 
