@@ -2,11 +2,33 @@
 
    Copyright (c) 1993.  Don Maszle, Frederic Bois.  All rights reserved.
 
+   This program is free software; you can redistribute it and/or
+   modify it under the terms of the GNU General Public License
+   as published by the Free Software Foundation; either version 2
+   of the License, or (at your option) any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+
+   contact
+     Frdric Bois / Don Maszle
+     BEHS, School of Public Health
+     University of California at Berkeley
+     Berkeley, CA 94720
+
+     fbois@diana.lbl.gov
+
    -- Revisions -----
-     Logfile:  %F%
-    Revision:  %I%
-        Date:  %G%
-     Modtime:  %U%
+     Logfile:  SCCS/s.simmonte.c
+    Revision:  1.25
+        Date:  28 Jul 1997
+     Modtime:  04:20:23
       Author:  @a
    -- SCCS  ---------
 
@@ -44,6 +66,23 @@ void SetParms (long cParms, HVAR *rghvar, double *rgdParm)
 } /* SetParms */
 
 
+/* -----------------------------------------------------------------------------
+   SetParmsLog
+
+   sets the parameters in the rghvar array to the log-transformed
+   values in the rgdParm array.
+*/
+
+void SetParmsLog (long cParms, HVAR *rghvar, double *rgdParm)
+{
+  long i;
+
+  for (i = 0; i < cParms; i++)
+    SetVar (rghvar[i], log(rgdParm[i]));
+
+} /* SetParmsLog */
+
+
 /* -----------------------------------------------------------------------------
    CalculateOneMCParm
 
@@ -65,11 +104,11 @@ int CalculateOneMCParm (PMCVAR pMCVar)
 
     default:
     case MCV_UNIFORM:
-      pMCVar->dVal = UniformRandom (dMin, dMax);
+      pMCVar->dVal = UniformRandom (dParm1, dParm2);
       break;
 
     case MCV_LOGUNIFORM:
-      pMCVar->dVal = LogUniformRandom (dMin, dMax);
+      pMCVar->dVal = LogUniformRandom (dParm1, dParm2);
       break;
 
     case MCV_BETA:
@@ -97,7 +136,7 @@ int CalculateOneMCParm (PMCVAR pMCVar)
       break;
 
     case MCV_LOGNORMALV:
-      pMCVar->dVal = LogNormalRandom (dParm1, exp(sqrt(log(dParm2))));
+      pMCVar->dVal = LogNormalRandom (dParm1, exp(sqrt(dParm2)));
       break;
 
     case MCV_TRUNCNORMALV:
@@ -105,7 +144,7 @@ int CalculateOneMCParm (PMCVAR pMCVar)
       break;
 
     case MCV_TRUNCLOGNORMALV:
-      pMCVar->dVal = TruncLogNormalRandom (dParm1, exp(sqrt(log(dParm2))),
+      pMCVar->dVal = TruncLogNormalRandom (dParm1, exp(sqrt(dParm2)),
                                            dMin, dMax);
       break;
 
@@ -137,6 +176,10 @@ int CalculateOneMCParm (PMCVAR pMCVar)
       pMCVar->dVal = PoissonRandom (dParm1);
       break;
 
+    case MCV_BINOMIALBETA: /* dMin is in fact beta */
+      pMCVar->dVal = BinomialBetaRandom (dParm1, dParm2, dMin);
+      break;
+
   } /* switch */
 
   return 0;
@@ -188,7 +231,7 @@ BOOL InitSetPoints (PMONTECARLO pMC)
 
   if (!(pfile = fopen(pMC->szSetPointsFilename, "r")))
     ReportError (NULL, RE_CANNOTOPEN | RE_FATAL,
-             pMC->szSetPointsFilename, NULL);
+                 pMC->szSetPointsFilename, NULL);
 
   pMC->pfileSetPoints = pfile;
 
@@ -273,16 +316,19 @@ Exit_ReadSetPoints:
 
 BOOL GetMCMods (PANALYSIS panal, double rgParms[])
 {
+  BOOL bOK;
+
   if (panal->iType == AT_MONTECARLO) { /* Random Monte Carlo mods */
     CalcMCParms (&panal->mc, rgParms, 0); /* start at 0, do them all */
     return TRUE;
   } /* if */
 
   else if (panal->iType == AT_SETPOINTS) {
-    /* take care of Distrib specs */
+    /* read set point mods */
+    bOK = ReadSetPoints (&panal->mc, rgParms);
+    /* eventually override by Distrib specs, FB - 21/07/97 */
     CalcMCParms (&panal->mc, rgParms, panal->mc.nSetParms);
-    /* eventually override with new set point mods */
-    return ReadSetPoints (&panal->mc, rgParms);
+    return bOK;
   } /* else if */
 
   return (FALSE);
@@ -290,4 +336,49 @@ BOOL GetMCMods (PANALYSIS panal, double rgParms[])
 } /* GetMCMods */
 
 
+/* -----------------------------------------------------------------------------
+   SetParents
+
+   FB 21/07/97: For each Monte Carlo variable, if there are parents referenced
+   in hParm for some of the distributionla parameters, pdParms are set to point
+   to the parent's dVals. If there is no parent, pdParms already point to their
+   own dParms and nothing is done.
+
+   The calculation starts at index iStart.
+*/
+
+void SetParents (PMONTECARLO pMC, long iStart)
+{
+  long i, j, k;
+  PMCVAR pMCVar1, pMCVar2;
+  BOOL bFound;
+
+  for (i = iStart; i < pMC->nParms; i++) { /* for each pMCVar considered */
+    pMCVar1 = pMC->rgpMCVar[i];
+    for (j = 0; j < 4; j++) { /* for each of its distrib. param */
+      if (pMCVar1->hParm[j] != 0) { 
+        /* if there is a parent, find it, but parents must appear before 
+           current pMCVar */
+        bFound = FALSE;
+        for (k = 0; k < i; k++) { 
+          pMCVar2 = pMC->rgpMCVar[k];
+          if (pMCVar1->hParm[j] == pMCVar2->hvar) {
+            /* Point to the parent dVal */
+            pMCVar1->pdParm[j] = &(pMCVar2->dVal);
+            bFound = TRUE;
+          }
+        }
+        if (!bFound) { /* oops, parent not found, error */
+          printf ("\n"
+                  "Error: parents must appear in Distrib statements before\n"
+                  "       childrens when creating sampling dependencies - Exiting.\n\n");
+          exit(0);
+        }
+      }
+    }
+  }
+
+} /* SetParents */
+
+
 /* End */
