@@ -5,15 +5,40 @@
 
    Copyright (c) 1993.  Don Maszle, Frederic Bois.  All rights reserved.
 
+   This program is free software; you can redistribute it and/or
+   modify it under the terms of the GNU General Public License
+   as published by the Free Software Foundation; either version 2
+   of the License, or (at your option) any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+
+   contact
+     Frdric Bois / Don Maszle
+     BEHS, School of Public Health
+     University of California at Berkeley
+     Berkeley, CA 94720
+
+     fbois@diana.lbl.gov
+
    -- Revisions -----
-     Logfile:  %F%
-    Revision:  %I%
-        Date:  %G%
-     Modtime:  %U%
+     Logfile:  SCCS/s.simo.c
+    Revision:  1.24
+        Date:  02 Jul 1997
+     Modtime:  08:37:22
       Author:  @a
    -- SCCS  ---------
 
    Output routines for the simulation
+
+   Modified 21 May 1997 by FB: removed OF_PARMLIST and OF_MCRESULT options 
+   handling in WriteMCOutput
 */
 
 #include <math.h>
@@ -53,7 +78,6 @@ void SaveOutputs (PEXPERIMENT pexp, PDOUBLE pdTout)
   memcpy (rgdInterpStates, pmod->pdModelVars,
           pmod->nModelVars*sizeof(double));
 
-  /* if (pexp->dTime > 0.0) FB 17/9/96 */
   /* I guess we have to do that at least for update inputs */
   CalcDeriv(rgdInterpStates, rgdInterpDeriv, pdTout);
 
@@ -74,16 +98,17 @@ void SaveOutputs (PEXPERIMENT pexp, PDOUBLE pdTout)
 
       else { /* Use current parm/input value */
         index = HINDEX(pos->phvar[i]);
-        if (IsInput(pos->phvar[i]) && (vrgInputs[index].iType == IFN_SPIKES)) {          
-          for (j = 0; j < vrgInputs[index].iDoseCur; j++) {
-            if (vrgInputs[index].rgT0s[j] == pexp->dTime) {
-              dTmp = vrgInputs[index].rgMags[j];
-              break;
-            }
-          }
+        j = vrgInputs[index].iDoseCur;
+        if (IsInput(pos->phvar[i]) && 
+            (vrgInputs[index].iType == IFN_SPIKES)) {
+ 
+          /* FB 11/6/97 */
+          if ((vrgInputs[index].rgT0s[j] == pexp->dTime) &&
+              (j < vrgInputs[index].nDoses))
+            dTmp = vrgInputs[index].rgMags[j];
+          else 
+            dTmp = 0;
 
-          if (j == vrgInputs[index].iDoseCur) 
-            dTmp = 0.0;
         }
         else
           dTmp = GetVarValue (pos->phvar[i]);
@@ -224,7 +249,7 @@ void CloseMCFiles (PANALYSIS panal)
    WriteMCOutput
 
    Output the parameters for this run and the results of the
-   simulation.
+   simulation (passed through TransformPred).
 */
 
 void WriteMCOutput (PANALYSIS panal, PMCPREDOUT pmcpredout)
@@ -234,24 +259,15 @@ void WriteMCOutput (PANALYSIS panal, PMCPREDOUT pmcpredout)
 
   pfileMC = pmc->pfileMCOut;
 
-  /* Write parameters, outputs, and the results of the
-     simulations (passed through TransformPred) depending on how the
-     fOptions flags are set in panal.
-  */
-
   fprintf (pfileMC, "%ld\t", panal->mc.lRun);
 
   /* Include parameter values for that run */
-  if (panal->fOptions & OF_PARMLIST) {
-    WriteArray (pfileMC, panal->mc.nParms, panal->mc.rgdParms);
-    fprintf (pfileMC, "\t");
-  }
+  WriteArray (pfileMC, panal->mc.nParms, panal->mc.rgdParms);
+  fprintf (pfileMC, "\t");
 
-  if (panal->fOptions & OF_MCRESULT) {
-    /* write the flattened and eventually transformed predictions */
-    WriteArray (pfileMC, pmcpredout->nbrdy, pmcpredout->pred);
-    fprintf (pfileMC, "\n");
-  }
+  /* write the flattened and eventually transformed predictions */
+  WriteArray (pfileMC, pmcpredout->nbrdy, pmcpredout->pred);
+  fprintf (pfileMC, "\n");
 
   fflush (pfileMC);
 
